package com.aijia.kmt.controller;

import com.aijia.core.web.BaseController;
import com.aijia.core.web.ResponseConstant;
import com.aijia.core.web.ResponseEntity;
import com.aijia.kmt.service.KmtExerciseQuestionListService;
import com.aijia.kmt.service.TokenManager;
import com.aijia.kmt.vo.ExerciseQuestionsRequest;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 *
 * kmt练习册
 * @author chchen
 * 2019-01-14
 */
@RestController
@RequestMapping("/api/train/kmt/exercise")
@Api(value = "kmt练习册题目列表接口", tags = "kmt练习册题目列表接口")
public class KmtExerciseQuestionListController extends BaseController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private TokenManager tokenManager;
    @Autowired
    private KmtExerciseQuestionListService kmtExerciseQuestionListService;

    @RequestMapping(value = "/getHandlingQuestionsV1", method = RequestMethod.POST)
    @ApiOperation(value = "待加工/待审核题目", httpMethod = "POST",  notes = "待加工/待审核题目")
    public ResponseEntity getHandlingQuestions(@RequestParam(name = "accessToken") String accessToken,
                                               @RequestBody ExerciseQuestionsRequest request) {
        long start = System.currentTimeMillis();
        ResponseEntity responseEntity = new ResponseEntity();
        try {
            String userId = tokenManager.getUserId(accessToken);
            responseEntity = kmtExerciseQuestionListService.findQuestionList(userId, request,1);
        } catch (Exception e) {
            logger.error("入参={}", request);
            logger.error("待加工/待审核题目", e);
            responseEntity.failure(ResponseConstant.CODE_500, "查询失败");
        }
        logger.info(accessToken + ":" + (System.currentTimeMillis() - start));
        return responseEntity;
    }

    @RequestMapping(value = "/getSelectHandlingQuestions", method = RequestMethod.POST)
    @ApiOperation(value = "挑选待加工/待审核题目", httpMethod = "POST",  notes = "挑选待加工/待审核题目")
    public ResponseEntity getSelectHandlingQuestions(@RequestParam(name = "accessToken") String accessToken,
                                               @RequestBody ExerciseQuestionsRequest request) {
        long start = System.currentTimeMillis();
        ResponseEntity responseEntity = new ResponseEntity();
        try {
            String userId = tokenManager.getUserId(accessToken);
            int type = request.getSearchType()==1?1:2;
            responseEntity = kmtExerciseQuestionListService.findQuestionList(userId, request,type);
        } catch (Exception e) {
            logger.error("入参={}", request);
            logger.error("待加工/待审核题目", e);
            responseEntity.failure(ResponseConstant.CODE_500, "查询失败");
        }
        logger.info(accessToken + ":" + (System.currentTimeMillis() - start));
        return responseEntity;
    }

}
