package com.aijia.kmt.mapper;

import com.aijia.kmt.dto.*;
import com.aijia.kmt.dto.exercise.*;
import com.aijia.kmt.po.*;
import com.aijia.kmt.po.user.TbLibManager;
import com.aijia.kmt.vo.*;
import com.aijia.kmt.vo.rollbackai.AddExerciseQuestionRequest;
import com.aijia.kmt.vo.rollbackai.ExerciseBooksEditRequest;
import com.aijia.kmt.vo.rollbackai.ExerciseQuestionsEditRequest;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.ibatis.annotations.*;

import java.util.*;

@Mapper
public interface KmtExerciseMapper {

    /**
     * in 查询题目列表
     * @param ids
     * @return
     */
    public List<KmtExerciseQuestion> findKmtExerciseQuestionsByIdIn(List<String> ids);

    /**
     * in 查询题目列表
     * @param ids
     * @return
     */
    public List<KmtExerciseQuestion> findAllKmtExerciseQuestionsByIdIn(List<String> ids);

    /**
     * 删除上一次加工的知识点
     * @param questionIds
     * @return
     */
    public void deleteKmtExerciseQuestionKnowByQuestionIds(@Param("questionIds") List<String> questionIds,@Param("userId") String userId);

    /**
     * 题目列表提交审核
     * @param questionIds
     */
    public void updateKmtExerciseQuestion2Review(List<String> questionIds);

    /**
     * 根据错题id查询题目关联的知识点
     * @param questionId 题目id
     * @return 知识点id列表
     */
    List<String> findWrongQuestionKnowHowDetailIdList(@Param("questionId") String questionId);

    /**
     * 查询待加工审核的题目
     * @param bookCourseId 课时
     * @param dealType 1加工，2审核
     * @param userId 加工员/审核员
     * @param rollbackType 查询允许打回的状态
     * @return
     */
    List<ExerciseQuetionDto> findQuestionByBookCourseIdAndUserId(@Param("bookCourseId") String bookCourseId,
                                                                 @Param("dealType") int dealType,
                                                                 @Param("userId") String userId,
                                                                 @Param("rollbackType") int rollbackType,
                                                                 @Param("exerciseBookId") String exerciseBookId,
                                                                 Page page);
    
    List<ExerciseQuetionDto> findQuestionByBookCourseIdAndUserIdNew(
            @Param("req") ExerciseQuestionsRequest req,
            @Param("userId") String userId,
            Page page,@Param("type") int type,@Param("searchType") int searchType,@Param("roleType") int roleType);

    List<ExerciseQuetionDto> findAllQuestionByGradeDetail(@Param("req") Map<String, Object> req,  Page page);



    /**
     * 查询待加工审核的题目
     * @param bookCourseId 课时
     * @return
     */
    List<ExerciseQuetionDto> findPartQuestionByBookCourseId(@Param("bookCourseId") String bookCourseId,
                                                            @Param("exerciseBookId") String exerciseBookId,
                                                            Page page);

    List<ExerciseQuetionDto> findPartQuestionByBookCourseIdNew(@Param("req") ExerciseQuestionsRequest req,
            Page page);
    
    

    /**
     * 查询课时下的所有题目
     * @param bookCourseId
     * @return
     */
    List<ExerciseQuetionDto> findQuestionByBookCourseId(@Param("bookCourseId") String bookCourseId);

    /**
     * 查询打回列表
     * @param questionIds
     * @param dealType
     * @return
     */
    List<KmtExerciseQuestionRollback> findKmtExerciseQuestionRollbacks(@Param("questionIds") List<String> questionIds,
                                                                       @Param("dealType") int dealType);

    /**
     * 查询题目的打回记录倒序（新用户）
     * @param questionId
     * @return
     */
    List<KmtExerciseQuestionRollBackInfo> findQuestionRollbackRecord(String questionId);



    /**
     * 查询题目的打回记录倒序（老用户）
     * @param questionId
     * @return
     */
    List<ExerciseQuestionRollbacks> findQuestionRollbackRecordOld(String questionId);
    /**
     * 统计错题数和总题数
     * @param vipId 学生id
     * @param subject 科目
     * @param beginDateTime 开始时间
     * @param endDateTime 结束时间
     */
    List<Map<String, Object>> countWrongAndTotal(@Param("vipId") String vipId, @Param("subject") String subject, @Param("beginDateTime") String beginDateTime, @Param("endDateTime") String endDateTime);

    /**
     * 统计平台批注知识点涉及次数
     *
     * @param vipId 学生id
     * @param subject 科目
     * @param beginDateTimeStr 开始时间
     * @param endDateTimeStr 结束时间
     */
    List<Map<String, Object>> countPlatformCorrectionKnowledgeTimes(@Param("vipId") String vipId,
                                                                    @Param("subject") String subject,
                                                                    @Param("beginDateTimeStr") String beginDateTimeStr,
                                                                    @Param("endDateTimeStr") String endDateTimeStr);

    /**
     * 查询校区批注作业相关练习册错题
     *
     * @param vipId 学生id
     * @param subject 科目
     * @param beginDateTimeStr 开始时间
     * @param endDateTimeStr 结束时间
     */
    List<Map<String, Object>> findWrongQuestion(@Param("vipId") String vipId,
                                                @Param("subject") String subject,
                                                @Param("beginDateTimeStr") String beginDateTimeStr,
                                                @Param("endDateTimeStr") String endDateTimeStr);

    /**
     * 根据练习册题目id统计关联知识点
     *
     * @param questionIdList 练习册题目id
     */
    List<Map<String, Object>> countCampusCorrectionKnowledgeTimesByQuestionIds(@Param("questionIdList") List<String> questionIdList);

    /**
     * 统计知识点掌握情况
     * @param vipId 学生id
     * @param subject 科目
     * @param beginDateTimeStr 开始时间
     * @param endDateTimeStr 结束时间
     */
    List<Map<String, Object>> countKnowledgeMastery(@Param("vipId") String vipId, @Param("subject") String subject, @Param("beginDateTimeStr") String beginDateTimeStr, @Param("endDateTimeStr") String endDateTimeStr);

    /**
     * 查询题目的关联知识点（去重）
     * @param questionIds
     * @return
     */
    List<String> findHowDetailIdsByQuestionIds(@Param("questionIds") List<String> questionIds);


    /**
     * 统计平台批注知识点掌握率
     *
     * @param vipId 学生id
     * @param subject 科目
     * @param beginDateTimeStr 开始时间
     * @param endDateTimeStr 结束时间
     *
     * @return 知识点统计情况
     */
    List<Map<String, Object>> countPlatformCorrectionKnowledgeMastery(@Param("vipId") String vipId, @Param("subject") String subject, @Param("beginDateTimeStr") String beginDateTimeStr, @Param("endDateTimeStr") String endDateTimeStr);

    /**
     * 查询作业涉及的练习册题目用于计算知识点掌握率
     *
     * @param vipId 学生id
     * @param subject 科目
     * @param beginDateTimeStr 开始时间
     * @param endDateTimeStr 结束时间
     *
     * @return 题目情况
     */
    List<Map<String, Object>> findWrongQuestion4Mastery(@Param("vipId") String vipId, @Param("subject") String subject,
                                                        @Param("beginDateTimeStr") String beginDateTimeStr,
                                                        @Param("endDateTimeStr") String endDateTimeStr);

    /**
     * 查询练习册题目的知识点
     *
     * @param questionId 练习册题目id
     *
     * @return 知识点
     */
    List<Map<String, Object>> findKnowledgeByQuestionId(@Param("questionId") String questionId);

    /**
     * 查询未解析的课时
     * @param analysisUserId
     * @param maxCount
     * @return
     */
    List<KmtExerciseBookCourse> findUnAnalysisBookCourse(@Param("analysisUserId") String analysisUserId, @Param("maxCount") int maxCount);

    /**
     * 根据练习册id列表查询需要更新到es的数据
     * @param questionIdList 题目id列表
     */
    List<Map<String, Object>> findData2EsByQuestionList(@Param("questionIdList") List<String> questionIdList);

    /**
     * 全量更新es索引kmt_exercise_question_know中的数据
     * @param pageSize 每次查询数据量
     */
    List<Map<String, Object>> findAll2Es(@Param("pageSize") int pageSize);

    /**
     * 全量同步数据到es时，更新加载状态
     * @param kmtExerciseQuestionKnowIdList 需要更新的记录id列表
     */
    void updateLoadStatus(@Param("kmtExerciseQuestionKnowIdList") List<String> kmtExerciseQuestionKnowIdList);

    /**
     * 查询未加工练习册
     * @param handleUserId
     * @return
     */
    List<ExerciseBookDto> findUnHandleBooks(@Param("handleUserId") String handleUserId,
                                            @Param("searchType") int searchType,
                                            @Param("bookName") String bookName);

    /**
     * 查询未分配加工人练习册
     * @return
     */
    List<ExerciseBookDto> findUndistributedHandlerBooks(@Param("subject") String subject,
                                                        @Param("semester") int semester,
                                                        @Param("versionType") int versionType,
                                                        @Param("grade") String grade);

    /**
     * 查询未审核练习册
     * @param auditingUserId
     * @return
     */
    List<ExerciseBookDto> findUnAuditingBooks(@Param("auditingUserId") String auditingUserId,
                                              @Param("searchType") int searchType,
                                              @Param("bookName") String bookName);


    /**
     * 查询未分配审核练习册
     * @return
     */
    List<ExerciseBookDto> findUndistributedAuditerBooks(@Param("subject") String subject,
                                                        @Param("semester") int semester,
                                                        @Param("versionType") int versionType,
                                                        @Param("grade") String grade);

    /**
     * 查询所有联系册
     * @param subject
     * @param semester
     * @param versionType
     * @param grade
     * @return
     */
    List<ExerciseBookDto> findAllExerciseBooks(@Param("subject") String subject,
                                               @Param("semester") int semester,
                                               @Param("versionType") int versionType,
                                               @Param("grade") String grade,
                                               @Param("searchType") int searchType,
                                               @Param("bookName") String bookName);

    /**
     * 查询待加工审核的单元课时
     * @param bookId
     * @param dealType
     * @return
     */
    List<ExerciseUnitBookcourseDto> findUnHandleExerciseBookCourse(@Param("bookId") String bookId,
                                                                   @Param("dealType") int dealType,
                                                                   @Param("questionId") String questionId,
                                                                   @Param("userId") String userId);
    
    /**
     * 查询待加工审核的单元课时New
     * @param bookId
     * @param dealType
     * @return
     */
    List<ExerciseUnitBookcourseDto> findUnHandleExerciseBookCourseNew(@Param("bookId") String bookId,
                                                                   @Param("dealType") int dealType,
                                                                   @Param("questionId") String questionId,
                                                                   @Param("userId") String userId);

    /**
     * 查询不同的单元课时
     * @param exerciseBookId
     * @return
     */
    List<ExerciseUnitBookcourseDto> findPartExerciseBookCourse(@Param("exerciseBookId") String exerciseBookId);


    /**
     * 查询解析未加工未审核的单元课时  kmt_exercise_book_course, kmt_exercise_questions, kmt_exercise_book_unit
     * @param bookId
     * @param dealType
     * @return
     */
    List<ExerciseUnitBookcourseDto> findAnalysisUnHandleExerciseBookCourse(@Param("bookId") String bookId,
                                                                           @Param("dealType") int dealType);

    /**
     * 查询练习册下的所有课时
     * @param bookId
     * @return
     */
    List<ExerciseUnitBookcourseDto> findAllExerciseBookCourse(@Param("bookId") String bookId,
                                                              @Param("exerciseBookId") String exerciseBookId);

    /**
     * 查询练习册下的所有课时
     * @param exerciseBookId
     * @return
     */
    List<ExerciseUnitBookcourseDto> findAllExerciseBookCourseByExerciseBookId(@Param("exerciseBookId") String exerciseBookId);

    
    /**
     * 查询练习册下的所有课时
     * @param exerciseBookId
     * @return
     */
    List<ExerciseUnitBookcourseDto> findAllExerciseBookCourseByExerciseBookIdNew(
    		@Param("bookId") String bookId,
    		@Param("exerciseBookId") String exerciseBookId,
    		@Param("isDiff") Integer isDiff,@Param("type") Integer type);

    /**
     * 查询未加工/审核的题目数量
     * @param bookCourseId
     * @param ids
     * @param dealType
     * @return
     */
    Integer findUnHandleQuestionCount(@Param("bookCourseId") String bookCourseId,
                                      @Param("ids") List<String> ids,
                                      @Param("dealType") int dealType);

    /**
     * 查询未加工/审核的课时数量
     * @param bookCourseId
     * @param unitId
     * @param dealType
     * @return
     */
    Integer findUnHandleBookCourseCount(@Param("unitId") String unitId,
                                        @Param("bookCourseId") String bookCourseId,
                                        @Param("dealType") int dealType);

    /**
     * 查询未加工/审核的单元数量
     * @param bookId
     * @param unitId
     * @param dealType
     * @return
     */
    Integer findUnHandleUnitCount(@Param("bookId") String bookId,
                                  @Param("unitId") String unitId,
                                  @Param("dealType") int dealType);

    /**
     * 查询搜索的题目, kmt_exercise_questions, kmt_sys_user
     * @param questionId
     * @return
     */
    List<ExerciseQuetionDto> findKmtQuestionsById(@Param("questionId") String questionId,
                                                  @Param("userId") String userId,
                                                  @Param("dealType") int dealType,
                                                  Page page);

    /**
     * 按知识点分页查询题目 kmt_exercise_questions, kmt_sys_user, kmt_exercise_question_know
     * @param howDetailsId
     * @param howDetailsKdId
     * @param userId
     * @param dealType
     * @param page
     * @return
     */
    List<ExerciseQuetionDto> findKmtQuestionByDetailId(@Param("howDetailsId") String howDetailsId,
                                                       @Param("howDetailsKdId") String howDetailsKdId,
                                                       @Param("userId") String userId,
                                                       @Param("dealType") int dealType,
                                                       @Param("userIdList") List<String> userIdList,
                                                       Page page);

    /**
     * 按年级、科目、版本、上下册分页查询题目 kmt_exercise_questions, kmt_sys_user, kmt_exercise_question_know,zyb_know_how_details_sort
     * @param grade
     * @param courseName
     * @param semester
     * @param versionType
     * @param userId
     * @param dealType
     * @param page
     * @return
     */
    List<ExerciseQuetionDto> findKmtQuestionByGradeSubjectVersionSemester(@Param("grade") String grade,
                                                                          @Param("courseName") String courseName,
                                                                          @Param("semester") int semester,
                                                                          @Param("type") int versionType,
                                                                          @Param("userId") String userId,
                                                                          @Param("dealType") int dealType,
                                                                          @Param("userIdList") List<String> userIdList,
                                                                          Page page);

    /**
     * 查询所有kmt题目  kmt_exercise_questions, kmt_sys_user,
     * @param userId
     * @param dealType
     * @param page
     * @return
     */
    List<ExerciseQuetionDto> findAllKmtQuestions(@Param("userId") String userId,
                                                 @Param("dealType") int dealType,
                                                 @Param("userIdList") List<String> userIdList,
                                                 Page page);

    /**
     * 查询小题 kmt_exercise_questions, kmt_sys_user
     * @param parentIds
     * @return
     */
    List<ExerciseQuetionDto> findSmallQuestionByParentIds(@Param("parentIds") List<String> parentIds);

    /**
     * 查询大题 kmt_exercise_questions
     * @param id
     * @return
     */
    ExerciseQuetionDto findBigQuestionById(@Param("id") String id);

    /**
     * 查询题目的知识点列表 kmt_exercise_question_know, zyb_know_how_details_sort
     * @param questionIds
     * @return
     */
    List<QuestionListKnowModel> findQuestionKnowByQuestionIds(@Param("questionIds") List<String> questionIds,@Param("userId") String userId,@Param("roleType") int roleType);


    /**
     * 查询题目的知识点列表 kmt_quality_question_know, zyb_know_how_details_sort
     * @param questionIds
     * @return
     */
    List<QuestionListKnowModel> findKmtQuestionKnowByQuestionIds(@Param("questionIds") List<String> questionIds);


    /**
     * 查询父题目
     * @param questionId
     * @return
     */
    KmtExerciseQuestion findPrentQuestionByChildQuestionId(@Param("questionId") String questionId);



    /**
     * 根据给定知识点id列表和知识点版本，统计练习册题目数量
     * @param knowledgeIdList 知识点id列表
     * @param version 知识点版本
     * @return 题目数量
     */
    int countQuestionByKnowledgeAndVersion(@Param("knowledgeIdList") List<String> knowledgeIdList, @Param("version") int version);

    /**
     * 更新练习册为未加工 kmt_exercise_book
     * @param id
     */
    void updateKmtExerciseBookNotHandle(@Param("id") String id);

    /**
     * 更新练习册为未审核 kmt_exercise_book
     * @param id
     */
    void updateKmtExerciseBookNotAuditing(@Param("id") String id);

    /**
     * 更新练习册为解析未加工  kmt_exercise_book
     * @param id
     */
    void updateKmtExerciseBookAnalysisNotHandle(@Param("id") String id);

    /**
     * 更新练习册为解析未审核 kmt_exercise_book
     * @param id
     */
    void updateKmtExerciseBookAnalysisNotAuditing(@Param("id") String id);

    /**
     * 更新练习册课时为未加工 kmt_exercise_book_course
     * @param id
     */
    void updateKmtExerciseBookCourseNotHandle(@Param("id") String id);

    /**
     * 更新练习册课时为未审核 kmt_exercise_book_course
     * @param id
     */
    void updateKmtExerciseBookCourseNotAuditing(@Param("id") String id);

    /**
     * 查询解析未加工的练习册 kmt_exercise_book  kmt_exercise_questions
     * @param
     * @return
     */
    List<ExerciseBookDto> findUnAnalysisHandleBooks(@Param("analysisUserId") String analysisUserId,
                                                    @Param("searchType") int searchType,
                                                    @Param("bookName") String bookName);

    /**
     * 查询解析未审核的练习册 kmt_exercise_book  kmt_exercise_questions
     * @param
     * @return
     */
    List<ExerciseBookDto> findUnAnalysisAuditingBooks(@Param("analysisAuditerId") String analysisAuditerId,
                                                      @Param("searchType") int searchType,
                                                      @Param("bookName") String bookName);

    /**
     * 查询校对的练习册
     * @param userId
     * @param bookName
     * @return
     */
    List<ExerciseBookDto> findUnCorrectAnswerBooks(@Param("userId") String userId,
                                                   @Param("searchType") int searchType,
                                                   @Param("bookName") String bookName);


    /**
     * 查询校对的练习册
     * @param userId
     * @param bookName
     * @return
     */
    List<ExerciseBookDto> findCorrectedAnswerBooks(@Param("userId") String userId,
                                                   @Param("searchType") int searchType,
                                                   @Param("bookName") String bookName);

    /**
     * 查询练习册列表
     * @param ids
     * @return
     */
    List<ExerciseBookDto> findExerciseBookListByIds(@Param("ids") List<String> ids);


    /**
     * 查询所有练习册
     * @param page
     * @param params
     * @return
     */
    List<String> getBookId(Page page, @Param("map") Map<String, Object> params);

    /**
     * 查询练习册
     * @param id
     * @return
     */
    KmtExerciseBook findExerciseBookById(@Param("bookId") String id);

    /**
     * 查询练习册
     * @return
     */
    KmtExerciseBook findExerciseBookByBookIdAndYearAndType(@Param("bookId") String bookId,
                                                           @Param("bookYear") int bookYear,
                                                           @Param("bookType") int bookType);

    /**
     * 查询单元课时记录
     * @param bookIds
     * @return
     */
    List<KmtExerciseBookCourse> findByBookIds(@Param("bookIds") List<String> bookIds);

    /**
     * 更新kmt题目的es加载状态
     * @param kmtExerciseQuestionKnowIdList 题目id列表
     * @param newLoaded 新的es加载状态
     * @return 更新行数
     */
    int updateExerciseQuestionKnowEsLoadStatus(@Param("kmtExerciseQuestionKnowIdList") List<String> kmtExerciseQuestionKnowIdList,
                                               @Param("newLoaded") int newLoaded);

    /**
     * 分页查询所有练习册
     *
     *
     * @param selectedExerciseBookIdList 练习册id list
     * @param offset 从第？条开始
     * @param pageSize 获取？条记录
     *
     * @return 练习册list
     */
    List<KmtExerciseBook> findAllExerciseBookPage(@Param("selectedExerciseBookIdList") List<String> selectedExerciseBookIdList,
                                                  @Param("offset") int offset, @Param("pageSize") int pageSize);

    /**
     * 查询最新的打回理由记录
     * @param questionId
     * @return
     */
    KmtExerciseQuestionRollback findLastExerciseRollbackByQuestionId(@Param("questionId") String questionId);

    /**
     * 分页查询练习册
     * @param page
     * @param params
     * @return
     */
    List<Exercise1Dto> findExerciseList(Page page, @Param("map") Map<String, Object> params);

    /**
     * 分页查询练习册
     * @param page
     * @param params
     * @return
     */
    List<Exercise1Dto> findTeacherExerciseList(Page page, @Param("map") Map<String, Object> params);

    /**
     * 分页查询机构
     * @param page
     * @param params
     * @return
     */
    List<TbLibJoin> findLibList(Page page, @Param("map") Map<String, Object> params);

    /**
     * 删除练习册状态统计数据
     * @param exerciseBookIdSet 练习册id
     * @return 删除数量
     */
    int deleteKmtExerciseBookStatusCountByExerciseBookIds(@Param("exerciseBookIdSet") Set<String> exerciseBookIdSet);

    /**
     * 查询练习册统计
     * @return 练习册状态统计数据列表
     */
    List<ExerciseBookCountDTO> findExerciseBookCount();

    /**
     * 查询单科练习册列表
     * @param page 分页
     * @param req 入参
     * @return 单科练习册列表分页结果
     */
    @Select("select distinct t1.exercise_book_id book_id, t1.book_name, t1.book_year,t1.course_name subject,t2.grade,t2.semester,t2.book_type from kmt_exercise_book_status_count t1 left join kmt_exercise_book t2 on t1.exercise_book_id=t2.id where t1.data_status = 1 and t1.status = #{req.status} and t1.course_name = #{req.subject} and t2.grade is not null")
    Page<ExerciseBookCountSubjectDetailDTO> findExerciseBookCountSubjectDetail(IPage<ExerciseBookCountSubjectDetailDTO> page,
                                                                               @Param("req") ExerciseBookCountSubjectDetailReq req);
    /**
     * 查询所有的馆
     */
    List<Map<String,Object>> findAllLib(Page page, @Param("subject") String subject);

    /**
     * 查询馆里所有的练习册
     */
    List<KmtExerciseBookLibDto> findAllExerciseBook(@Param("libId") String libId, @Param("subject") String subject);

    /**
     * 查询所有的馆
     */
    List<Map<String,Object>> findAllExportLib(@Param("subject") String subject);

    /**
     * 查询大标题以及大标题下的所有小题 kmt_exercise_questions
     * @param parentIds
     * @return
     */
    List<KmtExerciseQuestion> findQuestionByParentId(@Param("parentIds") List<String> parentIds);


    /**
     * 根据题目id查询新老用户
     */
    List<KmtExerciseQuestionRollback> findKmtExerciseQuestionRollbackListByQuestionId(@Param("questionId") String questionId);

    TbLibManager findByUserIdAndTypeIn(@Param("userId") String userId);

    /**
     * 查询签约数
     */
    List<ExercisePriorityDto> findLibSignCount();

    /**
     * 更新练习册涉及到的机构签约数
     */
    void updateKmtExerciseBookByExerciseBookId(@Param("exerciseRelevancyNum") int exerciseRelevancyNum, @Param("exerciseBooId") String exerciseBookId);

    /**
     * 查询所有的有签约数的练习册
     */
     List<KmtExerciseBook> findAllExerciseBookBySignNum();

    /**
     * 更新优先级
     */
    void updateKmtExerciseBookById(@Param("priority") String priority, @Param("id") String id);

    /**
     * 更新优先级
     */
    void updateKmtExerciseBookByPriority(@Param("totalGradePriority") String totalGradePriority, @Param("id") String id);

    /**
     * 查询练习册的所有题目
     * @param exerciseBookId
     * @return
     */
    List<KmtExerciseQuestion> findExerciseQuestionByExerciseBookId(@Param("exerciseBookId") String exerciseBookId);

    /**
     * 查询题目列表
     * @param ids
     * @return
     */
    List<KmtExerciseQuestion> findExerciseQuestionByIds(@Param("ids") List<String> ids);

    /**
     * 查询答案报错练习册列表
     *
     * @param page 分页
     * @param req 入参
     */
    IPage<AnswerProofreadExerciseBookDTO> findAnswerProofreadExerciseBookList(IPage<AnswerProofreadExerciseBookDTO> page,
                                                                              @Param("req") AnswerProofreadExerciseBookListReq req);

    /**
     * 查询答案校对后需要修改答案的题目(即答案错误的题目)
     * @param page 分页
     * @param req 入参
     */
    IPage<ExerciseQuetionDto> findQuestionListAnswerWrong(IPage<ExerciseQuetionDto> page,
                                                          @Param("req") AnswerProofreadQuestionListReq req);

    /**
     * 查询打回的题目
     *
     * @param req 入参
     * @param dealStatusList 题目状态
     *
     * @return 题目列表
     */
    List<ExerciseQuetionDto> findRepulsedQuestion(@Param("req") ExerciseTeacherReviseQuestionReq req,
                                                  @Param("dealStatusList") List<Integer> dealStatusList);

    /**
     *
     * @param
     * @return
     */
    List<Map<String,Object>> findUnproofreadExerciseId();

    List<KmtExerciseQuestion> findExerciseStatusByBookId(@Param("bookId") String bookId);

    void updateExerciseBookStatusByIdAndStatus(@Param("proofreadStatus") Integer status, @Param("id") String id);

    /**
     * 更新练习册的校对状态
     * @param type
     * @param exerciseBookId
     */
    void updateExerciseBookProofreadResult(@Param("type") int type,
                                           @Param("exerciseBookId") String exerciseBookId);

    /**
     * 统计答案错误的题目
     *
     * @param exerciseBookId 练习册id
     * @param excludeQuestionIdSet 排除这些题目
     */
    int countAnswerWrongQuestion(@Param("exerciseBookId") String exerciseBookId, @Param("excludeQuestionIdSet") Set<String> excludeQuestionIdSet);

    /**
     * 更新练习册校对状态
     *
     * @param exerciseBookId 练习册id
     * @param newProofread 新校对状态
     */
    @Update("update kmt_exercise_book set proofread_status = #{newProofread} where id = #{exerciseBookId}")
    int updateKmtExerciseBookProofreadStatus(@Param("exerciseBookId") String exerciseBookId, @Param("newProofread") int newProofread);

    Map<String, Object> teacherAnswerProofreadDetail(@Param("userId") String userId);

    /**
     * 注销后更新校对中的老师状态
     */
    @Update("update kmt_exercise_book set answer_correcter_id=null where answer_correcter_id=#{answerCorrecterId} and answer_corrected_status=0")
    int updateKmtExerciseBookAnswerCorrecterId(@Param("answerCorrecterId") String answerCorrecterId);

    /**
     * 查询练习册单元课时列表2
     *
     * @param request 入参
     *
     * @return 单元课时列表
     */
    List<ExerciseUnitBookcourseDto> findUnitAndPeriodList(@Param("request") UnitAndPeriodListRequest request);

    /**
     * 查询题目的报错记录
     * @param questionIds
     * @param beType 被报错人的角色类型
     * @return
     */
    List<KmtExerciseQuestionErrors> findKmtExerciseQuestionErrorsByQuestionIds(@Param("questionIds") List<String> questionIds,
                                                                               @Param("beType") int beType);

    /**
     * 查询报错明细， kmt_exercise_question_errors
     * @param userId
     * @param page
     * @return
     */
    List<KmtExerciseQuestionErrors> findKmtExerciseQuestionByErrorUserId (@Param("userId") String userId,
                                                                          @Param("exerciseBookId") String exerciseBookId,
                                                                          Page page);

    /**
     * 计算练习册老师的角色下的错误题数
     * @param userId
     * @param exerciseBookId
     * @param type
     * @return
     */
    int countQuestionErrorCountByExerciseBookId(@Param("userId") String userId, @Param("exerciseBookId") String exerciseBookId,
                                                @Param("type") int type);

    /**
     * 找出重复的练习册
     *
     * @return
     */
    List<KmtExerciseBook> findSameExerciseBookList();

    /**
     * 计算未加工数量
     * @param exerciseBookId
     * @param type
     * @return
     */
    int countUnFinishQuestionsByExerciseBookId(@Param("exerciseBookId") String exerciseBookId,
                                               @Param("type") int type);

    /**
     * 统计练习册分配情况
     */
    ExerciseDistributionCountDTO findDistributionCount();

    /**
     * 待分配/已分配练习册列表
     *
     * @param req 入参
     */
    List<DistributionExerciseBookDTO> findDistributionExerciseBookList(@Param("req") DistributionExerciseBookListReq req,
                                                                       @Param("offset") int offset,
                                                                       @Param("pageSize") int pageSize);

    /**
     * 统计待分配练习册数量
     *
     * @param req 入参
     */
    int countDistributionExerciseBook(@Param("req") DistributionExerciseBookListReq req);

    /**
     * 更新kmt_exercise_book校对老师
     * @param teacherId
     * @param exerciseBookIds
     */
    void updateAnswerCorrecterIdByIds(@Param("teacherId") String teacherId, @Param("exerciseBookIds") List<String> exerciseBookIds);
    /**
     * 更新kmt_exercise_question校对老师
     * @param teacherId
     * @param exerciseBookIds
     */
    void updateAnswerCorrecterIdByExerciseBookIds(@Param("teacherId") String teacherId, @Param("exerciseBookIds") List<String> exerciseBookIds);

    /**
     * 查询需要导出的已分配练习册列表
     *
     * @param req 入参
     * @param offset 查询？条
     */
    List<ExportDistributionExerciseBookDTO> findExportDistributionExerciseBookList(@Param("req") ExportDistributionExerciseBookListReq req, @Param("offset") int offset);

    /**
     * 查询题目处理人（加工、加审、解析、解审）信息
     *
     * @param exerciseBookId 练习册id
     * @param userId 处理人userId
     * @param authorityType 处理人类型
     */
    DistributionTeacherDTO findDistributionTeacher(@Param("exerciseBookId") String exerciseBookId,
                                                   @Param("userId") String userId,
                                                   @Param("authorityType") int authorityType);

    /**
     * 查询未校对的题目数量
     * @param exerciseBookId
     * @return
     */
    int countNotCheckAnswerCount(@Param("exerciseBookId") String exerciseBookId);

    /**
     * 计算练习册下proofread = 0的数量
     * @param exerciseBookId
     * @return
     */
    int countAllNotCheckAnswerCount(@Param("exerciseBookId") String exerciseBookId);

    /**
     * 查询练习册的所有记录
     * @param bookId
     * @return
     */
    List<KmtExerciseBook> findExerciseBooksByBookId(@Param("bookId") String bookId);

    /**
     * 查询分配老师的记录
     * @return
     */
    List<KmtExerciseBookTeacherRecord> findExerciseTeacherByExerciseBookIdsAndAuthType(@Param("exerciseBookIds") List<String> exerciseBookIds,
                                                                                       @Param("authType") int authType);

    /**
     * 查询book_count记录
     * @param exerciseBookId
     * @param type
     * @return
     */
    KmtExerciseBookCount findBookCountByExerciseBookIdAndType(@Param("exerciseBookId") String exerciseBookId,
                                                              @Param("type") int type);

    /**
     * 更新正确率
     * @param bookCountId
     * @param accuracy
     */
    void updateKmtExerciseBookCountById(@Param("bookCountId") String bookCountId,
                                        @Param("accuracy") int accuracy);

    /**
     * 查询练习册已分配导出数据
     *
     * @param exerciseBookId 练习册id
     *
     * @return 练习册已分配导出数据
     */
    @Select("select * from kmt_exercise_book_distribution_count where exercise_book_id = #{exerciseBookId}")
    KmtExerciseBookDistributionCount findKmtExerciseBookDistributionCountByExerciseBookId(@Param("exerciseBookId") String exerciseBookId);

    /**
     * 统计单元和课时的数量
     *
     * @param exerciseBookId 练习册id
     *
     * @return 单元和课时的数量map
     */
    @Select("select count(*) courseCount, count(distinct unit_id) unitCount from kmt_exercise_book_course where exercise_book_id = #{exerciseBookId}")
    Map<String, Long> countUnitAndCourse(@Param("exerciseBookId") String exerciseBookId);

    /**
     * 查询各加工身份最新提交时间
     *
     * @param exerciseBookId 练习册id
     *
     * @return 最新提交时间map
     */
    @Select("select max(handler_submit_time) handlerLastSubmitTime,\n" +
            "       max(publish_time) handleAuditUserLastSubmitTime,\n" +
            "       max(analysis_handler_submit_time) analyserLastSubmitTime,\n" +
            "       max(analysis_submit_time) analysisAuditUserLastSubmitTime\n" +
            "from kmt_exercise_questions\n" +
            "where exercise_book_id = #{exerciseBookId}")
    Map<String, Date> findLastSubmitTime(@Param("exerciseBookId") String exerciseBookId);

    /**
     * 查询各加工身份正确率
     *
     * @param exerciseBookId 练习册id
     *
     * @return 正确率map
     */
    @Select("select type, accuracy from kmt_exercise_book_count where exercise_book_id = #{exerciseBookId} and type between 1 and 3")
    List<Map<String, Integer>> findAccuracy(@Param("exerciseBookId") String exerciseBookId);

    /**
     * 查询练习册各状态统计
     *
     * @param exerciseBookId 练习册id
     * @param status 状态 com.ssj.service.kmt.exercisesys.enums.ExerciseBookStatusEnum
     *
     * @return 练习册状态数据
     */
    @Select("select * from kmt_exercise_book_status_count where exercise_book_id = #{exerciseBookId} and status = #{status}")
    KmtExerciseBookStatusCount findKmtExerciseBookStatusCountByExerciseBookIdAndStatus(@Param("exerciseBookId") String exerciseBookId,
                                                                                       @Param("status") int status);

    /**
     * 查询练习册加工情况
     *
     * @param exerciseBookId 练习册
     * @param type 身份类型(1练习册加工，2练习册审核，3练习册解析，4练习册解审，5KMT加工，6KMT审核)
     *
     * @return 练习册加工情况
     */
    @Select("select * from kmt_exercise_book_count where exercise_book_id = #{exerciseBookId} and type = #{type}")
    KmtExerciseBookCount findKmtExerciseBookCountByExerciseBookIdAndType(@Param("exerciseBookId") String exerciseBookId,
                                                                         @Param("type") int type);

    /**
     * 查找可更新的练习册数据
     * @return
     */
    @Select("select t1.id,t2.press,t2.print_year,t2.lib_name from kmt_exercise_book t1\n" +
            "                         join kmt_exercise_book_modify t2 on\n" +
            "            t1.book_name=t2.book_name\n" +
            "        and t1.course_name=t2.course_name\n" +
            "        and t1.grade=t2.grade\n" +
            "        and t1.version_name =t2.version_name\n" +
            "        and t1.semester=t2.semester\n" +
            "        and t1.book_type=t2.book_type\n" +
            "        and t1.book_year=t2.book_year\n" +
            "order by t2.id\n" +
            "limit 5000")
    List<Map<String, String>> findUpdateExerciseBookList();
    /**
     * 查询所有有签约数的不同的馆
     * @param subject
     * @return
     */
     List<Map<String,Object>> findAllDistinctLibBySubject(@Param("page") Page page, @Param("subjct") String subject);

//    /**
//     * 通过馆查询对应的签约数
//     */
//     Merchant findSignService(@Param("libId") String libId);

    @Update("update kmt_exercise_book set had_handle=#{book.hadHandle}, had_auditing=#{book.hadAuditing}, " +
            "analysis_handled=#{book.analysisHandled}, analysis_audited=#{book.analysisAudited},progress=#{book.progress}  where id=#{book.id}")
    int updateKmtExerciseBookHandleStatus(@Param("book") KmtExerciseBook book);

    @Update("update kmt_exercise_questions set deal_status=#{question.dealStatus}," +
            "analysis_submit_time=#{question.analysisSubmitTime}, handler_submit_time=#{question.handlerSubmitTime}, " +
            "proofread_status=#{question.proofreadStatus}," +
            " question_type=#{question.questionType},difficulty_level=#{question.difficultyLevel}," +
            "real_exam_type=#{question.realExamType},training_skill=#{question.trainingSkill}, " +
            "publish_time=#{question.publishTime}, analysis_handler_submit_time=#{question.analysisHandlerSubmitTime}\n" +
            " where id=#{question.id}")
    int updateKmtExerciseQuestions(@Param("question") KmtExerciseQuestion question);

    @Delete("delete kmt_exercise_questions where question_id=#{questionId}")
    int deleteKmtExerciseQuestionKnowsByQuestionId(@Param("questionId") String questionId);

    @Insert("insert into kmt_exercise_question_know (id, question_id, question_parent_id, how_details_id, " +
            "how_details_kd_id, create_time, how_detail_sort_id, how_detail_kd_sort_id, version) " +
            " values (#{questionKnow.id}, #{questionKnow.questionId},  #{questionKnow.questionParentId}, " +
            "#{questionKnow.howDetailsId}, #{questionKnow.howDetailsKdId}, #{questionKnow.createTime}, " +
            "#{questionKnow.howDetailSortId}, #{questionKnow.howDetailKdSortId}, #{questionKnow.version})")
    int insertKmtExerciseQuestionKnow(@Param("questionKnow") KmtExerciseQuestionKnow questionKnow);

    /**
     * 根据题目id查询练习册
     * @param questionId
     * @return
     */
    List<ExerciseBookDto> findExerciseBookByQuestionId(@Param("questionId") String questionId,@Param("roleType") int roleType,@Param("subject") String subject);

    /**
     * 根据题目id查询单元课时
     * @param questionId
     * @return
     */
    List<ExerciseUnitBookcourseDto> findExerciseBookCourseByQuestionId(@Param("questionId") String questionId);
    
    /**
     * 根据题目id查询单元课时new
     * @param questionId
     * @return
     */
    List<ExerciseUnitBookcourseDto> findExerciseBookCourseByQuestionIdNew(@Param("questionId") String questionId);

    /**
     * 查询练习册系统的统一格式题目
     * @param questionId
     * @param page
     * @return
     */
    List<ExerciseQuetionDto> findExerciseQuestionByQuestionId (@Param("questionId") String questionId, Page page);


    /**
     * 查询练习册系统的统一格式题目
     * @param questionId
     * @return
     */
    List<ExerciseQuetionDto> findExerciseQuestionByQuestionIdAndBookId (@Param("questionId") String questionId,
                                                                        @Param("bookId") String bookId,@Param("userId") String userId);

    /**
     * 查询题目以及题目的小题
     * @param questionId
     * @return
     */
    @Select("select id, num from kmt_exercise_questions where id=#{questionId} or parent_id=#{questionId} order by num")
    List<String> findQuestionIdByParentId(@Param("questionId") String questionId);

    /**
     * 查询题目以及题目的小题
     * @param questionId
     * @return
     */
    @Select("select * from kmt_exercise_questions where id=#{questionId} or parent_id=#{questionId} order by num")
    List<KmtExerciseQuestion> findQuByParentId (@Param("questionId")String questionId);

    @Update("update kmt_exercise_questions set parent_id=#{parentId}, question_class_type=#{questionClassType}" +
            " where id=#{id}")
    int updateKmtExerciseQuestionClassTypeAndParentId(@Param("parentId") String parentId,
                                                      @Param("questionClassType") int questionClassType,
                                                      @Param("id") String id);

    @Update("update kmt_exercise_questions set big_question_index=#{req.bigQuestionIndex}, small_question_name=#{req.smallQuestionIndex}, image=#{questionImage}, " +
            " answer_image=#{answerImage}, deal_status = #{dealStatus}" +
            " where id=#{req.questionId}")
    int updateQuestionEditData(@Param("req") AddExerciseQuestionRequest req,
                               @Param("questionImage") String questionImage,
                               @Param("answerImage") String answerImage,
                               @Param("dealStatus") int dealStatus);

    @Update("update kmt_exercise_questions set small_question_name=#{req.smallQuestionIndex}, image=#{questionImage}, " +
            " answer_image=#{answerImage}, deal_status = #{dealStatus}" +
            " where id=#{req.questionId}")
    int updateSmallQuestionEditData(@Param("req") AddExerciseQuestionRequest req,
                               @Param("questionImage") String questionImage,
                               @Param("answerImage") String answerImage,
                               @Param("dealStatus") int dealStatus);

    @Select("select * from kmt_exercise_questions where book_id=#{bookId} and num >= #{num} order by num limit 1")
    KmtExerciseQuestion findNextByBookIdAndNum(@Param("bookId") String bookId,
                                               @Param("num") int num);

    @Select("select * from kmt_exercise_questions where book_id=#{bookId} and num <= #{num} order by num desc limit 1")
    KmtExerciseQuestion findPreviousByBookIdAndNumOld(@Param("bookId") String bookId,
                                                   @Param("num") int num);

    @Select("SELECT	t.*, c.course_name AS course, "
    		+ "	c.unit_name, "
    		+ "	c.number, "
    		+ " c.page_index as pate_index_new "
    		+ " FROM"
    		+ " q_question_workbook c "
    		+ " LEFT JOIN kmt_exercise_questions t ON t.id = c.question_id"
    		+ " AND t.exercise_book_id = c.sub_book_id"
    		+ " WHERE"
    		+ "	c.book_id = #{bookId}"
    		+ " AND c.number <= #{num}"
    		+ " ORDER BY"
    		+ "	c.number DESC LIMIT 1")    
    KmtExerciseQuestion findPreviousByBookIdAndNum(@Param("bookId") String bookId,
                                                   @Param("num") int num);
    


    @Update("update kmt_exercise_questions set num = num + 1 where book_id=#{bookId} and num >= #{num}")
    int updateQuestionNumByBookIdAndNum(@Param("bookId") String bookId,
                                        @Param("num") int num);

    @Update("update kmt_exercise_questions set parent_id=#{parentId}, question_class_type=#{questionClassType} where book_id=#{bookId} and num = #{num}")
    int updateParentIdAndClassType(@Param("parentId") String parentId,
                                   @Param("questionClassType") int questionClassType,
                                   @Param("bookId") String bookId,
                                   @Param("num") int num);

    @Select("select big_question_index from kmt_exercise_questions where book_course_id=#{bookCourseId} " +
            " and big_question_index=#{bigQuestionIndex} and deal_status !=6 limit 1")
    String findBigQuestionIndexByBookCourseId(@Param("bookCourseId") String bookCourseId,
                                              @Param("bigQuestionIndex") String bigQuestionIndex);

    @Select("select * from kmt_exercise_questions where book_course_id=#{bookCourseId} " +
            " and big_question_index=#{bigQuestionIndex} ")
    List<KmtExerciseQuestion> findQuestionsByBookCourseIdAndBigIndex(@Param("bookCourseId") String bookCourseId,
                                                                     @Param("bigQuestionIndex") String bigQuestionIndex);

    @Update("update kmt_exercise_questions set num = #{updateNum} where book_id=#{bookId} and num = #{num}")
    int updateByBookIdAndNum(@Param("bookId") String bookId,
                             @Param("updateNum") int updateNum,
                             @Param("num") int num);

    @Update("update kmt_exercise_questions set num = #{updateNum} where id=#{questionId}")
    int updateQuestionNumByQuestionId(@Param("questionId") String questionId,
                                      @Param("updateNum") int updateNum);

    int updateQuestionMinusNumByQuestionIds(@Param("questionIds") List<String> questionIds,
                                            @Param("size") int size);

    int updateQuestionAddNumByQuestionIds(@Param("questionIds") List<String> questionIds,
                                          @Param("size") int size);

    @Select("select * from kmt_exercise_questions where book_course_id=#{bookCourseId}")
    List<KmtExerciseQuestion> findByBookCourseId(@Param("bookCourseId") String bookCourseId);

    @Select("select * from kmt_exercise_questions where book_id=#{bookId} limit 2000")
    List<KmtExerciseQuestion> findByBookId(@Param("bookId") String bookId);

    @Update("update kmt_exercise_questions set big_question_index=#{bigQuestionIndex} where parent_id=#{parentId}")
    int updateBigQuestionIndexByParentId(@Param("bigQuestionIndex") String bigQuestionIndex,
                                         @Param("parentId") String parentId);

    List<ExerciseBookEditDto> findEditExerciseBooks(@Param("request") ExerciseBooksEditRequest request);


    List<ExerciseQuestionEditDto> findEditExerciseQuestions (@Param("request") ExerciseQuestionsEditRequest request,
                                                             Page page);

    @Select("select eb.id from kmt_exercise_book eb, q_question_workbook qw where qw.sub_book_id=eb.id" +
            " and qw.question_id=#{questionId} and eb.book_type=3 limit 1")
    String findDifferentExerciseBookId(@Param("questionId") String questionId);

    @Select("select id from kmt_exercise_book where id=#{exerciseBookId} and book_type = 3 limit 1")
    String findDifferentExerciseBookIdByBookId(@Param("exerciseBookId") String exerciseBookId);

    @Update("update kmt_exercise_questions set parent_id=#{parentId} where id=#{id}")
    int updateParentIdById (@Param("parentId") String parentId,
                            @Param("id") String id);

    @Update("update kmt_exercise_questions set parent_id=#{parentId}, question_class_type=#{questionClassType}," +
            " num = #{num} where id=#{id}")
    int updateParentIdAndQuestionClassTypeAndNumById (@Param("parentId") String parentId,
                                                      @Param("questionClassType") int questionClassType,
                                                      @Param("num") int num,
                                                      @Param("id") String id);

    @Update("update kmt_exercise_book set had_auditing = 0 where id=#{exerciseBookId}")
    int updateExerciseBookNotAuditing (@Param("exerciseBookId") String exerciseBookId);

    List<ExerciseBookEditDto> findErrorExerciseBooks(@Param("bookName") String bookName,
                                                     @Param("userId") String userId);

    @Update("update tr_train_user_exercise set concise_id=#{startX}, analysis_submit_type=#{startY}, update_type=#{endX}," +
            " es_loaded=#{endY}, enumerate=#{enumerate} where question_id=#{questionId} and user_id = #{userId}")
    int updateQuestionStartPointAndEndPoint (@Param("startX") int startX,
                                             @Param("startY") int startY,
                                             @Param("endX") int endX,
                                             @Param("endY") int endY,
                                             @Param("questionId") String questionId,
                                             @Param("enumerate") int enumerate,@Param("userId") String userId);

    List<ExerciseQuestionNumberDto> findExerciseQuestionNum (@Param("exerciseBookIds") List<String> exerciseBookIds);

    @Select("select image from kmt_exercise_questions where id=#{id}")
    String imagePath(@Param("id") String id);

    int deleteKnowByQuestionId (@Param("questionIds") List<String> questionIds);


    @Select("select id from kmt_exercise_book where create_time > #{beginDate} and part_flag !=3")
    List<String> findNearKmtExerciseBookIds (@Param("beginDate") String beginDate);

    Integer findIsNotFinishTask (@Param("userId") String userId);

    Map<String, Integer> findMaxSmallQuestionNumber (@Param("parentId") String parentId);

    @Select("select distinct book_id from kmt_exercise_book where part_flag != 3 order by book_id")
    List<String> findAllBookId (Page page);

    List<QuestionListKnowModel> findQuestionKnowByQuestionIds2(@Param("questionIds")List<String> copyList);
}
