package com.aijia.kmt.mapper;

import com.aijia.kmt.dto.papercreate.KnowledgePoint;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;

@Mapper
public interface KmtPaperCreateMapper{

    @Select("select how_details_kd_id point_id, how_id, alias point_name from zyb_know_how_details_sort where stat=3 and lib_id='SSJ' and grade=#{grade} " +
            "and course_name=#{subject} and type=#{versionType}")
    List<KnowledgePoint> getKnowledgePoint(@Param("grade") String grade, @Param("subject") String subject, @Param("versionType") int versionType);

    @Select({
            "<script>",
            "select distinct detail.how_id from zyb_know_how_details_sort detail, zyb_know_how_details de," ,
            "(select how_id, num, create_time from zyb_know_how_details_sort where course_name=#{subject} and type=#{versionType} and grade =#{grade}" ,
            "and semester = 1 and lib_id = 'SSJ' and stat =1) how",
            "where detail.how_id=how.how_id and detail.how_details_id=de.id and detail.stat=2 and detail.lib_id = 'SSJ'",
            "and detail.course_name=#{subject} and detail.type=#{versionType} and detail.grade=#{grade} and detail.semester = 1 ",
            "and de.is_zt = 0 order by how.num, how.create_time limit #{num}",
            "</script>"
    })
    List<String> findHowID(@Param("grade") String grade,@Param("subject") String subject,@Param("num") int num,@Param("versionType") int versionType);

    @Select({
            "select distinct how_id from zyb_know_how_details_sort where stat =1 and course_name=#{subject} and type=#{versionType} and grade =#{grade}",
            "and semester = 1 and lib_id = 'SSJ' and alias in ('词语', '句子', '口语交际', '课外阅读')"
    })
    List<String> findChineseHowId(@Param("grade") String grade,@Param("subject") String subject,@Param("versionType") int versionType);

    @Select({
            "<script>",
            "select kd.how_details_kd_id point_id, kd.alias point_name from zyb_know_how_details_sort kd ",
            " left join zyb_know_how_details_sort how on how.stat=1 and kd.how_id=how.how_id and how.course_name=#{subject}",
            " and how.type=#{versionType} and how.grade=#{grade} and how.semester=1",
            " left join zyb_know_how_details_sort detail on kd.how_details_id=detail.how_details_id and detail.stat=2",
                    "    and detail.course_name=#{subject} and detail.type=#{versionType} and detail.grade=#{grade} and detail.semester=1",
            " where kd.how_id in ",
            "<foreach collection='howIds' item='howid' open='(' separator=',' close=')'>",
            "#{howid}",
            "</foreach>",
            " and kd.stat=3 and kd.course_name=#{subject} and kd.type=#{versionType} and kd.grade =#{grade} ",
            " and kd.semester = 1 and kd.lib_id = 'SSJ'",
            " order by how.num, how.create_time, detail.num, kd.num",
            "</script>"
    })
    List<KnowledgePoint> findKmtPaperKnowledge(@Param("grade") String grade,@Param("subject") String subject,@Param("num") int num,@Param("versionType") int versionType,@Param("howIds") List<String> howIds);


    @Select("select how_id,alias from zyb_know_how_details_sort where stat=1 and how_id=#{howId} and lib_id='SSJ' and grade=#{grade} and course_name=#{subject} and type=#{versionType}")
    String getHowID(@Param("howId") String howId,@Param("grade") String grade, @Param("subject") String subject, @Param("versionType") int versionType);

    @Select({
            "<script>",
            " select distinct book_id from kmt_exercise_book where grade=#{grade}",
            " and version=#{versionType} and course_name=#{courseName} and semester = 1 ",
            " and ",
            "<foreach collection='bookNames' item='bookName' open='(' separator=' or ' close=')'>",
            " book_name like concat('%',#{bookName},'%')",
            "</foreach>",
            "</script>"
    })
    List<String> findDefaultBookId (@Param("courseName") String courseName,
                                    @Param("versionType") int versionType,
                                    @Param("grade") String grade,
                                    @Param("semester") int semester,
                                    @Param("bookNames") List<String> bookNames);

    @Select("select distinct detail.how_details_id from zyb_know_how_details_sort detail, zyb_know_how_details de," +
            " (select how_id from zyb_know_how_details_sort where course_name=#{courseName} and type=#{versionType} and grade =#{grade}" +
            "  and semester = #{semester} and lib_id = 'SSJ' and stat =1 order by num, create_time limit #{count}) how " +
            " where detail.how_id=how.how_id and detail.how_details_id=de.id and detail.stat=2 and detail.lib_id = 'SSJ' " +
            " and detail.course_name=#{courseName} and detail.type= #{versionType} and detail.grade=#{grade}" +
            " and detail.semester = #{semester} and de.is_zt = 0")
    List<String> findCommonDefaultHowDetailIds (@Param("courseName") String courseName,
                                                @Param("versionType") int versionType,
                                                @Param("grade") String grade,
                                                @Param("semester") int semester,
                                                @Param("count") int count);

    @Select("select distinct detail.how_details_id from zyb_know_how_details_sort detail,\n" +
            "              (select how_id\n" +
            "               from zyb_know_how_details_sort how\n" +
            "               where stat = 1\n" +
            "                 and course_name =#{courseName}\n" +
            "                 and type = #{versionType}\n" +
            "                 and grade = #{grade}\n" +
            "                 and semester = #{semester}\n" +
            "                 and lib_id = 'SSJ'\n" +
            "                 and alias in ('词语', '句子', '口语交际','课外阅读')\n" +
            "              ) how\n" +
            "              where detail.how_id=how.how_id\n" +
            "              and  detail.stat=2\n" +
            "                and detail.course_name =#{courseName}\n" +
            "                and detail.type = #{versionType}\n" +
            "                and detail.grade = #{grade}\n" +
            "                and detail.semester = #{semester}\n" +
            "                and detail.lib_id = 'SSJ'")
    List<String> findZtDefaultHowDetailIds (@Param("courseName") String courseName,
                                            @Param("versionType") int versionType,
                                            @Param("grade") String grade,
                                            @Param("semester") int semester);
}
