package com.aijia.kmt.mapper;

import com.aijia.kmt.dto.ExerciseTeacherDTO;
import com.aijia.kmt.dto.ExerciseTeacherDetailDTO;
import com.aijia.kmt.dto.FindCheckTeacherListDto;
import com.aijia.kmt.dto.TeacherInfoStatusDetailDTO;
import com.aijia.kmt.po.user.TbLibManager;
import com.aijia.kmt.po.user.TbLibManagerAudit;
import com.aijia.kmt.vo.ExerciseTeacherDetailReq;
import com.aijia.kmt.vo.ExerciseTeacherListReq;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;

import java.util.List;
import java.util.Map;

@Mapper
public interface LibManagerMapper extends BaseMapper<TbLibManager> {

    @Select("select * from tb_lib_manager  where user_id = #{userId} and type = #{type} and state = 1 and is_cancelled=0 limit 1")
    TbLibManager findByUserIdAndType(@Param("userId") String userId,
                                     @Param("type") int type);

    @Select("select * from tb_lib_manager where name regexp #{name} and type=#{type} and state=1")
    List<TbLibManager> findByNameAndType(@Param("name") String name,
                                         @Param("type") int type);

    @Select({
            "<script>",
            "select * from tb_lib_manager where user_id=#{userId} and type in ",
            "<foreach collection='types' item='type' open='(' separator=',' close=')'>",
            "#{type}",
            "</foreach>",
            " and state = #{state} and is_cancelled=0 limit 1",
            "</script>"
    })
    TbLibManager findTopByUserIdAndTypeInAndState(
            @Param("userId") String userId, @Param("types")int[] types, int state);

    @Select({
            "<script>",
            "select * from tb_lib_manager where user_id=#{userId} and type in ",
            "<foreach collection='types' item='type' open='(' separator=',' close=')'>",
            "#{type}",
            "</foreach>",
            "and state = 1 limit 1",
            "</script>"
    })
    TbLibManager findTopByUserIdAndTypeIn(
            @Param("userId") String userId, @Param("types")int[] types);

    @Select("select * from tb_lib_manager where user_id=#{teacherId} and type=#{type} and state=1 and is_cancelled=0")
    List<TbLibManager> findAllByUserIdAndTypeAndStatus(@Param("teacherId") String teacherId,
                                                       @Param("type") int type);

    /**
     * 查询指定角色的用户信息（所有）
     * @param userId
     * @param types
     * @return
     */

    @Select({
            "<script>",
            "select * from tb_lib_manager where user_id=#{userId} and type in ",
            "<foreach collection='types' item='type' open='(' separator=',' close=')'>",
            "#{type}",
            "</foreach>",
            "order by create_time desc",
            "</script>"
    })
    List<TbLibManager> findAllByUserIdAndTypeIn(@Param("userId") String userId, @Param("types")int[] types);

    @Select({
            "<script>",
            "select * from tb_lib_manager where user_id in ",
            "<foreach collection='userIds' item='userId' open='(' separator=',' close=')'>",
            "#{userId}",
            "</foreach>",
            " and type in ",
            "<foreach collection='types' item='type' open='(' separator=',' close=')'>",
            "#{type}",
            "</foreach>",
            " and state = 1 and is_cancelled=0",
            "</script>"
    })
    List<TbLibManager> findTopByUserIdsInAndTypeIn(
            @Param("userId") List<String> userIds,
            @Param("types") int[] types);

    /**
     * 查询用户信息 tb_lib_manager
     * @param userIds
     * @param types
     * @return
     */

    List<TbLibManager> findTbLibManagerByUserIdsAndType(@Param("userIds") List<String> userIds,
                                                        @Param("types") List<Integer> types);

    /**
     * 查找相同名字的老师
     * @param name
     * @return
     */
    String findExerciseTeacherByName(@Param("name") String name);

    /**
     * 分页查询审核记录
     *
     * @param name 名称
     */
    List<TbLibManagerAudit> findPageByName(@Param("page") Page page, @Param("name") String name,
                                           @Param("subject") String subject, @Param("authority") String authority);

    /**
     * 查询指定角色的所有用户信息,区分注销
     *
     * @param type
     * @param userId
     * @return
     */
    @Select("select * from tb_lib_manager where user_id=#{userId} and type=#{type} and is_cancelled=0 order by create_time desc limit 1")
    TbLibManager findByTypeAndUserIdAndCancelled(@Param("userId") String userId,
                                                 @Param("type") int type);

    @Update("update tb_lib_manager set state=2 where user_id=#{userId} and type=#{type} and is_cancelled=#{cancelled} and state=1")
    void updateByUserIdAndTypeAndCancelled(String userId, int type, int cancelled);

    List<Map<String, Object>> findAllExreciseTeacherInfo(@Param("map") Map map, Page page);

    @Select("select * from tb_lib_manager where type=#{type} and state=#{state}  and create_time  is not null order by create_time desc limit 1")
    TbLibManager findExerciseByTypeAndState(int type,int state);

    /**
     * 查询符合条件的老师
     */
    List<TbLibManager> findTbLibManagersByCreateTime(@Param("page") Page page,@Param("req") ExerciseTeacherListReq req);

    Map<String,Object> findKmtTeacherLists(@Param("userId") String userId);

    /**
     * 分页查询kmt老师列表, tb_lib_manager,
     * @param subject 科目
     * @param authority 老师类型
     * @param type 数据类型:1练习册老师汇总数据，2kmt老师汇总数据，3练习册老师下练习册汇总数据，4KMT老师下知识点汇总数据，5题库录入老师汇总数据，6题库录入老师下知识点汇总数据
     * @param teacherName 老师名称
     * @param page 分页
     * @return
     */
    List<ExerciseTeacherDTO> findKmtTeacherList(@Param("subject") String subject,
                                                @Param("authority") String authority,
                                                @Param("type") int type,
                                                @Param("teacherName") String teacherName,
                                                Page page);

    @Update("update tb_lib_manager set is_online=0,logout_time = now() where user_id=#{teacherId} and state=1")
    int updateTeacherOnlineStatusByTeacherid(String teacherId);

    /**
     * 分页查询练习册老师详情
     * @param page 分页
     * @param req 入参
     * @return 练习册老师详情
     */
    IPage<ExerciseTeacherDetailDTO> findExerciseTeacherDetails(IPage<ExerciseTeacherDetailDTO> page,
                                                               @Param("req") ExerciseTeacherDetailReq req);

    /**
     * 查询老师信息状态详情
     * @param teacherId 老师id
     * @return 老师信息状态详情
     */
    TeacherInfoStatusDetailDTO findTeacherStatusDetail(@Param("teacherId") String teacherId);


    /**
     * 查询已校对和未校对总本数
     */
    Map<String,Object> findCheckTeacherDetails(@Param("teacherId")String teacherId,@Param("maxCreateTime")String maxCreateTime);

    List<FindCheckTeacherListDto>  findCheckTeacherList(@Param("teacherName") String teacherName,
                                                        @Param("authorityType") int authorityType,
                                                        @Param("subjects") List<String> subjects,
                                                        Page page);

    List<Map<String,Object>> findCheckTeacherDetailsList(@Param("teacherId") String teacherId,@Param("maxCreateTime") String maxCreateTime, Page page);

    /**
     * 随机获取以为题目审核老师
     * @param subject
     * @return
     */
    TbLibManager findRandManagerByTypeAndSubject(@Param("subject") String subject);


    int findUserManagerNormal (@Param("userId") String userId, @Param("authority") int authority);

}
