package com.aijia.kmt.service;

import com.aijia.core.web.ResponseEntity;
import com.aijia.kmt.dto.*;
import com.aijia.kmt.dto.exercise.ExerciseQuestionRollbacks;
import com.aijia.kmt.dto.exercise.KmtExerciseBookLibDto;
import com.aijia.kmt.dto.exercise.KmtExerciseQuestionRollBackInfo;
import com.aijia.kmt.po.KmtExerciseQuestion;
import com.aijia.kmt.po.KmtExerciseQuestionRollback;
import com.aijia.kmt.vo.*;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author chchen
 * 类描述:练习册mybatis接口
 * 创建时间:2019/08/08 13:54
 */
public interface KmtExerciseMapperService {
    /**
     * in 查询题目列表
     * @param ids
     * @return
     */
    public List<KmtExerciseQuestion> findKmtExerciseQuestionsByIdIn(List<String> ids);

    /**
     * in 查询所有练习册相关的题目列表
     * @param ids
     * @return
     */
    public List<KmtExerciseQuestion> findAllKmtExerciseQuestionsByIdIn(List<String> ids);

    /**
     * 删除上一次加工的知识点
     * @param questionIds
     * @return
     */
    public void deleteKmtExerciseQuestionKnowByQuestionIds(List<String> questionIds);

    /**
     * 题目列表提交审核
     * @param questionIds
     */
    public void updateKmtExerciseQuestion2Review(List<String> questionIds);


    /**
     * 查询整道错题关联的知识点，如果传入的题目id是一道小题，则返回这道大题所有小题关联的知识点，否则返回对应题目的知识点
     * @param questionId 题目id
     * @return 知识点id的列表
     */
    List<String> findWrongQuestionKnowHowDetailIdList(String questionId);


    /**
     * 查询待加工审核的题目
     * @param userId 加工员/审核员
     * @return
     */
    List<ExerciseQuetionDto> findQuestionByBookCourseIdAndUserId (ExerciseQuestionsRequest request, Page page, String userId);

    /**
     * 查询打回列表
     * @param questionIds
     * @param dealType
     * @return
     */
    List<KmtExerciseQuestionRollback> findKmtExerciseQuestionRollbacks(List<String> questionIds, int dealType);

    /**
     * 查询题目的打回记录(新用户)
     * @param questionId
     * @return
     */
    List<KmtExerciseQuestionRollBackInfo> findQuestionRollbackRecord(String questionId);

    /**
     * 查询题目的打回记录（老用户）
     * @param questionId
     * @return
     */
    List<ExerciseQuestionRollbacks> findQuestionRollbackRecordOld(String questionId);

    /**
     * 获取知识点占比列表
     * @param vipId 学生id
     * @param subject 科目
     * @param beginDateTime 开始时间
     * @param endDateTime 结束时间
     * @return 知识点占比列表
     */
    List<KnowledgePointProportionDTO> getKnowledgePointProportionList(String vipId, String subject, Date beginDateTime, Date endDateTime);
    /**
     * 根据练习册id列表查询需要更新到es的数据
     * @param questionIdList 题目id列表
     */
    List<Map<String, Object>> findData2EsByQuestionList(List<String> questionIdList);

    /**
     * 全量更新es索引kmt_exercise_question_know中的数据
     * @return
     */
    long updateTotal2Es();

    /**
     * 查询练习册
     * @param request
     * @param userId
     * @return
     */
    List<ExerciseBookDto> findExerciseBooks(ExerciseConditionRequest request, String userId);

    /**
     * 查询练习册的单元课时
     * @param request
     * @return
     */
    List<ExerciseBookUnitDto> findExerciseBookDetail(ExerciseBookIdDealTypeRequest request, String userId,int type);

    /**
     * 查询kmt的题目 kmt_exercise_questions
     * @param request
     * @param userId
     * @return
     */
    Map<String, Object> getKmtHandlingQuestions(KmtHandlingQuestionRequest request, String userId);

    /**
     * 拼接大题目
     * @param questions
     * @return
     */
    List<ExerciseQuetionDto> findAndAppendBigQuestion2List(List<ExerciseQuetionDto> questions);

    /**
     * 根据给定知识点id列表和知识点版本，统计练习册题目数量
     * @param knowledgeIdList 知识点id列表
     * @param version 知识点版本
     * @return 题目数量
     */
    int countQuestionByKnowledgeAndVersion(List<String> knowledgeIdList, int version);

    /**
     * es字段转成实体字段
     * @param list
     * @return
     */
    List<ExerciseQuetionDto> parseMapToExerciseQuestionList(List<Map> list);

    /**
     * 查询所有大小题题目
     * @param parentIds
     * @return
     */
    ResponseEntity findQuestionListByParentId(List<String> parentIds);

    /**
     * 查询练习册单元课时列表2
     *
     * @param request 入参
     *
     * @return 单元课时列表
     */
    List<ExerciseBookUnitDto> findUnitAndPeriodList(UnitAndPeriodListRequest request);

    /**
     * 查询题目的正确格式
     * @return
     */
    List<ExerciseBookCourseClassDto> findClassifyQuestionsByIds(List<String> questionIds);


    /**
     * 查询机构识别率相关计算
     */
    List<KmtExerciseBookLibDto> findOrgCount(String libId, String subject);
}
