package com.aijia.kmt.service.impl;

import com.aijia.kmt.common.taskthread.AijiaThreadPool;
import com.aijia.kmt.es.service.ExerciseQuestionSearchService;
import com.aijia.kmt.es.service.base.BaseSearchService;
import com.aijia.kmt.mapper.KmtExerciseMapper;
import com.aijia.kmt.mapper.KmtQualityQuestionsMapper;
import com.aijia.kmt.po.KmtQualityQuestions;
import com.aijia.kmt.service.DevelopService;
import com.aijia.kmt.task.SyncSaveExerciseBookTask;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Service
public class DevelopServiceImpl implements DevelopService {
    private final Logger logger = LoggerFactory.getLogger(DevelopServiceImpl.class);

    @Autowired
    private KmtExerciseMapper kmtExerciseMapper;
    @Autowired
    private ExerciseQuestionSearchService exerciseQuestionSearchService;
    @Autowired
    private KmtQualityQuestionsMapper kmtQualityQuestionsMapper;
    @Autowired
    private BaseSearchService baseSearchService;

    @Override
    public void initExerciseQuestionAndKnowEs() {
        int pageNo = 1, pageSize = 500;
        List<String> bookIds = null;

        while (true){
            Page page = new Page(pageNo, pageSize);
            bookIds = kmtExerciseMapper.findAllBookId(page);
            if (bookIds != null){
                bookIds.forEach(bookId -> {
                    try{
                        AijiaThreadPool.getInstance().submit(new SyncSaveExerciseBookTask(kmtExerciseMapper, exerciseQuestionSearchService, bookId));
                        TimeUnit.SECONDS.sleep(3);
                    } catch (Exception e){
                        logger.error("同步异常", e);
                    }
                });
            }
            if (bookIds == null || bookIds.size() < pageSize){
                break;
            } else {
                pageNo += 1;
            }
        }
    }

    @Override
    public void initOneExerciseQuestionAndKnowEs(String bookId){
        AijiaThreadPool.getInstance().submit(new SyncSaveExerciseBookTask(kmtExerciseMapper,
                exerciseQuestionSearchService, bookId));
    }

    @Override
    public void initKmtQualityQuestions(){
        int pageNo = 1, pageSize = 100;
        List<String> questionIds = null;
        while (true){
            Page page = new Page(pageNo, pageSize);
            questionIds = kmtQualityQuestionsMapper.findPageQuestionIds(page);
            if (questionIds != null) {
                List<KmtQualityQuestions> kmtExerciseQuestions = kmtQualityQuestionsMapper.findQuestionByParentId(questionIds);
                if (kmtExerciseQuestions != null) {
                    kmtExerciseQuestions.forEach(keq -> {
                        exerciseQuestionSearchService.saveImmediate(JSON.parseObject(JSON.toJSONString(keq), new TypeReference<Map<String, Object>>() {
                        }));
                    });
                    List<Map<String, Object>> dataMapList2Es = kmtQualityQuestionsMapper.findData2EsByQuestionList(questionIds);
                    if (CollectionUtils.isNotEmpty(dataMapList2Es)) {
                        baseSearchService.insertBatch("kmt_exercise_question_know", dataMapList2Es, true);
                    }
                }
            }
            if (questionIds == null || questionIds.size() < pageSize){
                break;
            } else {
                pageNo += 1;
            }
        }
    }
}
