package com.aijia.kmt.service.impl;

import com.aijia.kmt.common.constant.KnowledgeVersion;
import com.aijia.kmt.dto.BookDTO;
import com.aijia.kmt.dto.ExerciseBookBaseDto;
import com.aijia.kmt.dto.ExerciseBookCourseDTO;
import com.aijia.kmt.dto.ExerciseBookDto;
import com.aijia.kmt.mapper.KmtExerciseBookCourseMapper;
import com.aijia.kmt.mapper.KmtExerciseQueryMapper;
import com.aijia.kmt.po.KmtExerciseBookCourse;
import com.aijia.kmt.service.KmtExerciseBookCourseService;
import com.aijia.kmt.vo.ExerciseConditionRequest;
import com.aijia.kmt.vo.QuestionDistributeRequest;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

@Service
public class KmtExerciseBookCourseServiceImpl implements KmtExerciseBookCourseService {
    @Autowired
    private KmtExerciseBookCourseMapper kmtExerciseBookCourseMapper;

    @Autowired
    private KmtExerciseQueryMapper kmtExerciseQueryMapper;

    @Autowired
    private KnowledgeVersion knowledgeVersion;

//    @Autowired
//    private KmtExerciseMapper kmtExerciseMapper;

    /**
     *  循环插入练习册到结构列表中
     * @param list
     * @return
     */
    private List<ExerciseBookDto> findBookList (List<KmtExerciseBookCourse> list){
        List<ExerciseBookDto> result = new ArrayList<>();
        if (list != null) {
            ExerciseBookDto book = null;
            for (KmtExerciseBookCourse course : list) {
                book = this.isBookExist(result, course.getBookId());
                if (book == null){
                    String bookName = course.getBookName();
                    book = new ExerciseBookDto(course.getBookId(), bookName);
                    if (StringUtils.isNotBlank(course.getGrade())){
                        //如果有年级-科目等信息，组装一个描述信息 年级-上下册-版本-科目
                        book.setSemester(course.getSemester() == 0 ? 1 : course.getSemester());
                        book.setGrade(course.getGrade());
                        book.setSubject(course.getCourseName());
                        String versionStr = null;
                        if (StringUtils.isNotBlank(course.getCourseName())){
                            versionStr = knowledgeVersion.getVersionNameBySubjectAndVersionNum(course.getCourseName(), course.getVersion());
                            bookName += '(' + versionStr.substring(0, 1) + ')';
                            book.setName(bookName);
                        }
                        ExerciseBookBaseDto version = new ExerciseBookBaseDto(course.getVersion().toString(), versionStr);
                        book.setVersion(version);
                    }
                    result.add(0, book);
                }
                book.addUnit(course.getUnitId(), course.getUnit(), course.getId(), course.getPeriod());
            }
        }
        return result;
    }

    /**
     * 练习册是否已经存在
     * @param list
     * @param bookId
     * @return
     */
    private ExerciseBookDto isBookExist (List<ExerciseBookDto> list, String bookId){
        ExerciseBookDto result = null;
        for (ExerciseBookDto book : list){
            if (book.getId().equals(bookId)){
                result = book;
                break;
            }
        }
        return result;
    }


    @Override
    public List<ExerciseBookDto> getExerciseCondition(ExerciseConditionRequest request, String userId) {
        return null;
    }

    @Override
    public String getExerciseBookByBookId(String bookId) {
        KmtExerciseBookCourse kmtExerciseBookCourse = kmtExerciseBookCourseMapper.findOneByBookId(bookId);
//        KmtExerciseBookCourse kmtExerciseBookCourse = dao.findOneByBookId(bookId);
        String result = "";
        if (kmtExerciseBookCourse != null){
            result = kmtExerciseBookCourse.getBookName();
        }
        return result;
    }

    @Override
    public List<ExerciseBookCourseDTO> getBookExerciseList(QuestionDistributeRequest request) {
        return null;
    }


    @Override
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    public List<KmtExerciseBookCourse> findByUnitId(String unitId) {
        return kmtExerciseBookCourseMapper.findByUnitId(unitId);
    }

    @Override
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    public List<KmtExerciseBookCourse> findByBookId(String bookId) {
        return kmtExerciseBookCourseMapper.findByBookId(bookId);
    }

    @Override
    public Page<BookDTO> findBookPage(String grade, String subject, String bookName, Pageable pageable) {
        Page<BookDTO> bookDTOPage = kmtExerciseQueryMapper.findBookPage(grade, subject, bookName, pageable);

        List<BookDTO> bookDTOList = bookDTOPage.getContent();
        if (CollectionUtils.isNotEmpty(bookDTOList)) {
            KnowledgeVersion knowledgeVersion = new KnowledgeVersion();
            knowledgeVersion.init();

            //根据版本编号和科目设置版本名称
            bookDTOList.forEach(bookDTO -> bookDTO.setVersionName(knowledgeVersion.getVersionNameBySubjectAndVersionNum(bookDTO.getSubject(), bookDTO.getVersion())));
        }

        return bookDTOPage;
    }

    @Override
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    public int findVersionTypeByBookId(String bookId) {
        List<KmtExerciseBookCourse> bookCourses = this.findByBookId(bookId);
        int bookVersion = 2;
        if (bookCourses != null && bookCourses.size() > 0){
            bookVersion = bookCourses.get(0).getVersion();
        }
        return bookVersion;
    }

    @Override
    public List<KmtExerciseBookCourse> getAllBookCourse(String subject, String grade, int semester, int versionType) {
        return kmtExerciseQueryMapper.getAllBookCourse(subject, grade, semester, versionType);
    }

    @Override
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    public KmtExerciseBookCourse findBookDataByBookId(String bookId) {
        List<KmtExerciseBookCourse> bookCourses = this.findByBookId(bookId);
        KmtExerciseBookCourse bookCourse = null;
        if (bookCourses != null && bookCourses.size() > 0){
            bookCourse = bookCourses.get(0);
        }
        return bookCourse;
    }

    @Override
    public List<KmtExerciseBookCourse> findBookDataByBookIds(List<String> bookIds) {
        return kmtExerciseQueryMapper.findBookDataByBookIds(bookIds);
    }

    @Override
    public String findCourseNameByQuestionId(String questionId) {
        return kmtExerciseBookCourseMapper.findCourseNameByQuestionId(questionId);
    }

    @Override
    public List<KmtExerciseBookCourse> findByBookId(List<String> bookIds) {
        return kmtExerciseBookCourseMapper.findByBookIdIn(bookIds);
    }
}