package com.aijia.kmt.service.impl;

import com.aijia.core.security.exception.CommonException;
import com.aijia.core.utils.StringUtil;
import com.aijia.core.web.ResponseConstant;
import com.aijia.core.web.ResponseEntity;
import com.aijia.kmt.common.constant.KnowledgeVersion;
import com.aijia.kmt.common.taskthread.AijiaThreadPool;
import com.aijia.kmt.dto.exercise.ExerciseBookEditDto;
import com.aijia.kmt.dto.exercise.ExerciseQuestionAnswerValue;
import com.aijia.kmt.dto.exercise.ExerciseQuestionEditDto;
import com.aijia.kmt.dto.exercise.QuestionWorkbookNumberSize;
import com.aijia.kmt.es.service.ExerciseQuestionSearchService;
import com.aijia.kmt.mapper.*;
import com.aijia.kmt.po.KmtExerciseBook;
import com.aijia.kmt.po.KmtExerciseQuestion;
import com.aijia.kmt.po.QuestionWorkbook;
import com.aijia.kmt.po.base.IdWorker;
import com.aijia.kmt.po.user.TbLibManager;
import com.aijia.kmt.service.KmtExerciseQuestionDealService;
import com.aijia.kmt.task.SyncSaveExerciseBookCourseTask;
import com.aijia.kmt.task.SyncSaveExerciseBookTask;
import com.aijia.kmt.task.SyncSaveExerciseQuestionsTask;
import com.aijia.kmt.task.SyncUpdateQuestionWorkbookNumber;
import com.aijia.kmt.utils.*;
import com.aijia.kmt.vo.QuestionNumMarkRemoveVo;
import com.aijia.kmt.vo.rollbackai.AddExerciseQuestionRequest;
import com.aijia.kmt.vo.rollbackai.ExerciseBooksEditRequest;
import com.aijia.kmt.vo.rollbackai.ExerciseQuestionDealRequest;
import com.aijia.kmt.vo.rollbackai.ExerciseQuestionsEditRequest;
import com.aijia.questioninfo.service.KmtExerciseQuestionInfoService;
import com.aijia.train.bean.TrTrainUserExercise;
import com.aijia.train.mapper.TrainUserExerciseMapper;
import com.aijia.utils.DateUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.stream.MemoryCacheImageInputStream;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

@Service
public class KmtExerciseQuestionDealServiceImpl implements KmtExerciseQuestionDealService {

    protected Logger logger = LoggerFactory.getLogger(this.getClass());
    @Autowired
    private KmtExerciseMapper kmtExerciseMapper;
    @Autowired
    private ExerciseQuestionSearchService exerciseQuestionSearchService;
    @Autowired
    private KnowledgeVersion knowledgeVersion;
    @Autowired
    private KmtExerciseBookMapper kmtExerciseBookMapper;
    @Autowired
    private KmtExerciseQuestionMapper kmtExerciseQuestionMapper;
    @Autowired
    private LibManagerMapper libManagerMapper;

    @Autowired
    private QuestionWorkbookMapper questionWorkbookMapper;
    @Autowired
    private KmtExerciseQuestionInfoService questionInfoService;

    @Autowired
    private TrainUserExerciseMapper userExerciseMapper;

    @Override
    public ResponseEntity addOrEditExerciseQuestionOld(String userId, AddExerciseQuestionRequest request) {
        ResponseEntity responseEntity = new ResponseEntity();
        if (isManagerSubmit(userId)){
            String questionImage = getImagePath(request.getQuestion());
            String answerImage = getImagePath(request.getAnswer());

            if (Objects.equals(1, request.getType()) || StringUtils.isBlank(request.getQuestionId())){
                //题目id不存在则是新增, 前端传过来的是新增题目的上一题
//                KmtExerciseBook book  = kmtExerciseBookMapper.findById(request.getExerciseBookId());
                KmtExerciseBook book  = kmtExerciseBookMapper.selectById(request.getExerciseBookId());
                KmtExerciseQuestion previousQuestion;
                if (StringUtil.isBlank(request.getQuestionId())) {
                    previousQuestion = kmtExerciseMapper.findPreviousByBookIdAndNum(book.getBookId(), request.getNum());
                } else {
                    previousQuestion = kmtExerciseQuestionMapper.selectById(request.getQuestionId());
                }
                KmtExerciseQuestion newQuestion;
                //新增小题
                if (request.getQuestionClassType() == 2){
                    newQuestion = saveQuestion(book, request, previousQuestion, questionImage, answerImage);
                    saveJSONFile(newQuestion, request, questionImage);
                    AijiaThreadPool.getInstance().submit(new SyncSaveExerciseBookTask(kmtExerciseMapper, exerciseQuestionSearchService, book.getBookId()));
                } else {
                    //新增大题, 如果已存在当前大题号，则报错
                    String bqi = kmtExerciseMapper.findBigQuestionIndexByBookCourseId(previousQuestion.getBookCourseId(), request.getBigQuestionIndex());
                    if (StringUtil.isBlank(bqi)){
                        newQuestion = saveQuestion(book, request, previousQuestion, questionImage, answerImage);
                        saveJSONFile(newQuestion, request, questionImage);
                        AijiaThreadPool.getInstance().submit(new SyncSaveExerciseBookTask(kmtExerciseMapper, exerciseQuestionSearchService, book.getBookId()));
                    } else {
                        return responseEntity.failure(ResponseConstant.CODE_000, "当前课时已存在该大题号，请在大题号下添加小题");
                    }
                }

                //更新练习册为审核中
                kmtExerciseMapper.updateExerciseBookNotAuditing(book.getId());

            } else {
                KmtExerciseQuestion kmtExerciseQuestion = kmtExerciseQuestionMapper.selectById(request.getQuestionId());
                int dealStatus = kmtExerciseQuestion.getDealStatus();
                if (kmtExerciseQuestion.getDealStatus() == 2){
                    dealStatus = 3;
                }
                if (kmtExerciseQuestion.getQuestionClassType() == 2){
                    if (!kmtExerciseQuestion.getBigQuestionIndex().equals(request.getBigQuestionIndex())){
                        return responseEntity.failure(ResponseConstant.CODE_000, "大题号不能修改");
                    }
                    kmtExerciseMapper.updateQuestionEditData(request, questionImage, answerImage, dealStatus);
                    saveJSONFile(kmtExerciseQuestion, request, questionImage);
                    AijiaThreadPool.getInstance().submit(new SyncSaveExerciseQuestionsTask(kmtExerciseMapper, exerciseQuestionSearchService,
                            Collections.singletonList(kmtExerciseQuestion.getId())));
                } else {
                    if (!kmtExerciseQuestion.getBigQuestionIndex().equals(request.getBigQuestionIndex())){
                        //如果大题号不同，判断当前课时的大题号是否存在，存在的话不能修改
                        String bigQuestionIndex = kmtExerciseMapper.findBigQuestionIndexByBookCourseId(kmtExerciseQuestion.getBookCourseId(), request.getBigQuestionIndex());
                        if (StringUtil.isNotBlank(bigQuestionIndex)){
                            return responseEntity.failure(ResponseConstant.CODE_000, "当前课时已存在该大题号，修改失败");
                        }
                    }
                    //修改大题以及小题的题号
                    kmtExerciseMapper.updateQuestionEditData(request, questionImage, answerImage, dealStatus);
                    saveJSONFile(kmtExerciseQuestion, request, questionImage);
                    if (kmtExerciseQuestion.getQuestionClassType() == 0){
                        //单题
                        AijiaThreadPool.getInstance().submit(new SyncSaveExerciseQuestionsTask(kmtExerciseMapper, exerciseQuestionSearchService,
                                Collections.singletonList(kmtExerciseQuestion.getId())));
                    } else {
                        //存在小题
                        kmtExerciseMapper.updateBigQuestionIndexByParentId(request.getBigQuestionIndex(), kmtExerciseQuestion.getId());
                        AijiaThreadPool.getInstance().submit(new SyncSaveExerciseBookCourseTask(kmtExerciseMapper, exerciseQuestionSearchService,
                                kmtExerciseQuestion.getBookCourseId()));
                    }

                }
                //更新练习册为审核中
                if (dealStatus == 3){
                    kmtExerciseMapper.updateExerciseBookNotAuditing(kmtExerciseQuestion.getExerciseBookId());
                }
            }
            return responseEntity.success("成功");
        } else {
            return responseEntity.failure("没有操作权限");
        }
    }

    /**
     * 保存文件json和es
     * @param kmtExerciseQuestion
     * @param request
     * @param questionImage
     */
    private void saveJSONFile (KmtExerciseQuestion kmtExerciseQuestion, AddExerciseQuestionRequest request, String questionImage){
        String content = JSONFileUtil.readJSONFile(PropertiesUtil.getValue("EXERCISE_JSON_FILE_PATH") + kmtExerciseQuestion.getQuestionPath());
        Map<String, Object> map = null;
        if (StringUtil.isBlank(content)){
            map = new HashMap<>();
        } else {
            map = JSON.parseObject(content, Map.class);
        }
        map.put("question", request.getQuestion());
        map.put("question_text", request.getQuestionText());
        if (StringUtil.isBlank(questionImage)){
            ExerciseQuestionAnswerValue questionJson = new ExerciseQuestionAnswerValue();
            questionJson.setType(1);
            questionJson.setValue(request.getQuestion());
            map.put("question_json", JSON.toJSONString(Collections.singletonList(questionJson)));
        } else {
            ExerciseQuestionAnswerValue questionJson = ExerciseUtil.getExerciseQuestionAnswerValue(questionImage);
            if (questionJson != null){
                map.put("question_json", JSON.toJSONString(Collections.singletonList(questionJson)));
            }
        }
        map.put("option_answer", request.getAnswer());
        map.put("answer_json", request.getAnswerJson());
        map.put("analysis", request.getQuestionAnalysis());
        map.put("analysis_json", request.getAnalysisJson());
        JSONFileUtil.saveJSONFile(PropertiesUtil.getValue("EXERCISE_JSON_FILE_PATH") + kmtExerciseQuestion.getQuestionPath(), JSON.toJSONString(map));
        questionInfoService.updateQuestionInfoByQuestionId(kmtExerciseQuestion.getId(), map);
    }

    /**
     * chax
     * @param htmlStr
     * @return
     */
    private String getImagePath (String htmlStr) {
        String result = null;
        List<String> list = new ArrayList<>();
        String img = "";
        Pattern p_image;
        Matcher m_image;
        // String regEx_img = "<img.*src=(.*?)[^>]*?>"; //图片链接地址
        String regEx_img = "<img.*src\\s*=\\s*(.*?)[^>]*?>";
        p_image = Pattern.compile(regEx_img, Pattern.CASE_INSENSITIVE);
        m_image = p_image.matcher(htmlStr);
        while (m_image.find()) {
            // 得到<img />数据
            img = m_image.group();
            // 匹配<img>中的src数据
            Matcher m = Pattern.compile("src\\s*=\\s*[\"|\']?(.*?)([\"|\']|>|\\s+)").matcher(img);
            while (m.find()) {
                list.add(m.group(1));
            }
        }
        if (list.size() > 0){
            result = list.get(0).replace(PropertiesUtil.getValue("STATIC_FILE_URL"), "");
            //测试环境图片可能用的是正式的图片，再做一次replace
            result = result.replace("https://img.sharingschool.com", "");
        }
        return result;
    }

    private KmtExerciseQuestion saveQuestion(KmtExerciseBook book,AddExerciseQuestionRequest request, KmtExerciseQuestion previousQuestion,
                                             String questionImage, String answerImage){
//        kmtExerciseMapper.updateQuestionNumByBookIdAndNum(book.getBookId(), previousQuestion.getNum() + 1);
        KmtExerciseQuestion kmtExerciseQuestion = new KmtExerciseQuestion();
        String newQuestionId = IdWorker.generateId();
        kmtExerciseQuestion.setId(newQuestionId);
        kmtExerciseQuestion.setBookId(previousQuestion.getBookId());
        kmtExerciseQuestion.setUnitId(previousQuestion.getUnitId());
        kmtExerciseQuestion.setBookCourseId(previousQuestion.getBookCourseId());
        if (request.getQuestionClassType() == 2){//增加小题
            kmtExerciseQuestion.setBigQuestionIndex(previousQuestion.getBigQuestionIndex());
            if (StringUtil.isBlank(previousQuestion.getParentId()) && previousQuestion.getQuestionClassType() == 0){
                //如果上一题是单题，修改为大标题
                kmtExerciseMapper.updateKmtExerciseQuestionClassTypeAndParentId(null, 1, previousQuestion.getId());
                kmtExerciseQuestion.setParentId(previousQuestion.getId());
                kmtExerciseQuestion.setQuestionClassType(2);
            } else {
                kmtExerciseQuestion.setParentId(previousQuestion.getId());
                kmtExerciseQuestion.setQuestionClassType(2);
            }
        } else {
            kmtExerciseQuestion.setBigQuestionIndex(request.getBigQuestionIndex());
            //kmtExerciseQuestion.setParentId(request.getQuestionId());
            kmtExerciseQuestion.setQuestionClassType(0);
        }
        kmtExerciseQuestion.setImage(questionImage);
        kmtExerciseQuestion.setCreateTime(new Date());
        kmtExerciseQuestion.setDealStatus(3);
        kmtExerciseQuestion.setAnswerImage(answerImage);
        kmtExerciseQuestion.setNum(previousQuestion.getNum() + 1);
        kmtExerciseQuestion.setPageIndex(request.getPageIndex());
        if (StringUtil.isBlank(request.getSmallQuestionIndex())){
            request.setSmallQuestionIndex("0");
        }
        kmtExerciseQuestion.setSmallQuestionName(request.getSmallQuestionIndex());
        kmtExerciseQuestion.setCourseName(previousQuestion.getCourseName());
        kmtExerciseQuestion.setIsCommonFlag(previousQuestion.getIsCommonFlag());
        kmtExerciseQuestion.setExerciseBookId(previousQuestion.getExerciseBookId());
        kmtExerciseQuestion.setHandlerSubmitTime(DateUtil.parse_yyyyMMddHHmmss("1970-01-01 00:00:01"));
        kmtExerciseQuestion.setQuestionPath(previousQuestion.getQuestionPath().substring(0, previousQuestion.getQuestionPath().lastIndexOf("/")) + "/" + newQuestionId + ".json");
        kmtExerciseQuestionMapper.insert(kmtExerciseQuestion);
        return kmtExerciseQuestion;
    }



    @Override
    public ResponseEntity deleteExerciseQuestion(String userId, ExerciseQuestionDealRequest request) {
        ResponseEntity responseEntity = new ResponseEntity();
        if (isManagerSubmit(userId)){
            if (request.getQuestionClassType() == 2){
                //this.deleteOneQuestions(request.getQuestionId());
            	return this.deleteOneQuestionsNew(request.getQuestionId());
            } else {
                //this.deleteOneBigQuestions(request.getQuestionId());
            	return this.deleteOneBigQuestionsNew(request.getQuestionId());
            }
        } else {
            return responseEntity.failure("没有操作权限");
        }
    }

    /**
     * 删除小题
     * @param questionId
     */
    private void deleteOneQuestions(String questionId){
        KmtExerciseQuestion kmtExerciseQuestion = kmtExerciseQuestionMapper.selectById(questionId);
        if  (kmtExerciseQuestion != null){
            //如果删除的是大题，找到第二个题，把它设置为大题
            if (kmtExerciseQuestion.getQuestionClassType() == 1){
                List<String> questionIds = kmtExerciseMapper.findQuestionIdByParentId(questionId);
                int questionClassType = 0;//默认是单题
                if (questionIds.size() > 2) {// 下面题目数量大于2时，才是大题
                    //删除后不只一个小题
                    questionClassType = 1;
                }

                //更新兄弟题目字段
                if (questionIds.size() > 1){
                    String parentId = questionIds.get(1);
                    kmtExerciseMapper.updateKmtExerciseQuestionClassTypeAndParentId(null, questionClassType, parentId);
                    for (int i = 2; i < questionIds.size(); i++){
                        kmtExerciseMapper.updateKmtExerciseQuestionClassTypeAndParentId(parentId, 2, questionIds.get(i));
                    }
                }
            }
            logger.info("删除={}", JSON.toJSONString(kmtExerciseQuestion));
            kmtExerciseQuestionMapper.deleteById(questionId);
            kmtExerciseMapper.deleteKnowByQuestionId(Collections.singletonList(questionId));

            //删除es
            exerciseQuestionSearchService.delete(questionId);
            QueryBuilder queryBuilder = QueryBuilders.termQuery("question_id.keyword", questionId);
            exerciseQuestionSearchService.deleteByQuery("kmt_exercise_question_know", queryBuilder);
            //sql，重新排序，修改num的值
            kmtExerciseQuestionMapper.updateQuestions(kmtExerciseQuestion.getBookId(),kmtExerciseQuestion.getNum());

            AijiaThreadPool.getInstance().submit(new SyncSaveExerciseBookTask(kmtExerciseMapper,
                    exerciseQuestionSearchService, kmtExerciseQuestion.getBookId()));
        }
    }



    /**
     * 删除大题
     * @param questionId
     */
    private void deleteOneBigQuestions(String questionId){
        List<String> questionIds = kmtExerciseMapper.findQuestionIdByParentId(questionId);
        KmtExerciseQuestion kmtExerciseQuestion = kmtExerciseQuestionMapper.selectById(questionId);
        if (CollectionUtils.isNotEmpty(questionIds)){
            kmtExerciseQuestionMapper.deleteBatchIds(questionIds);
            kmtExerciseMapper.deleteKnowByQuestionId(questionIds);

            kmtExerciseQuestionMapper.updateBigQuestion(kmtExerciseQuestion.getBookId(),questionIds.size(),kmtExerciseQuestion.getNum());
            //删除es
            QueryBuilder queryBuilder = QueryBuilders.termsQuery("id.keyword", questionIds);
            exerciseQuestionSearchService.deleteByQuery("kmt_exercise_questions", queryBuilder);
            queryBuilder = QueryBuilders.termsQuery("question_id.keyword", questionIds);
            exerciseQuestionSearchService.deleteByQuery("kmt_exercise_question_know", queryBuilder);
        }
    }


//    @Override
    public ResponseEntity moveExerciseQuestionOld(String userId, ExerciseQuestionDealRequest request) {
        ResponseEntity responseEntity = new ResponseEntity();
        if (!isManagerSubmit(userId)){
            return responseEntity.failure("没有操作权限");
        }

        KmtExerciseQuestion kmtExerciseQuestion  = kmtExerciseQuestionMapper.selectById(request.getQuestionId());
        if(kmtExerciseQuestion ==null) {
        	return responseEntity.failure("题目不存在！");
        }


        if (request.getQuestionClassType() == 2 && kmtExerciseQuestion.getQuestionClassType() == 0){
            //如果是单题，移动一个小题不能移动
            return responseEntity.failure(ResponseConstant.CODE_000, "单题不能移动小题，请使用移动大题功能");
        }

        KmtExerciseQuestion questionWorkbook  = kmtExerciseQuestionMapper.findPreviousById(kmtExerciseQuestion.getExerciseBookId(), request.getQuestionId());
        if (request.getQuestionClassType() == 2){

            //如果移动到最开始或最末尾， 提示不能再移动
            String parentId = null;
            if (StringUtil.isBlank(kmtExerciseQuestion.getParentId())){
                parentId = kmtExerciseQuestion.getId();
            } else {
                parentId = kmtExerciseQuestion.getParentId();
            }
            List<KmtExerciseQuestion> tempList = kmtExerciseMapper.findQuByParentId(parentId);
            //移动小题
            if (request.getType() == 1){
                //第一个小题不能往上移
                if (tempList != null && tempList.size() > 0){
                    if (kmtExerciseQuestion.getId().equals(tempList.get(0).getId())){
                        return  responseEntity.failure(ResponseConstant.CODE_000, "已移动到第一小题");
                    }
                    //如果是第二个小题上移， 修改题目的父子关系
                    if (kmtExerciseQuestion.getId().equals(tempList.get(1).getId())){
                        //先特殊修改第一题和第二题， 第三题开始修改父题
                        kmtExerciseMapper.updateParentIdAndQuestionClassTypeAndNumById(null,
                                1, kmtExerciseQuestion.getNum() - 1, kmtExerciseQuestion.getId());
                        kmtExerciseMapper.updateParentIdAndQuestionClassTypeAndNumById(kmtExerciseQuestion.getId(),
                                2, kmtExerciseQuestion.getNum(), tempList.get(0).getId());
                        for (int i =2; i < tempList.size(); i++){
                            kmtExerciseMapper.updateParentIdById(kmtExerciseQuestion.getId(), tempList.get(i).getId());
                        }
                    } else {
                        //上移
                        kmtExerciseMapper.updateByBookIdAndNum(kmtExerciseQuestion.getBookId(), kmtExerciseQuestion.getNum(),kmtExerciseQuestion.getNum() - 1);
                        kmtExerciseMapper.updateQuestionNumByQuestionId(kmtExerciseQuestion.getId(), kmtExerciseQuestion.getNum() - 1);
                    }
                }
            } else if (request.getType() == 2){
                //最后一个小题不能往下移
                if (tempList != null && tempList.size() > 0){
                    if (kmtExerciseQuestion.getId().equals(tempList.get(tempList.size() - 1).getId())){
                        return responseEntity.failure(ResponseConstant.CODE_000, "已移动到最后一个小题的位置");
                    }
                    //如果是第一小题的下移， 修改题目父子关系
                    if (kmtExerciseQuestion.getId().equals(tempList.get(0).getId())){
                        //先特殊修改第一题和第二题， 第三题开始修改父题
                        kmtExerciseMapper.updateParentIdAndQuestionClassTypeAndNumById(tempList.get(1).getId(),
                                2, kmtExerciseQuestion.getNum() + 1, kmtExerciseQuestion.getId());
                        kmtExerciseMapper.updateParentIdAndQuestionClassTypeAndNumById(null,
                                1, kmtExerciseQuestion.getNum(), tempList.get(1).getId());
                        for (int i =2; i < tempList.size(); i++){
                            kmtExerciseMapper.updateParentIdById(tempList.get(1).getId(), tempList.get(i).getId());
                        }
                    } else {
                        //下移
                        kmtExerciseMapper.updateByBookIdAndNum(kmtExerciseQuestion.getBookId(), kmtExerciseQuestion.getNum(), kmtExerciseQuestion.getNum() + 1);
                        kmtExerciseMapper.updateQuestionNumByQuestionId(kmtExerciseQuestion.getId(), kmtExerciseQuestion.getNum() + 1);
                    }
                }
            }
        } else  if (request.getQuestionClassType() == 1){
            if (request.getType() == 1){
                //大题上移, 先查找上一题的题目id列表
                KmtExerciseQuestion previousQuestion = kmtExerciseMapper.findPreviousByBookIdAndNum(kmtExerciseQuestion.getBookId(), kmtExerciseQuestion.getNum() - 1);
                if (previousQuestion ==null ||  !previousQuestion.getBookCourseId().equals(kmtExerciseQuestion.getBookCourseId())){
                    return responseEntity.failure(ResponseConstant.CODE_000, "已移动到第一题位置");
                }

                List<KmtExerciseQuestion> previousQuestions = kmtExerciseMapper.findQuestionsByBookCourseIdAndBigIndex(previousQuestion.getBookCourseId(), previousQuestion.getBigQuestionIndex());
                List<KmtExerciseQuestion> thisQuestions = kmtExerciseMapper.findQuestionsByBookCourseIdAndBigIndex(kmtExerciseQuestion.getBookCourseId(), kmtExerciseQuestion.getBigQuestionIndex());
                if (previousQuestions != null && thisQuestions != null){
                    int preSize = previousQuestions.size();
                    int thisSize = thisQuestions.size();
                    List<String> previousIds = previousQuestions.stream().map(KmtExerciseQuestion::getId).collect(Collectors.toList());
                    List<String> thisIds = thisQuestions.stream().map(KmtExerciseQuestion::getId).collect(Collectors.toList());
                    kmtExerciseMapper.updateQuestionMinusNumByQuestionIds(thisIds, preSize);
                    kmtExerciseMapper.updateQuestionAddNumByQuestionIds(previousIds, thisSize);
                }
            } else if (request.getType() == 2){
                List<KmtExerciseQuestion> thisQuestions = kmtExerciseMapper.findQuestionsByBookCourseIdAndBigIndex(kmtExerciseQuestion.getBookCourseId(), kmtExerciseQuestion.getBigQuestionIndex());
                int thisSize = thisQuestions.size();
                KmtExerciseQuestion nextQuestion = kmtExerciseMapper.findNextByBookIdAndNum(kmtExerciseQuestion.getBookId(), kmtExerciseQuestion.getNum() + thisSize);
                if (nextQuestion == null || !nextQuestion.getBookCourseId().equals(kmtExerciseQuestion.getBookCourseId())){
                    return responseEntity.failure(ResponseConstant.CODE_000, "已移动到最后一题位置");
                }
                List<KmtExerciseQuestion> nextQuestions = kmtExerciseMapper.findQuestionsByBookCourseIdAndBigIndex(nextQuestion.getBookCourseId(), nextQuestion.getBigQuestionIndex());
                if (nextQuestions != null && thisQuestions != null){
                    int nextSize = nextQuestions.size();
                    List<String> nextIds = nextQuestions.stream().map(KmtExerciseQuestion::getId).collect(Collectors.toList());
                    List<String> thisIds = thisQuestions.stream().map(KmtExerciseQuestion::getId).collect(Collectors.toList());
                    kmtExerciseMapper.updateQuestionMinusNumByQuestionIds(nextIds, thisSize);
                    kmtExerciseMapper.updateQuestionAddNumByQuestionIds(thisIds, nextSize);
                }
            }
        }
        AijiaThreadPool.getInstance().submit(new SyncSaveExerciseBookCourseTask(kmtExerciseMapper, exerciseQuestionSearchService, kmtExerciseQuestion.getBookCourseId()));
        return responseEntity.success("成功");

    }



    /**
     * 是否是管理员提交
     * @param userId
     * @return
     */
    private boolean isManagerSubmit (String userId){
        TbLibManager manager = libManagerMapper.findByUserIdAndType(userId, 18);
        if (manager != null) {
            //管理员把状态设置为3，不改任何状态
            return true;
        }
        return false;
    }


    @Override
    public ResponseEntity findEditExerciseBooks(ExerciseBooksEditRequest request) {
        ResponseEntity responseEntity = new ResponseEntity();
        List<ExerciseBookEditDto> list = kmtExerciseMapper.findEditExerciseBooks(request);
        list = this.sortByBookName(list);
        Map<String, Object> data = new HashMap<>();
        data.put("books", list);
        return responseEntity.success(data, "成功");
    }

    // TODO: 2020/11/19    9.24 19:29
    /**
     * 练习册名称排序
     * @param list
     * @return
     */
    private List<ExerciseBookEditDto> sortByBookName (List<ExerciseBookEditDto> list){
        if (list != null){
            Map<String, List<ExerciseBookEditDto>> temp = new HashMap<>();
            // TODO: 2020/11/19  9.26 12:22
            List<ExerciseBookEditDto> tmpList = null;
            for (ExerciseBookEditDto item : list){
                if (temp.containsKey(item.getExerciseBookName())){
                    tmpList = temp.get(item.getExerciseBookName());
                    tmpList.add(item);
                    temp.put(item.getExerciseBookName(), tmpList);
                } else {
                    tmpList = new ArrayList<>();
                    tmpList.add(item);
                    temp.put(item.getExerciseBookName(), tmpList);
                }
            }
            ArrayList tempList = new ArrayList(temp.keySet());
            List<String> sortList = SortByChineseUtil.sortByChinese(tempList);
            list.clear();
            for (String str : sortList) {
                list.addAll(temp.get(str));
            }
        }
        return list;
    }

    @Override
    public ResponseEntity findEditExerciseQuestions(ExerciseQuestionsEditRequest request) {
        Page page = new Page(request.getPageNo(), request.getPageSize());
        List<ExerciseQuestionEditDto> list = kmtExerciseMapper.findEditExerciseQuestions(request, page);
        Map<String, String> users = new HashMap<>();
        Map<String, Integer> temp = new HashMap<>(); //临时变量，用来保存题目是否已在列表中
        for (ExerciseQuestionEditDto questionEditDto : list){
            // TODO: 2020/11/18 迁移  9.17 15:53
            String versionStr = knowledgeVersion.getVersionNameBySubjectAndVersionNum(questionEditDto.getSubject(), questionEditDto.getVersionType());
            questionEditDto.setVersionName(versionStr);

            // TODO: 2020/11/19 迁移  9.23 11:39  9.24 20:57
            if (temp.containsKey(questionEditDto.getQuestionId())){
                questionEditDto.setEditStatus(2);
            } else {
                temp.put(questionEditDto.getQuestionId(), 1);
                if (questionEditDto.getDealStatus() != 2){
                    questionEditDto.setEditStatus(2);
                }
            }

            if (questionEditDto.getBookType() == 1){
                questionEditDto.setBookTypeName("上半");
            } else if (questionEditDto.getBookType() == 2){
                questionEditDto.setBookTypeName("下半");
            } else if (questionEditDto.getBookType() == 3) {
                questionEditDto.setBookTypeName("不同");
                questionEditDto.setBookType(0);
            } else {
                questionEditDto.setBookTypeName("整本");
            }
            if (users.containsKey(questionEditDto.getUserId())){
                questionEditDto.setCreateUserName(users.get(questionEditDto.getUserId()));
            } else {
                TbLibManager libManager = libManagerMapper.findByUserIdAndType(questionEditDto.getUserId(), 17);
                if (libManager != null){
                    questionEditDto.setCreateUserName(libManager.getName());
                    users.put(questionEditDto.getUserId(), libManager.getName());
                }
            }
        }
        Map<String, Object> data = new HashMap<>();
        data.put("pageNo", page.getCurrent());
        data.put("pageSize", page.getSize());
        data.put("pages", page.getPages());
        data.put("total", page.getTotal());
        data.put("list", list);
        return new ResponseEntity().success(data, "成功");
    }

    @Override
    public ResponseEntity removeQuestionNumMark(String userId, QuestionNumMarkRemoveVo request) throws CommonException, IOException {
        // TODO: 2020/11/19  11.13 15:30
        ResponseEntity responseEntity = new ResponseEntity();
        if (request.getStartX() < 0 || request.getStartY() < 0 || request.getEndX() < 0 || request.getEndY() < 0){
            return  responseEntity.failure("请把坐标定位在图片内");
        } else {
            KmtExerciseQuestion kmtExerciseQuestion = kmtExerciseQuestionMapper.selectById(request.getQuestionId());
            if (kmtExerciseQuestion != null && StringUtil.isNotBlank(kmtExerciseQuestion.getImage())) {
                Map<String, Integer> widthAndHeight = ExerciseUtil.getImageWidthAndHeight(kmtExerciseQuestion.getImage());
                int width = widthAndHeight.get("width");
                int height = widthAndHeight.get("height");
                if (request.getStartX() > width || request.getEndX() > width || request.getStartY() > height || request.getEndY() > height){
                    return responseEntity.failure("请把坐标定位在图片内");
                }
                // 新加入字段
                TrTrainUserExercise questionInfo = userExerciseMapper.selectOne(
                        new QueryWrapper<TrTrainUserExercise>().eq("question_id",request.getQuestionId())
                                .eq("user_id", userId));
                if(Objects.isNull(questionInfo)){
                    questionInfo = new TrTrainUserExercise();
                    questionInfo.setUpdateTime(DateUtil.format_yyyyMMddHHmmss(new Date()));
                    questionInfo.setCreateTime(DateUtil.format_yyyyMMddHHmmss(new Date()));
                    questionInfo.setUserId(userId);
                    questionInfo.setExerciseId(kmtExerciseQuestion.getBookId());
                    questionInfo.setQuestionId(request.getQuestionId());
                    questionInfo.setConciseId(request.getStartX());
                    questionInfo.setAnalysisSubmitType(request.getStartY());
                    questionInfo.setUpdateType(request.getEndX());
                    questionInfo.setEsLoaded(request.getEndY());
                    questionInfo.setEnumerate(request.getEnumerate());
                    questionInfo.setStatus(0);
                    userExerciseMapper.insert(questionInfo);
                }else{
                    kmtExerciseMapper.updateQuestionStartPointAndEndPoint(request.getStartX(), request.getStartY(),
                            request.getEndX(), request.getEndY(), request.getQuestionId(),request.getEnumerate(),userId);
                }


                // 判断为 1
//                if (request.getEnumerate()==1){
//                        String path = this.removeQuestionNumMark(request.getStartX(), request.getStartY(), request.getEndX(), request.getEndY(), kmtExerciseQuestion.getImage());
//                        String content = JSONFileUtil.readJSONFile(PropertiesUtil.getValue("EXERCISE_JSON_FILE_PATH") + kmtExerciseQuestion.getQuestionPath());
//                        if (StringUtil.isNotBlank(content) && StringUtil.isNotBlank(path)) {
//                            Map<String, Object> map = JSON.parseObject(content, Map.class);
//                            ExerciseQuestionAnswerValue questionAnswerValue = ExerciseUtil.getExerciseQuestionAnswerValue(path);
//                            if (questionAnswerValue == null){
//                                map.put("question_json", "[]");
//                            } else {
//                                map.put("question_json", Collections.singletonList(questionAnswerValue));
//                            }
//                            map.put("question", "<img src='" + PropertiesUtil.getValue("STATIC_FILE_URL") + path + "'/>");
//                            map.put("userId",userId);
//                            map.put("questionPath",path);
//                            map.put("bookId",kmtExerciseQuestion.getBookId());
//                            //JSONFileUtil.saveJSONFile(PropertiesUtil.getValue("EXERCISE_JSON_FILE_PATH") + kmtExerciseQuestion.getQuestionPath(), JSON.toJSONString(map));
//                            questionInfoService.updateQuestionAndJsonByQuestionId(kmtExerciseQuestion.getId(), map);
//                            //AijiaThreadPool.getInstance().submit(new SyncSaveExerciseQuestionsTask(kmtExerciseMapper, exerciseQuestionSearchService,  Collections.singletonList(kmtExerciseQuestion.getId())));
//                        }
//                }
            }
            return responseEntity.success("成功");
        }
    }

    @Override
    public String removeQuestionNumMark(int startX, int startY, int endX, int endY, String filePath) throws CommonException, IOException {
        int temp = 0;
        if (startX > endX){
            temp = startX;
            startX = endX;
            endX = temp;
        }
        if (startY > endY){
            temp = startY;
            startY = endY;
            endY = temp;
        }
        String result = null;
        if (StringUtil.isNotBlank(filePath) && endX > 0 && endY > 0){
            //1、查询是否有备份图片，没有的话新增一个备份图片
            //this.checkBakImage(filePath);
            //2、生成一个临时图片， 图片区域填充， 临时图片要随机, 把图片上传到oss
            result = this.createNewFile(startX, startY, endX, endY, filePath);
        }
        return result;
    }

    /**
     * 检查是否有备份图片
     * @param relatePath
     */
    private void checkBakImage (String relatePath){
        String bakParentPath = "/www/files/data/upload/ai/picture_bak/"; //备份图片父级目录
        if (relatePath.indexOf("/ai/results") != -1){
            String filePath = relatePath.replace("/ai/results/", bakParentPath); //备份图片绝对路径
            //如果目录不存在，新建目录，如果存在，判断文件是否存在
            String dir = filePath.substring(0, filePath.lastIndexOf("/") + 1);
            File dirFile = new File(dir);
            // TODO: 2020/11/19  11.10 16:56
            if (!dirFile.exists()){
                dirFile.mkdirs();
            }
            File file  = new File(filePath);
            if (!file.exists()){
                OSSUtils.downloadFile(filePath, relatePath.substring(1));
            }
        } else {
            logger.error("图片路径不存在 = {}", relatePath);
        }
    }

    // TODO: 2020/11/19  11.12 14:08
    private String createNewFile (int startX, int startY, int endX, int endY, String relatePath) throws CommonException, IOException {
        String prefix = PropertiesUtil.getValue("file_save_path");
        String dir = relatePath.substring(0, relatePath.lastIndexOf("/") + 1);
        String fileName = UUID.randomUUID().toString();
        String resultFilePath = prefix + dir  + "result_" + fileName + relatePath.substring(relatePath.lastIndexOf("."));
//        OSSUtils.downloadFile(resultFilePath, relatePath.substring(1));
        boolean result = drawImagePartColorByPosition(startX, startY, endX, endY, resultFilePath, relatePath);
        if (result){
            OSSUtils.uploadFile(resultFilePath, resultFilePath.replace(prefix + "/", ""));
            return resultFilePath.replace(prefix, "");
        } else {
            return null;
        }
    }

    /**
     * 区域颜色填充，以左上角颜色填充
     * @param minx
     * @param miny
     * @param maxx
     * @param maxy
     * @param filePath
     */
    public boolean drawImagePartColorByPosition (int minx, int miny,  int maxx, int maxy, String filePath, String relatePath) throws CommonException, IOException {
        BufferedImage bi = null;
        boolean result = true;
        //有宽高的才处理
        if (maxx > 0 && maxy > 0){
//            String prefix = PropertiesUtil.getValue("OSS_INTERNAL_PATH");
//                String prefix = "https://xt-sharingschool.oss-cn-shenzhen.aliyuncs.com";
            String prefix = "https://img-sharingschool-new.oss-cn-shenzhen-internal.aliyuncs.com";
            InputStream in = getImageStream(prefix + relatePath);
            logger.info("原图={}", prefix + relatePath);
            if (in == null){
                result = false;
                logger.error("读取图片文件流失败，path={}", relatePath);
                throw new CommonException("读取文件失败");
            }
            bi = ImageIO.read(in);
            logger.info("图片宽={}, 图片高={}", bi.getWidth(), bi.getHeight());
            int pixel = bi.getRGB(minx, miny); //左上角像素值
//                int pixel = -256;
            for (int i = minx; i < maxx; i++) {
                for (int j = miny; j < maxy; j++) {
                    bi.setRGB(i, j, pixel);
                }
            }
            File file = new File(filePath);
            String dirPath = filePath.substring(0, filePath.lastIndexOf("/") + 1);
            File dir = new File(dirPath);
            if (!dir.exists()){
                dir.mkdirs();
            }
            logger.info("结果图={}", filePath);
            ImageIO.write(bi, getContentType(relatePath), file);
        }
        return result;
    }

    public String getContentType(String relatePath) throws IOException {
//        String prefix = PropertiesUtil.getValue("OSS_INTERNAL_PATH");
//        String prefix = "https://xt-sharingschool.oss-cn-shenzhen.aliyuncs.com";
        String prefix = "https://img-sharingschool-new.oss-cn-shenzhen-internal.aliyuncs.com";
        InputStream in = getImageStream(prefix + relatePath);
        byte[] buf = new byte[10240];
        in.read(buf);
        String type = "";
        ByteArrayInputStream bais = null;
        MemoryCacheImageInputStream mcis = null;
        try {
            bais = new ByteArrayInputStream(buf);
            mcis = new MemoryCacheImageInputStream(bais);
            Iterator itr = ImageIO.getImageReaders(mcis);
            while (itr.hasNext()) {
                ImageReader reader = (ImageReader)itr.next();
                type = reader.getFormatName().toLowerCase();
            }
        }
        finally {
            if (bais != null)
                try {
                    bais.close();
                }
                catch (IOException ioe)
                {
                }
            if (mcis != null)
                try {
                    mcis.close();
                }
                catch (IOException ioe)
                {
                }
        }
        return type;
    }


    // TODO: 2020/11/19 11.12 18:43
    /**
     * 获取网络图片流
     *
     * @param url
     * @return
     */
    private InputStream getImageStream(String url) {
        try {
            HttpURLConnection connection = (HttpURLConnection) new URL(url).openConnection();
            connection.setReadTimeout(60000);
            connection.setConnectTimeout(60000);
            connection.setRequestMethod("GET");
            logger.info("返回码={}, 返回信息={}", connection.getResponseCode(), connection.getResponseMessage());
            if (connection.getResponseCode() == HttpURLConnection.HTTP_OK) {
                InputStream inputStream = connection.getInputStream();
                return inputStream;
            }
        } catch (IOException e) {
            logger.error("获取网络图片出现异常，图片路径为：" + url);
        }
        return null;
    }

    private ResponseEntity editExerciseQuestionNew(String userId, AddExerciseQuestionRequest request ){
    	 ResponseEntity responseEntity = new ResponseEntity();
    	String questionImage = getImagePath(request.getQuestion());
        String answerImage = getImagePath(request.getAnswer());
        KmtExerciseQuestion kmtExerciseQuestion = kmtExerciseQuestionMapper.selectById(request.getQuestionId());
        int dealStatus = kmtExerciseQuestion.getDealStatus();
        if (kmtExerciseQuestion.getDealStatus() == 2){
            dealStatus = 3;
        }
        if (kmtExerciseQuestion.getQuestionClassType() == 2){
            kmtExerciseMapper.updateSmallQuestionEditData(request, questionImage, answerImage, dealStatus);
            saveJSONFile(kmtExerciseQuestion, request, questionImage);
            AijiaThreadPool.getInstance().submit(new SyncSaveExerciseQuestionsTask(kmtExerciseMapper, exerciseQuestionSearchService,
                    Collections.singletonList(kmtExerciseQuestion.getId())));
        } else {
            if (!kmtExerciseQuestion.getBigQuestionIndex().equals(request.getBigQuestionIndex())){
                //如果大题号不同，判断当前课时的大题号是否存在，存在的话不能修改
                String bigQuestionIndex = kmtExerciseMapper.findBigQuestionIndexByBookCourseId(kmtExerciseQuestion.getBookCourseId(), request.getBigQuestionIndex());
                if (StringUtil.isNotBlank(bigQuestionIndex)){
                    return responseEntity.failure(ResponseConstant.CODE_000, "当前课时已存在该大题号，修改失败");
                }
            }
            //修改大题以及小题的题号
            kmtExerciseMapper.updateQuestionEditData(request, questionImage, answerImage, dealStatus);
            saveJSONFile(kmtExerciseQuestion, request, questionImage);
            if (kmtExerciseQuestion.getQuestionClassType() == 0){
                //单题
                AijiaThreadPool.getInstance().submit(new SyncSaveExerciseQuestionsTask(kmtExerciseMapper, exerciseQuestionSearchService,
                        Collections.singletonList(kmtExerciseQuestion.getId())));
            } else {
                //存在小题
                kmtExerciseMapper.updateBigQuestionIndexByParentId(request.getBigQuestionIndex(), kmtExerciseQuestion.getId());
                AijiaThreadPool.getInstance().submit(new SyncSaveExerciseBookCourseTask(kmtExerciseMapper, exerciseQuestionSearchService,
                        kmtExerciseQuestion.getBookCourseId()));
            }

        }
        //更新练习册为审核中
        if (dealStatus == 3){
            kmtExerciseMapper.updateExerciseBookNotAuditing(kmtExerciseQuestion.getExerciseBookId());
        }

        return responseEntity.success("成功");
    }

    private ResponseEntity addExerciseQuestionNew(String userId, AddExerciseQuestionRequest request) {
    	 ResponseEntity responseEntity = new ResponseEntity();
         if (!isManagerSubmit(userId)){
             return responseEntity.failure("没有操作权限");
         }

         String questionImage = getImagePath(request.getQuestion());
         String answerImage = getImagePath(request.getAnswer());

         //题目id不存在则是新增, 前端传过来的是新增题目的上一题
         KmtExerciseBook book  = kmtExerciseBookMapper.findExerciseBookByQuestionId(request.getQuestionId());
         KmtExerciseQuestion previousQuestion;
         if (StringUtil.isBlank(request.getQuestionId())) {
             previousQuestion = kmtExerciseMapper.findPreviousByBookIdAndNum(book.getParentBookId(), request.getNum());
         } else {
          	 previousQuestion = kmtExerciseQuestionMapper.findPreviousById(book.getId(),request.getQuestionId());
          	 previousQuestion.setExerciseBookId(book.getId());
         }

         KmtExerciseQuestion newQuestion;
         //新增小题
         if (request.getQuestionClassType() == 2){
             newQuestion = saveQuestion(book, request, previousQuestion, questionImage, answerImage);
             saveJSONFile(newQuestion, request, questionImage);
             AijiaThreadPool.getInstance().submit(new SyncSaveExerciseBookTask(kmtExerciseMapper, exerciseQuestionSearchService, book.getBookId()));
         } else {
             //新增大题, 如果已存在当前大题号，则报错
             String bqi = kmtExerciseMapper.findBigQuestionIndexByBookCourseId(previousQuestion.getBookCourseId(), request.getBigQuestionIndex());
             if (StringUtil.isBlank(bqi)){
                 newQuestion = saveQuestion(book, request, previousQuestion, questionImage, answerImage);
                 saveJSONFile(newQuestion, request, questionImage);
                 AijiaThreadPool.getInstance().submit(new SyncSaveExerciseBookTask(kmtExerciseMapper, exerciseQuestionSearchService, book.getBookId()));
             } else {
                 return responseEntity.failure(ResponseConstant.CODE_000, "当前课时已存在该大题号，请在大题号下添加小题");
             }
         }


         // cc 新增关联表
        Map<String, Integer> previousNumberAndSortNumber = getPreviousNumber(previousQuestion);
        QuestionWorkbook questionWorkbook = new QuestionWorkbook();
        questionWorkbook.setBookId(book.getParentBookId());
        questionWorkbook.setCourseId("");
        questionWorkbook.setCourseName(previousQuestion.getCourse());
        questionWorkbook.setIsDiff(0);
        questionWorkbook.setNumber(previousNumberAndSortNumber.get("number") + 1);
        questionWorkbook.setPageIndex(request.getPageIndex());
        questionWorkbook.setParentQuestionId(newQuestion.getParentId());
        questionWorkbook.setQuestionId(newQuestion.getId());
        questionWorkbook.setSubBookId(book.getId());
        questionWorkbook.setUnitName(previousQuestion.getUnitName());
        questionWorkbook.setSortNumber(previousNumberAndSortNumber.get("sortNumber") + 1);
        questionWorkbookMapper.insert(questionWorkbook);
         
         //cc 更新关联表中的number字段，重新排序
        AijiaThreadPool.getInstance().submit(new SyncUpdateQuestionWorkbookNumber(book.getParentBookId(), this));
         //更新练习册为审核中
         kmtExerciseMapper.updateExerciseBookNotAuditing(book.getId());

         return responseEntity.success("成功");
    }

    /**
     * 更新number
     * @param bookId
     */
    @Override
    public void updateQuestionWorkBookNumber(String bookId) {
        List<QuestionWorkbookNumberSize> list = questionWorkbookMapper.findQuestionQuestionWorkBook(bookId);
        if (CollectionUtils.isNotEmpty(list)){
            list.forEach(e -> {
//                int sortNumber = e.getBookType() * 10000000 + e.getSize() * 1000 + e.getPageIndex();
                if (e.getNumber() != e.getSize()){
                    questionWorkbookMapper.updateNumberByBookIdAndQuestionId(e.getBookId(), e.getQuestionId(), e.getSize());
                }
            });
        }
    }

    /**
     * 获取上一道题的number, 如果是单题新增小题或新增大题， 上一个就是单题， 如果是大小题新增小题或新增大题，取上个题的小题
     * @param previousQuestion
     * @return
     */
    private Map<String, Integer> getPreviousNumber (KmtExerciseQuestion previousQuestion) {
        Map<String, Integer> previousNumberAndSortNumber = new HashMap<>();
        if (previousQuestion.getQuestionClassType() == 0){
            previousNumberAndSortNumber.put("number", previousQuestion.getNumber());
            previousNumberAndSortNumber.put("sortNumber", previousQuestion.getSortNumber());
        } else {
            Map<String, Integer> temp = kmtExerciseMapper.findMaxSmallQuestionNumber(previousQuestion.getId());
            if (temp == null){
                previousNumberAndSortNumber.put("number", previousQuestion.getNumber());
                previousNumberAndSortNumber.put("sortNumber", previousQuestion.getSortNumber());
            } else {
                previousNumberAndSortNumber = temp;
                previousNumberAndSortNumber.put("sortNumber", previousNumberAndSortNumber.get("sort_number"));
            }
        }
        return previousNumberAndSortNumber;
    }

    @Override
    public ResponseEntity addOrEditExerciseQuestion(String userId, AddExerciseQuestionRequest request) {
    	ResponseEntity responseEntity = new ResponseEntity();
        if (!isManagerSubmit(userId)){
        	 return responseEntity.failure("没有操作权限");
        }
        if (Objects.equals(1, request.getType()) || StringUtils.isBlank(request.getQuestionId())){
        	return addExerciseQuestionNew(userId, request);
        } else {
        	return editExerciseQuestionNew(userId, request);
        }
    }

    /**
     * 删除小题
     * @param questionId
     * @return
     */
    private ResponseEntity deleteOneQuestionsNew(String questionId){
    	ResponseEntity responseEntity = new ResponseEntity();
        KmtExerciseQuestion kmtExerciseQuestion = kmtExerciseQuestionMapper.selectById(questionId);

        if(kmtExerciseQuestion==null) {
        	return responseEntity.failure("题目不存在");
        }else if( kmtExerciseQuestion.getQuestionClassType() == 1){
        	//return responseEntity.failure("删除小题时，不可以删大题！");
        	return responseEntity.failure(ResponseConstant.CODE_000, "删除第一道小题时，请使用删除大题功能");
        }

        Integer parentChildQuestionCount = kmtExerciseQuestionMapper.countChildQuestions (kmtExerciseQuestion.getParentId());
        if (parentChildQuestionCount == null || parentChildQuestionCount == 0){
            //大题的小题数为空，更新大题的questionClassType为0
            kmtExerciseMapper.updateKmtExerciseQuestionClassTypeAndParentId(null, 0, kmtExerciseQuestion.getParentId());
            AijiaThreadPool.getInstance().submit(new SyncSaveExerciseQuestionsTask(kmtExerciseMapper,
                    exerciseQuestionSearchService, Collections.singletonList(kmtExerciseQuestion.getParentId())));
        }
        logger.info("删除={}", JSON.toJSONString(kmtExerciseQuestion));
        kmtExerciseQuestion.setDealStatus(6); //设置为删除状态
        kmtExerciseQuestionMapper.updateById(kmtExerciseQuestion);

        //cc 删除关联表
        List<String> bookIds = questionWorkbookMapper.queryBookIds(questionId);
        if(bookIds!=null && bookIds.size()>0) {
	        questionWorkbookMapper.deleteQuestionByQuestionIdAndBookId(questionId, null);
            logger.info("========4: {}", System.currentTimeMillis());

            for (String bookId : bookIds) {
		        //cc 更新关联表中的number字段，重新排序
                AijiaThreadPool.getInstance().submit(new SyncUpdateQuestionWorkbookNumber(bookId, this));
			}
            logger.info("========5: {}", System.currentTimeMillis());
        }
        AijiaThreadPool.getInstance().submit(new SyncSaveExerciseQuestionsTask(kmtExerciseMapper,
                    exerciseQuestionSearchService, Collections.singletonList(questionId)));
        
        return responseEntity.success();        
    }

    /**
     * 删除大题
     * @param questionId
     */
    private ResponseEntity deleteOneBigQuestionsNew(String questionId){
    	ResponseEntity responseEntity = new ResponseEntity();
        List<String> questionIds = kmtExerciseMapper.findQuestionIdByParentId(questionId);
        KmtExerciseQuestion kmtExerciseQuestion = kmtExerciseQuestionMapper.selectById(questionId);
        if(kmtExerciseQuestion==null) {
        	return responseEntity.failure("题目不存在");
        }else if( kmtExerciseQuestion.getQuestionClassType() == 2){
        	return responseEntity.failure("删除大题时，不可以传入小题的ID");
        }

        if (CollectionUtils.isNotEmpty(questionIds)){
            kmtExerciseQuestionMapper.updateQuestionDealStatus(questionIds, 6); // 设置为删除状态
            //cc 删除关联表
            List<String> bookIds = questionWorkbookMapper.queryBookIdsByQuestionIds(questionIds);

            if(bookIds!=null && bookIds.size()>0) {
	            questionWorkbookMapper.deleteQuestions( null, questionIds);
		        for (String bookId : bookIds) {
			        //cc 更新关联表中的number字段，重新排序
                    AijiaThreadPool.getInstance().submit(new SyncUpdateQuestionWorkbookNumber(bookId, this));
				}
            }
            AijiaThreadPool.getInstance().submit(new SyncSaveExerciseQuestionsTask(kmtExerciseMapper, exerciseQuestionSearchService, questionIds));
        }        
        return responseEntity.success();
    }

    // 移动小题，小题只能在大题里面移动，不可以移到大题的外面
    private ResponseEntity moveSmallQuestionNew(ExerciseQuestionDealRequest request,KmtExerciseQuestion questionWorkbook){
    	ResponseEntity responseEntity = new ResponseEntity();
    	String parentQuestionId = questionWorkbook.getParentId();
    	String bookId = questionWorkbookMapper.queryBookId(questionWorkbook.getId(), questionWorkbook.getExerciseBookId());
    	List<Integer> sortNumbers = questionWorkbookMapper.querySortNumberByParentQuestionId(parentQuestionId,bookId);

    	int size = sortNumbers.size();
    	if(size==1) {
    		return responseEntity.failure(ResponseConstant.CODE_000, "该大题下只有一道小题，不需要移动");
    	}

    	Integer currSortNumber = questionWorkbook.getSortNumber();

    	if(request.getType()==1) {//上移
	    	if(sortNumbers.get(0) >= currSortNumber) {
	    		 return responseEntity.failure(ResponseConstant.CODE_000, "已移动到第一道小题的位置");
	    	}

	    	//找到下一个题目的sortNumber
    		for(int i=size-1;i>0;i--) {
    			int tmpSortNumber = sortNumbers.get(i);
    			if(currSortNumber>tmpSortNumber) {
    				currSortNumber = tmpSortNumber;
    				break;
    			}else if(currSortNumber==tmpSortNumber) {
    				currSortNumber = sortNumbers.get(i-1);
    				break;
    			}
    		}
    		currSortNumber--;//加一，当前题目排到下一个题目的后面，实际上就是交换了位置
    	}else {// 下移
    		if(sortNumbers.get(size-1) <= currSortNumber) {
    	         return responseEntity.failure(ResponseConstant.CODE_000, "已移动到最后一道小题的位置");
	    	}

	    	//找到下一个题目的sortNumber
    		for(int i=0;i<size-1;i++) {
    			int tmpSortNumber = sortNumbers.get(i);
    			if(currSortNumber<tmpSortNumber) {
    				currSortNumber = tmpSortNumber;
    				break;
    			}else if(currSortNumber==tmpSortNumber) {
    				currSortNumber	= sortNumbers.get(i+1);
    				break;
    			}
    		}
    		currSortNumber++;//加一，当前题目排到下一个题目的后面，实际上就是交换了位置
    	}

    	// 更新sortNumber顺序
		questionWorkbookMapper.updateSortNumberByQuestionId(questionWorkbook.getId(), questionWorkbook.getExerciseBookId(), currSortNumber);

		//cc 更新关联表中的number字段，重新排序
        this.updateQuestionWorkBookNumber(bookId);
		return responseEntity.success();
    }

    // 移动大题，大题只能整体一起移动
    private ResponseEntity moveBigQuestionNew(ExerciseQuestionDealRequest request,KmtExerciseQuestion questionWorkbook){
    	ResponseEntity responseEntity = new ResponseEntity();
    	String parentQuestionId = request.getQuestionId();
    	String bookId = questionWorkbookMapper.queryBookId(questionWorkbook.getId(), questionWorkbook.getExerciseBookId());
    	QuestionWorkbook bigQuestionWorkbook = questionWorkbookMapper.queryQuestionWorkbook(request.getQuestionId(), bookId);

    	List<QuestionWorkbook> sortNumbers = questionWorkbookMapper.queryBigQuestionSortNumber(questionWorkbook.getUnitName(), questionWorkbook.getCourse(), bookId);

    	int size = sortNumbers.size();
    	if(size==1) {
    		return responseEntity.failure(ResponseConstant.CODE_000, "该课时下只有一道大题，不需要移动");
    	}

    	Integer startSortNumber = questionWorkbook.getSortNumber();

    	//查询该大题下面的所有小题
    	List<QuestionWorkbook> smallQuestions  = questionWorkbookMapper.querySmallQuestionByParentQuestionId(parentQuestionId, bookId);

    	if(request.getType()==1) {//上移
	    	if(sortNumbers.get(0).getSortNumber() >= questionWorkbook.getSortNumber()) {
	    		 return responseEntity.failure(ResponseConstant.CODE_000, "已移动到第一道大题的位置");
	    	}

    		QuestionWorkbook nextQuestionWorkbook=null;
    		for(int i=size-1;i>0;i--) {
    			QuestionWorkbook tmpQuestionWorkbook = sortNumbers.get(i);
    			if(questionWorkbook.getSortNumber()>tmpQuestionWorkbook.getSortNumber()) {
    				nextQuestionWorkbook = tmpQuestionWorkbook;
    			}else if(questionWorkbook.getSortNumber()==tmpQuestionWorkbook.getSortNumber()) {
    					nextQuestionWorkbook = sortNumbers.get(i-1);
    			}
    		}

    		if(nextQuestionWorkbook!=null) {
		    	startSortNumber = nextQuestionWorkbook.getSortNumber()-1-smallQuestions.size();
		    	bigQuestionWorkbook.setSortNumber(startSortNumber++);
		    	questionWorkbookMapper.updateById(bigQuestionWorkbook);
		    	for (QuestionWorkbook smallQuestionWorkbook : smallQuestions) {
					smallQuestionWorkbook.setSortNumber(startSortNumber++);
					questionWorkbookMapper.updateById(smallQuestionWorkbook);
				}
    		}
    	}else {// 下移
    		if(sortNumbers.get(size-1).getSortNumber() <= questionWorkbook.getSortNumber()) {
    	         return responseEntity.failure(ResponseConstant.CODE_000, "已移动到最后一道大题的位置");
	    	}

    		QuestionWorkbook nextQuestionWorkbook=null;
    		for(int i=0;i<size-1;i++) {
    			QuestionWorkbook tmpQuestionWorkbook = sortNumbers.get(i);
    			if(questionWorkbook.getSortNumber()<tmpQuestionWorkbook.getSortNumber()) {
    				nextQuestionWorkbook = tmpQuestionWorkbook;
    				break;
    			}else if(questionWorkbook.getSortNumber()==tmpQuestionWorkbook.getSortNumber()) {
    				nextQuestionWorkbook = sortNumbers.get(i+1);
    				break;
    			}
    		}

    		if(nextQuestionWorkbook!=null) {
    			List<QuestionWorkbook> nextSmallQuestions = questionWorkbookMapper.querySmallQuestionByParentQuestionId(nextQuestionWorkbook.getQuestionId(), bookId);
    			int nextSize = nextQuestionWorkbook.getSortNumber();
    			if(nextSmallQuestions!=null && nextSmallQuestions.size()>0) {
    				nextSize = nextSmallQuestions.get(nextSmallQuestions.size()-1).getSortNumber();
    			}
	    		startSortNumber = nextSize+1;
	    		bigQuestionWorkbook.setSortNumber(startSortNumber++);
		    	questionWorkbookMapper.updateById(bigQuestionWorkbook);
	        	for (QuestionWorkbook smallQuestionWorkbook : smallQuestions) {
	    			smallQuestionWorkbook.setSortNumber(startSortNumber++);
	    			questionWorkbookMapper.updateById(smallQuestionWorkbook);
	    		}
    		}
    	}

        //cc 更新关联表中的number字段，重新排序
        this.updateQuestionWorkBookNumber(bookId);
		return responseEntity.success();
    }

    private ResponseEntity moveSmallQuestionOld(KmtExerciseQuestion kmtExerciseQuestion,ExerciseQuestionDealRequest request) {
    	ResponseEntity responseEntity = new ResponseEntity();
    	 //如果移动到最开始或最末尾， 提示不能再移动
        String parentId = null;
        if (StringUtil.isBlank(kmtExerciseQuestion.getParentId())){
            parentId = kmtExerciseQuestion.getId();
        } else {
            parentId = kmtExerciseQuestion.getParentId();
        }
        List<KmtExerciseQuestion> tempList = kmtExerciseMapper.findQuByParentId(parentId);
        //移动小题
        if (request.getType() == 1){
            //第一个小题不能往上移
            if (tempList != null && tempList.size() > 0){
                if (kmtExerciseQuestion.getId().equals(tempList.get(0).getId())){
                    return  responseEntity.failure(ResponseConstant.CODE_000, "已移动到第一小题");
                }
                //如果是第二个小题上移， 修改题目的父子关系
                if (kmtExerciseQuestion.getId().equals(tempList.get(1).getId())){
                    //先特殊修改第一题和第二题， 第三题开始修改父题
                    kmtExerciseMapper.updateParentIdAndQuestionClassTypeAndNumById(null,
                            1, kmtExerciseQuestion.getNum() - 1, kmtExerciseQuestion.getId());
                    kmtExerciseMapper.updateParentIdAndQuestionClassTypeAndNumById(kmtExerciseQuestion.getId(),
                            2, kmtExerciseQuestion.getNum(), tempList.get(0).getId());
                    for (int i =2; i < tempList.size(); i++){
                        kmtExerciseMapper.updateParentIdById(kmtExerciseQuestion.getId(), tempList.get(i).getId());
                    }
                } else {
                    //上移
                    kmtExerciseMapper.updateByBookIdAndNum(kmtExerciseQuestion.getBookId(), kmtExerciseQuestion.getNum(),kmtExerciseQuestion.getNum() - 1);
                    kmtExerciseMapper.updateQuestionNumByQuestionId(kmtExerciseQuestion.getId(), kmtExerciseQuestion.getNum() - 1);
                }
            }
        } else if (request.getType() == 2){
            //最后一个小题不能往下移
            if (tempList != null && tempList.size() > 0){
                if (kmtExerciseQuestion.getId().equals(tempList.get(tempList.size() - 1).getId())){
                    return responseEntity.failure(ResponseConstant.CODE_000, "已移动到最后一个小题的位置");
                }
                //如果是第一小题的下移， 修改题目父子关系
                if (kmtExerciseQuestion.getId().equals(tempList.get(0).getId())){
                    //先特殊修改第一题和第二题， 第三题开始修改父题
                    kmtExerciseMapper.updateParentIdAndQuestionClassTypeAndNumById(tempList.get(1).getId(),
                            2, kmtExerciseQuestion.getNum() + 1, kmtExerciseQuestion.getId());
                    kmtExerciseMapper.updateParentIdAndQuestionClassTypeAndNumById(null,
                            1, kmtExerciseQuestion.getNum(), tempList.get(1).getId());
                    for (int i =2; i < tempList.size(); i++){
                        kmtExerciseMapper.updateParentIdById(tempList.get(1).getId(), tempList.get(i).getId());
                    }
                } else {
                    //下移
                    kmtExerciseMapper.updateByBookIdAndNum(kmtExerciseQuestion.getBookId(), kmtExerciseQuestion.getNum(), kmtExerciseQuestion.getNum() + 1);
                    kmtExerciseMapper.updateQuestionNumByQuestionId(kmtExerciseQuestion.getId(), kmtExerciseQuestion.getNum() + 1);
                }
            }
        }

        return responseEntity.success();
    }

    private ResponseEntity moveBigQuestionOld(KmtExerciseQuestion kmtExerciseQuestion,ExerciseQuestionDealRequest request) {
    	ResponseEntity responseEntity = new ResponseEntity();
    	if (request.getType() == 1){
             //大题上移, 先查找上一题的题目id列表
             KmtExerciseQuestion previousQuestion = kmtExerciseMapper.findPreviousByBookIdAndNum(kmtExerciseQuestion.getBookId(), kmtExerciseQuestion.getNum() - 1);
             if (previousQuestion ==null ||  !previousQuestion.getBookCourseId().equals(kmtExerciseQuestion.getBookCourseId())){
                 return responseEntity.failure(ResponseConstant.CODE_000, "已移动到第一题位置");
             }

             List<KmtExerciseQuestion> previousQuestions = kmtExerciseMapper.findQuestionsByBookCourseIdAndBigIndex(previousQuestion.getBookCourseId(), previousQuestion.getBigQuestionIndex());
             List<KmtExerciseQuestion> thisQuestions = kmtExerciseMapper.findQuestionsByBookCourseIdAndBigIndex(kmtExerciseQuestion.getBookCourseId(), kmtExerciseQuestion.getBigQuestionIndex());
             if (previousQuestions != null && thisQuestions != null){
                 int preSize = previousQuestions.size();
                 int thisSize = thisQuestions.size();
                 List<String> previousIds = previousQuestions.stream().map(KmtExerciseQuestion::getId).collect(Collectors.toList());
                 List<String> thisIds = thisQuestions.stream().map(KmtExerciseQuestion::getId).collect(Collectors.toList());
                 kmtExerciseMapper.updateQuestionMinusNumByQuestionIds(thisIds, preSize);
                 kmtExerciseMapper.updateQuestionAddNumByQuestionIds(previousIds, thisSize);
             }
         } else if (request.getType() == 2){
             List<KmtExerciseQuestion> thisQuestions = kmtExerciseMapper.findQuestionsByBookCourseIdAndBigIndex(kmtExerciseQuestion.getBookCourseId(), kmtExerciseQuestion.getBigQuestionIndex());
             int thisSize = thisQuestions.size();
             KmtExerciseQuestion nextQuestion = kmtExerciseMapper.findNextByBookIdAndNum(kmtExerciseQuestion.getBookId(), kmtExerciseQuestion.getNum() + thisSize);
             if (nextQuestion == null || !nextQuestion.getBookCourseId().equals(kmtExerciseQuestion.getBookCourseId())){
                 return responseEntity.failure(ResponseConstant.CODE_000, "已移动到最后一题位置");
             }
             List<KmtExerciseQuestion> nextQuestions = kmtExerciseMapper.findQuestionsByBookCourseIdAndBigIndex(nextQuestion.getBookCourseId(), nextQuestion.getBigQuestionIndex());
             if (nextQuestions != null && thisQuestions != null){
                 int nextSize = nextQuestions.size();
                 List<String> nextIds = nextQuestions.stream().map(KmtExerciseQuestion::getId).collect(Collectors.toList());
                 List<String> thisIds = thisQuestions.stream().map(KmtExerciseQuestion::getId).collect(Collectors.toList());
                 kmtExerciseMapper.updateQuestionMinusNumByQuestionIds(nextIds, thisSize);
                 kmtExerciseMapper.updateQuestionAddNumByQuestionIds(thisIds, nextSize);
             }
         }

    	return responseEntity.success();
    }

    @Override
    public ResponseEntity moveExerciseQuestion(String userId, ExerciseQuestionDealRequest request) {
        ResponseEntity responseEntity = new ResponseEntity();
        if (!isManagerSubmit(userId)){
            return responseEntity.failure("没有操作权限");
        }

        KmtExerciseQuestion kmtExerciseQuestion  = kmtExerciseQuestionMapper.selectById(request.getQuestionId());
        if(kmtExerciseQuestion ==null) {
        	return responseEntity.failure("题目不存在！");
        }

        if (request.getQuestionClassType() == 2 && (kmtExerciseQuestion.getQuestionClassType() == 0 || kmtExerciseQuestion.getQuestionClassType() == 1)){
            //如果是单题，移动一个小题不能移动
            return responseEntity.failure(ResponseConstant.CODE_000, "单题或者大题不能移动小题，请使用移动大题功能");
        }

        String subBookId = kmtExerciseQuestion.getExerciseBookId();
        String questionId = request.getQuestionId();

        KmtExerciseQuestion questionWorkbook  = kmtExerciseQuestionMapper.findPreviousById(subBookId, questionId);
        questionWorkbook.setExerciseBookId(subBookId);

        if (request.getQuestionClassType() == 2){
     	   responseEntity = this.moveSmallQuestionNew(request, questionWorkbook);
//           if(responseEntity.getCode().equals("999")) {
//        	   responseEntity = this.moveSmallQuestionOld(kmtExerciseQuestion, request);
//        	   ThreadPool.getInstance().addTask(new SyncSaveExerciseBookCourseTask(kmtExerciseMapper, exerciseQuestionSearchService, kmtExerciseQuestion.getBookCourseId()));
//           }
           return responseEntity;
        }
        //} else  if (request.getQuestionClassType() == 1){
        responseEntity = this.moveBigQuestionNew(request, questionWorkbook);
        //if(responseEntity.getCode().equals("999")) {
        	//responseEntity = this.moveBigQuestionOld(kmtExerciseQuestion, request);
           	//ThreadPool.getInstance().addTask(new SyncSaveExerciseBookCourseTask(kmtExerciseMapper, exerciseQuestionSearchService, kmtExerciseQuestion.getBookCourseId()));
        //}
        return responseEntity;
    }

}
