package com.aijia.kmt.service.impl;

import com.aijia.core.utils.StringUtil;
import com.aijia.core.web.ResponseEntity;
import com.aijia.kmt.common.constant.KnowledgeVersion;
import com.aijia.kmt.dto.*;
import com.aijia.kmt.mapper.KmtExerciseMapper;
import com.aijia.kmt.mapper.LibManagerMapper;
import com.aijia.kmt.po.KmtExerciseQuestionRollback;
import com.aijia.kmt.po.user.TbLibManager;
import com.aijia.kmt.service.KmtExerciseQuestionListService;
import com.aijia.kmt.utils.JSONFileUtil;
import com.aijia.kmt.utils.PropertiesUtil;
import com.aijia.kmt.vo.ExerciseQuestionsRequest;
import com.aijia.train.bean.TrTrainPartUser;
import com.aijia.train.bean.TrTrainUserExercise;
import com.aijia.train.mapper.TrainUserExerciseMapper;
import com.aijia.train.mapper.TrainUserServiceMapper;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class KmtExerciseQuestionListServiceImpl implements KmtExerciseQuestionListService {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    @Autowired
    private KmtExerciseMapper kmtExerciseMapper;
    @Autowired
    private KnowledgeVersion knowledgeVersion;
    @Autowired
    private LibManagerMapper libManagerMapper;

    @Autowired
    private TrainUserExerciseMapper userExerciseMapper;

    @Autowired
    private TrainUserServiceMapper userServiceMapper;

    @Override
    public ResponseEntity findQuestionList (String userId, ExerciseQuestionsRequest request,int type) {
        TrTrainPartUser user = userServiceMapper.selectById(userId);
        String paramUserId = StringUtils.isBlank(request.getUserId())?userId:request.getUserId();

        TrTrainPartUser partUser = userServiceMapper.selectById(paramUserId);
        int roleType = partUser.getRoleType();
        //管理员
        if(partUser.getManagerType() == 1){
            roleType = 0;
        }
        ResponseEntity responseEntity = new ResponseEntity();
        Map<String, Object> data = new HashMap<>();
        Page page = new Page(request.getPageNo(), request.getPageSize());

            if (StringUtil.isNotBlank(request.getQuestionId())){
                //搜索题目id
                page = new Page(1, 10, 1);
            }
        List<ExerciseQuetionDto> questions  = kmtExerciseMapper.findQuestionByBookCourseIdAndUserIdNew(request, paramUserId, page,user.getManagerType(), type,roleType);
        //管理员看到的题目列表是两层级， 老师看到的列表是一层级
        //默认两层级的结构
        List<QuestionListV1Dto> list = changeDbQuestion(questions,true,paramUserId,type);
        data.put("list", list);
        data.put("pageNo", page.getCurrent());
        data.put("pageSize", page.getSize());
        data.put("pages", page.getPages());
        data.put("total", page.getTotal());
        responseEntity.success(data, "查询成功");
        return responseEntity;
    }

    /**
     * 把数据库结构的题目转成前端反参的形式
     * @param questions
     */
    private List<QuestionListV1Dto> changeDbQuestion (List<ExerciseQuetionDto> questions, boolean isTwoLevel,String userId,int type) {
        List<QuestionListV1Dto> list = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(questions)){
            this.appendQuestionDtoDataFromJSONFile(questions,userId);
            for (ExerciseQuetionDto q : questions){
                if (StringUtil.isNotBlank(q.getQuestionImage())){
                    q.setQuestion("<img src='" + PropertiesUtil.getValue("STATIC_FILE_URL") + q.getQuestionImage() + "'/>");
                }
            }
            list = this.getQuestionGroupList(questions, isTwoLevel,userId,type);
        }
        return list;
    }

    public void appendQuestionDtoDataFromJSONFile(List<ExerciseQuetionDto> kmtExerciseQuestions,String userId) {
        if (CollectionUtils.isNotEmpty(kmtExerciseQuestions)){

            for (ExerciseQuetionDto kmtExerciseQuestion : kmtExerciseQuestions){
                boolean check = false;
                //读取大字段文件
                TrTrainUserExercise userExercise = userExerciseMapper.selectOne(
                        new QueryWrapper<TrTrainUserExercise>().eq("user_id",userId)
                        .eq("question_id",kmtExerciseQuestion.getQuestionId())
                );
                if(Objects.nonNull(userExercise)){
                    check = true;
                }
                if (StringUtil.isNotBlank(kmtExerciseQuestion.getQuestionPath())){
                    String path = PropertiesUtil.getValue("EXERCISE_JSON_FILE_PATH") + kmtExerciseQuestion.getQuestionPath();
                    logger.debug("文件路径:{}", path);
                    String content = JSONFileUtil.readJSONFile(path);
                    logger.debug("文件内容：{}", content);
                    if (StringUtils.isNotBlank(content)){
                        JSONObject jsonObject = JSONObject.parseObject(content);
                        kmtExerciseQuestion.setQuestion(jsonObject.getString("question"));
                        kmtExerciseQuestion.setQuestionText(check && StringUtil.isNotBlank(userExercise.getQuestionText())?userExercise.getQuestionText():jsonObject.getString("question_text"));
                        kmtExerciseQuestion.setAnswer(check && StringUtil.isNotBlank(userExercise.getAnswer())?userExercise.getAnswer():jsonObject.getString("option_answer"));
                        kmtExerciseQuestion.setAnswerJson(check && StringUtil.isNotBlank(userExercise.getAnswerJson())?userExercise.getAnswerJson():jsonObject.getString("answer_json"));
//                        kmtExerciseQuestion.setQuestionAnalysis(check && StringUtil.isNotBlank(userExercise.getAnalysis())?userExercise.getAnalysis():jsonObject.getString("analysis"));
                        kmtExerciseQuestion.setAnalysisJson(check && StringUtil.isNotBlank(userExercise.getAnalysisJson())?userExercise.getAnalysisJson():"");
                        kmtExerciseQuestion.setOriginAnswer(check && StringUtil.isNotBlank(userExercise.getAnswerText())?userExercise.getAnswerText():jsonObject.getString("question_answer"));
                    }else{
                        kmtExerciseQuestion.setQuestion(check && StringUtil.isNotBlank(userExercise.getQuestion())?userExercise.getQuestion():"");
                        kmtExerciseQuestion.setQuestionText(check && StringUtil.isNotBlank(userExercise.getQuestionText())?userExercise.getQuestionText():"");
                        kmtExerciseQuestion.setAnswer(check && StringUtil.isNotBlank(userExercise.getAnswer())?userExercise.getAnswer():"");
                        kmtExerciseQuestion.setAnswerJson(check && StringUtil.isNotBlank(userExercise.getAnswerJson())?userExercise.getAnswerJson():"");
//                        kmtExerciseQuestion.setQuestionAnalysis(check && StringUtil.isNotBlank(userExercise.getAnalysis())?userExercise.getAnalysis():"");
                        kmtExerciseQuestion.setAnalysisJson(check && StringUtil.isNotBlank(userExercise.getAnalysisJson())?userExercise.getAnalysisJson():"");
                        kmtExerciseQuestion.setOriginAnswer(check && StringUtil.isNotBlank(userExercise.getAnswerText())?userExercise.getAnswerText():"");
                    }
                }
            }
        }
    }

    public void appendQuestionDtoDataFromJSONFile(List<ExerciseQuetionDto> kmtExerciseQuestions) {
        if (CollectionUtils.isNotEmpty(kmtExerciseQuestions)){
            for (ExerciseQuetionDto kmtExerciseQuestion : kmtExerciseQuestions){
                //读取大字段文件
                if (StringUtil.isNotBlank(kmtExerciseQuestion.getQuestionPath())){
                    String path = PropertiesUtil.getValue("EXERCISE_JSON_FILE_PATH") + kmtExerciseQuestion.getQuestionPath();
                    logger.debug("文件路径:{}", path);
                    String content = JSONFileUtil.readJSONFile(path);
                    logger.debug("文件内容：{}", content);
                    if (StringUtils.isNotBlank(content)){
                        JSONObject jsonObject = JSONObject.parseObject(content);
                        kmtExerciseQuestion.setQuestion(jsonObject.getString("question"));
                        kmtExerciseQuestion.setQuestionText(jsonObject.getString("question_text"));
                        kmtExerciseQuestion.setAnswer(jsonObject.getString("option_answer"));
                        kmtExerciseQuestion.setAnswerJson(jsonObject.getString("answer_json"));
                        kmtExerciseQuestion.setQuestionAnalysis(jsonObject.getString("analysis"));
                        kmtExerciseQuestion.setAnalysisJson(jsonObject.getString("analysis_json"));;
                        kmtExerciseQuestion.setOriginAnswer(jsonObject.getString("question_answer"));
                    }
                }
            }
        }
    }

    /**
     *
     * @param list 题目列表
     * @param isTwoLevel 是否是两层级
     * @return
     */
    public List<QuestionListV1Dto> getQuestionGroupList (List<ExerciseQuetionDto> list, boolean isTwoLevel){
        List<QuestionListV1Dto> result = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(list)){
            List<String> questionIds = list.stream().map(ExerciseQuetionDto::getQuestionId).collect(Collectors.toList());
            List<QuestionListKnowModel> questionKnows = kmtExerciseMapper.findQuestionKnowByQuestionIds(questionIds,null,0);
            List<TbLibManager> users = getDealUsers(list);

            list.forEach(temp -> {
                QuestionListV1Dto tar = changeExerciseDto2QuestionListDto(temp, questionKnows, users,null);
                if (isTwoLevel && StringUtil.isNotBlank(temp.getParentId())){
                    addTar2Result(tar, result, temp.getParentId(), temp.getExerciseBookId(),null,0);
                } else {
                    result.add(tar);
                }
            });
        }
        return  result;
    }
    /**
     *
     * @param list 题目列表
     * @param isTwoLevel 是否是两层级
     * @return
     */
    public List<QuestionListV1Dto> getQuestionGroupList (List<ExerciseQuetionDto> list, boolean isTwoLevel,String userId,int type){
        List<QuestionListV1Dto> result = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(list)){
            List<String> questionIds = list.stream().map(ExerciseQuetionDto::getQuestionId).collect(Collectors.toList());

            TrTrainPartUser user = userServiceMapper.selectById(userId);
            //已提交
            List<QuestionListKnowModel> questionKnows = null;
            if(type == 2){
                questionKnows = kmtExerciseMapper.findQuestionKnowByQuestionIds2(questionIds);
            }else{
                if(user.getStudyStatus()>3){
                    questionKnows = kmtExerciseMapper.findQuestionKnowByQuestionIds(questionIds,userId,type);
                }else{
                    //查询用户提交记录
                    List<TrTrainUserExercise> userExerciseList = userExerciseMapper.selectList(
                            new QueryWrapper<TrTrainUserExercise>().eq("user_id",userId)
                    );
                    List<String> postList = new ArrayList<>();
                    List<String> copyList = new ArrayList<>();
                    copyList.addAll(questionIds);
                    questionIds.forEach(s->{
                        userExerciseList.forEach(ue->{
                            if(ue.getQuestionId().equals(s) && ue.getStatus()>0){
                                postList.add(s);
                                copyList.remove(s);
                            }
                        });
                    });
                    if(postList.size()>0){
                        questionKnows = kmtExerciseMapper.findQuestionKnowByQuestionIds(postList,userId,type);
                    }
                    if(questionKnows == null || questionKnows.size()==0){
                        questionKnows = new ArrayList<>();
                    }
                    if(user.getRoleType()>1){
                        questionKnows.addAll(kmtExerciseMapper.findQuestionKnowByQuestionIds2(copyList));
                    }
                }
            }

            List<TbLibManager> users = getDealUsers(list);

            List<QuestionListKnowModel> finalQuestionKnows = questionKnows;
            list.forEach(temp -> {
                QuestionListV1Dto tar = changeExerciseDto2QuestionListDto(temp, finalQuestionKnows, users,userId);
                if (isTwoLevel && StringUtil.isNotBlank(temp.getParentId())){
                    addTar2Result(tar, result, temp.getParentId(), temp.getExerciseBookId(),userId,type);
                } else {
                    result.add(tar);
                }
            });
        }
        return  result;
    }

    /**
     * 把exerciseDto 转换成QuestionListV1Dto
     * @param exerciseDto
     * @param questionKnows
     * @param users
     * @return
     */
    private QuestionListV1Dto changeExerciseDto2QuestionListDto (ExerciseQuetionDto exerciseDto, List<QuestionListKnowModel> questionKnows,
                                                                 List<TbLibManager> users,String userId) {
        QuestionListV1Dto tar = new QuestionListV1Dto();
//            TrTrainPartUser user = userServiceMapper.selectById(userId);
            //设置知识点
            setQuestionKnowledge(tar, questionKnows.stream().filter(map ->
                    exerciseDto.getQuestionId().equals(map.getQuestionId())).collect(Collectors.toList()));
        //设置打回人和被打回人
//        setFromUserAndToUser(tar, exerciseDto);
        //设置加工审核人
//        appendQuestionDealUser(users, tar, exerciseDto);
        //状态
        tar.setQuestionStatus(this.getQuestionStatus(exerciseDto.getQuestionStatus()));
        //设置一致的字段
        setEqualField(tar, exerciseDto);
        return tar;
    }

    private void addTar2Result (QuestionListV1Dto tar, List<QuestionListV1Dto> result, String parentId, String bookId,String userId,int type){
        if (StringUtil.isNotBlank(parentId)){
            boolean exist = false;
            for (QuestionListV1Dto questionListV1Dto : result) {
                if (parentId.equals(questionListV1Dto.getQuestionId())){
                    questionListV1Dto.getChildQuestions().add(tar);
                    exist = true;
                    break;
                }
            }

            //如果大题不存在， 新增空大题（只有题目id）， 可能大题已经被加工
            if (!exist){
                QuestionListV1Dto parent = getParentQuestionV1Dto(parentId, bookId,userId,type);
                parent.setIsQuestionCanDeal(1); //大题不可加工
                parent.getChildQuestions().add(tar);
                result.add(parent);
            }
        }
    }

    /**
     * 查询父题dto
     * @return
     */
    private QuestionListV1Dto getParentQuestionV1Dto (String parentId, String bookId,String userId,int type) {
        QuestionListV1Dto parent = new QuestionListV1Dto();
        if (StringUtil.isNotBlank(parentId)){
            String temp = kmtExerciseMapper.findDifferentExerciseBookId(parentId);
            List<ExerciseQuetionDto> questions = kmtExerciseMapper.findExerciseQuestionByQuestionIdAndBookId(parentId, bookId,userId);
            if (CollectionUtils.isNotEmpty(questions)){
                if (StringUtil.isNotBlank(temp)){
                    for (ExerciseQuetionDto q : questions){
                        q.setIsCommonFlag(q.getIsCommonFlag() == 0 ? 1 : 0);
                    }
                }
                List<QuestionListKnowModel> questionKnows = kmtExerciseMapper.findQuestionKnowByQuestionIds(Collections.singletonList(parentId),userId,type);
                List<TbLibManager> users = getDealUsers(questions);
                this.appendQuestionDtoDataFromJSONFile(questions,userId);
                for (ExerciseQuetionDto q : questions){
                    if (StringUtil.isNotBlank(q.getQuestionImage())){
                        q.setQuestion("<img src='" + PropertiesUtil.getValue("STATIC_FILE_URL") + q.getQuestionImage() + "'/>");
                    }
                }
                parent = changeExerciseDto2QuestionListDto(questions.get(0), questionKnows, users,userId);
            }
        }

        return parent;
    }

    /**
     * 设置一致的值
     * @param questionListV1Dto
     * @param origin
     */
    private void setEqualField (QuestionListV1Dto questionListV1Dto, ExerciseQuetionDto origin) {
        questionListV1Dto.setQuestionId(origin.getQuestionId());
        questionListV1Dto.setBookName(origin.getBookName());
        questionListV1Dto.setQuestionHtml(origin.getQuestion());
        questionListV1Dto.setAnswerHtml(origin.getAnswer());
        questionListV1Dto.setReason(origin.getReason());
        questionListV1Dto.setQuestionText(origin.getQuestionText());
        questionListV1Dto.setAnswerJson(origin.getAnswerJson());
        questionListV1Dto.setAnalysisHtml(origin.getQuestionAnalysis());
        questionListV1Dto.setAnalysisJson(origin.getAnalysisJson());
        questionListV1Dto.setNum(origin.getNum());
        questionListV1Dto.setPageIndex(origin.getPageIndex());
        questionListV1Dto.setQuestionImage(origin.getQuestionImage());
        questionListV1Dto.setIsCommonFlag(origin.getIsCommonFlag());
        questionListV1Dto.setStartX(origin.getStartX());
        questionListV1Dto.setStartY(origin.getStartY());
        questionListV1Dto.setEndX(origin.getEndX());
        questionListV1Dto.setEndY(origin.getEndY());
        questionListV1Dto.setProofreadStatus(origin.getProofreadStatus());
        questionListV1Dto.setQuestionTypes(origin.getQuestionTypes());
        questionListV1Dto.setRealExamTypes(origin.getRealExamTypes());
        questionListV1Dto.setDifficultyLevels(origin.getDifficultyLevels());
        questionListV1Dto.setTrainingSkills(origin.getTrainingSkills());
        questionListV1Dto.setEnumerate(origin.getEnumerate());
        if (StringUtil.isBlank(origin.getParentId())){
            questionListV1Dto.setQuestionIndex(origin.getBigQuestionIndex());
            questionListV1Dto.setQuestionIndexDesc("大题: " +questionListV1Dto.getQuestionIndex());
        } else {
            questionListV1Dto.setQuestionIndex(origin.getSmallQuestionIndex());
            questionListV1Dto.setQuestionIndexDesc("小题: " +questionListV1Dto.getQuestionIndex());
        }
    }

    public int getQuestionStatus(int questionStatus) {
        if (questionStatus == 0 || questionStatus == 1){
            return 1;
        } else if (questionStatus == 3 || questionStatus == 7){
            return  3;
        } else if (questionStatus == 5 || questionStatus == 8) {
            return 8;
        } else if (questionStatus == 9 || questionStatus == 10) {
            return 10;
        } else {
            return questionStatus;
        }
    }

    /**
     * 设置加工审核人
     * @param users
     * @param questionListV1Dto
     * @param origin
     */
    public void appendQuestionDealUser (List<TbLibManager> users, QuestionListV1Dto questionListV1Dto, ExerciseQuetionDto origin){
        if (CollectionUtils.isNotEmpty(users)){
            for (TbLibManager manager : users){
                if (manager.getUserId().equals(origin.getHandleUserId()) && StringUtils.isNotBlank(manager.getName())){
                    questionListV1Dto.setDealUser(manager.getName());
                }
                if (manager.getUserId().equals(origin.getAuditingUserId()) && StringUtils.isNotBlank(manager.getName())){
                    questionListV1Dto.setAuditingUser(manager.getName());
                }
                if (manager.getUserId().equals(origin.getAnalysisUserId()) && StringUtils.isNotBlank(manager.getName())){
                    questionListV1Dto.setAnalysisDealUser(manager.getName());
                }
                if (manager.getUserId().equals(origin.getAnalysisAuditerId()) && StringUtils.isNotBlank(manager.getName())){
                    questionListV1Dto.setAnalysisAuditingUser(manager.getName());
                }
            }
        }
    }

    /**
     * 设置知识点列表
     * @param exerciseQuestionDto
     * @param questionKnowList
     */
    private void setQuestionKnowledge (QuestionListV1Dto exerciseQuestionDto, List<QuestionListKnowModel> questionKnowList){
        List<KnowHowDetailsDTO> knowHowDTOList = new ArrayList<>();
        if  (CollectionUtils.isNotEmpty(questionKnowList)){
            questionKnowList.forEach(map -> {
                if (map.getHowDetailsId() == null) return;
                boolean exists = false;
                //创建考点对象
                TestPointDTO point = new TestPointDTO(map.getHowDetailsKdId(), map.getKdName());
                //判断已有数据结构中是否
                for (KnowHowDetailsDTO knowHowDetailsDTO : knowHowDTOList) {
                    if (knowHowDetailsDTO.getId().equals(map.getHowDetailsId())) {
                        exists = true;
                        if (StringUtils.isNotBlank(map.getHowDetailsKdId())) {
                            knowHowDetailsDTO.addPoint(point);
                        }
                        break;
                    }
                }
                if (!exists) {
                    KnowHowDetailsDTO knowHowDetailsDTO = new KnowHowDetailsDTO(map.getHowDetailsId(), map.getDetailName());
                    knowHowDetailsDTO.setGrade(map.getGrade());
                    knowHowDetailsDTO.setSemester(map.getSemester());
                    knowHowDetailsDTO.setVersionName(knowledgeVersion.getVersionNameBySubjectAndVersionNum(map.getCourseName(), map.getVersionType()));
                    knowHowDetailsDTO.setHowName(map.getHowName());
                    if (StringUtils.isNotBlank(map.getHowDetailsKdId())) {
                        knowHowDetailsDTO.addPoint(point);
                    }
                    knowHowDTOList.add(knowHowDetailsDTO);
                }
            });
            exerciseQuestionDto.setKnowledges(knowHowDTOList);
        }
    }

    /**
     * 查询题目的加工审核员
     * @param list
     * @return
     */
    private List<TbLibManager> getDealUsers (List<ExerciseQuetionDto> list){
        List<String> userIds = this.findUserIdsInExerciseQuestionList(list);
        List<TbLibManager> users = null;
        if (CollectionUtils.isNotEmpty(userIds)){
            Integer[] types = {17, 18};
            users = libManagerMapper.findTbLibManagerByUserIdsAndType(userIds, Arrays.asList(types));
        }
        return users;
    }

    /**
     * 从题目列表中提取四个用户的id
     * @param list
     * @return
     */
    private List<String> findUserIdsInExerciseQuestionList (List<ExerciseQuetionDto> list){
        Set<String> result = new HashSet<>();
        if (list != null) {
            list.stream().forEach(item -> {
                if (StringUtils.isNotBlank(item.getHandleUserId())) {
                    result.add(item.getHandleUserId());
                }
                if (StringUtils.isNotBlank(item.getAuditingUserId())) {
                    result.add(item.getAuditingUserId());
                }
                if (StringUtils.isNotBlank(item.getAnalysisUserId())) {
                    result.add(item.getAnalysisUserId());
                }
                if (StringUtils.isNotBlank(item.getAnalysisAuditerId())) {
                    result.add(item.getAnalysisAuditerId());
                }
                if (StringUtils.isNotBlank(item.getFromUserId())) {
                    result.add(item.getFromUserId());
                }
                if (StringUtils.isNotBlank(item.getToUserId()) && !"1".equals(item.getToUserId())) {
                    result.add(item.getToUserId());
                }
            });
        }
        return new ArrayList<>(result);
    }

    /**
     * 类型获取名称
     * @param type
     * @return
     */
    private String getAuthorityByType (int type){
        String result = "管理员";
        if (type == 9){
            result = "";
        } else {
            switch (type){
                case 1: result = "题目加工人";break;
                case 2: result = "题目审核人";break;
                case 3: result = "解析加工人";break;
                case 4: result = "解析审核人";break;
                case 5: result = "题库迁移人";break;
                case 6: result = "题库审核人";break;
                default: result = "管理员"; break;
            }
        }
        return result;
    }

    /**
     * 设置打回人和被打回人
     * @param questionListV1Dto
     * @param origin
     */
    private void setFromUserAndToUser (QuestionListV1Dto questionListV1Dto, ExerciseQuetionDto origin) {
        //有打回理由查询理由的最新记录
        if (StringUtils.isNotBlank(origin.getReason())){
            KmtExerciseQuestionRollback rollback = kmtExerciseMapper.findLastExerciseRollbackByQuestionId(origin.getQuestionId());
            if (rollback != null){
                String fromUserId=rollback.getUserId();
                TbLibManager tbLibManager=kmtExerciseMapper.findByUserIdAndTypeIn(fromUserId);
                if(null != tbLibManager){
                    questionListV1Dto.setFromUser(getAuthorityByType(rollback.getFromUserType()) + tbLibManager.getName());
                }
                if(StringUtils.isBlank(rollback.getToUserId())){
                    questionListV1Dto.setToUser("AI");
                } else {
                    TbLibManager tbLibManager2=kmtExerciseMapper.findByUserIdAndTypeIn(rollback.getToUserId());
                    if (null != tbLibManager2){
                        questionListV1Dto.setToUser(getAuthorityByType(rollback.getToUserType()) + tbLibManager2.getName());
                    }
                }
            }
        }
    }
}
