package com.aijia.kmt.service.impl;

import com.aijia.core.utils.Constant;
import com.aijia.core.utils.StringUtil;
import com.aijia.core.web.ResponseEntity;
import com.aijia.kmt.common.constant.CourseConstant;
import com.aijia.kmt.common.constant.KnowledgeVersion;
import com.aijia.kmt.dto.*;
import com.aijia.kmt.dto.exercise.ExerciseQuestionRollbacks;
import com.aijia.kmt.dto.exercise.QuestionAttributeFilterDto;
import com.aijia.kmt.dto.exercise.QuestionSubjectAttribute;
import com.aijia.kmt.dto.knowledge.KnowledgeBaseDto;
import com.aijia.kmt.es.entiy.ElasticEntity;
import com.aijia.kmt.es.service.ExerciseQuestionSearchService;
import com.aijia.kmt.es.service.base.BaseSearchService;
import com.aijia.kmt.mapper.*;
import com.aijia.kmt.po.KmtExerciseBook;
import com.aijia.kmt.po.KmtExerciseBookCourse;
import com.aijia.kmt.po.KmtExerciseQuestion;
import com.aijia.kmt.po.KmtExerciseQuestionRollback;
import com.aijia.kmt.po.user.KmtSysUser;
import com.aijia.kmt.po.user.TbLibManager;
import com.aijia.kmt.service.KmtExerciseMapperService;
import com.aijia.kmt.service.KmtExerciseQuestionService;
import com.aijia.kmt.service.KmtExerciseService;
import com.aijia.kmt.utils.PropertiesUtil;
import com.aijia.kmt.utils.RedisUtil;
import com.aijia.kmt.vo.ExerciseQuestionsRequest;
import com.aijia.kmt.vo.ExerciseTeacherClassRequest;
import com.aijia.utils.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.script.Script;
import org.elasticsearch.script.ScriptType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class KmtExerciseQuestionServiceImpl implements KmtExerciseQuestionService {

    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    @Autowired
    private KmtExerciseMapperService kmtExerciseMapperService;
    @Autowired
    private KmtExerciseMapper kmtExerciseMapper;

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private ExerciseQuestionSearchService exerciseQuestionSearchService;
    @Autowired
    private KnowledgeVersion knowledgeVersion;
    @Autowired
    private BaseSearchService baseSearchService;
    @Autowired
    private LibManagerMapper libManagerMapper;
    @Autowired
    private KmtExerciseService kmtExerciseService;
    @Autowired
    private KmtSysUserMapper kmtSysUserMapper;
    @Autowired
    private KmtExerciseBookMapper kmtExerciseBookMapper;
    @Autowired
    private KmtExerciseQuestionMapper kmtExerciseQuestionMapper;

    @Override
    public ResponseEntity findExerciseQuestions(String userId, ExerciseQuestionsRequest request) {
        long beginTime = System.currentTimeMillis();
        ResponseEntity responseEntity = new ResponseEntity();
        Map<String, Object> data = new HashMap<>();
        List<ExerciseBookCourseClassDto> list = new ArrayList<>();
        com.baomidou.mybatisplus.extension.plugins.pagination.Page page = new com.baomidou.mybatisplus.extension.plugins.pagination.Page(request.getPageNo(), request.getPageSize());


        if (StringUtils.isNotBlank(request.getPeriodId())){
            List<ExerciseQuetionDto> questions = null;
            if (StringUtil.isNotBlank(request.getQuestionId())){
                String temp = kmtExerciseMapper.findDifferentExerciseBookId(request.getQuestionId().trim());
                questions = kmtExerciseMapper.findExerciseQuestionByQuestionId(request.getQuestionId().trim(), page);
                if (StringUtil.isNotBlank(temp)){
                    for (ExerciseQuetionDto q : questions){
                        q.setIsCommonFlag(q.getIsCommonFlag() == 0 ? 1 : 0);
                    }
                }
            } else if(request.getBookPartType() == 2) {
                //查询不同的题目
                String temp = kmtExerciseMapper.findDifferentExerciseBookId(request.getQuestionId().trim());
                questions = kmtExerciseMapper.findPartQuestionByBookCourseIdNew(request,page);
                if (StringUtil.isNotBlank(temp)){
                    for (ExerciseQuetionDto q : questions){
                        q.setIsCommonFlag(q.getIsCommonFlag() == 0 ? 1 : 0);
                    }
                }
            } else {
                String temp = kmtExerciseMapper.findDifferentExerciseBookIdByBookId(request.getBookId());
                questions = kmtExerciseMapperService.findQuestionByBookCourseIdAndUserId(request, page,  userId);
                if (StringUtil.isNotBlank(temp)){
                    for (ExerciseQuetionDto q : questions){
                        q.setIsCommonFlag(q.getIsCommonFlag() == 0 ? 1 : 0);
                    }
                }
            }
            kmtExerciseService.appendQuestionDtoDataFromJSONFile(questions);
            for (ExerciseQuetionDto q : questions){
                if (StringUtil.isNotBlank(q.getQuestionImage())){
                    q.setQuestion("<img src='" + PropertiesUtil.getValue("STATIC_FILE_URL") + q.getQuestionImage() + "'/>");
                }
            }
            list = this.getQuestionGroupList(questions, true,userId);
        }
        data.put("list", list);
        data.put("pageNo", page.getCurrent());
        data.put("pageSize", page.getSize());
        data.put("pages", page.getPages());
        data.put("total", page.getTotal());
        responseEntity.success(data, "查询成功");
        logger.info("耗时" + (System.currentTimeMillis() - beginTime));
        return responseEntity;
    }

    @Override
    public List<ExerciseBookCourseClassDto> getQuestionGroupList (List<ExerciseQuetionDto> list, boolean hasBigQuestionIndex,String userId){
        List<ExerciseBookCourseClassDto> result = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(list)){
            List<String> questionIds = list.stream().map(ExerciseQuetionDto::getQuestionId).collect(Collectors.toList());
            List<QuestionListKnowModel> questionKnows = kmtExerciseMapper.findQuestionKnowByQuestionIds(questionIds,userId,0);
            result = getCommonQuestionGroupList(list, questionKnows, hasBigQuestionIndex);
        }
        return  result;
    }

    @Override
    public List<ExerciseBookCourseClassDto> getSimplifyQuestionGroupList (List<ExerciseQuetionDto> list, boolean hasBigQuestionIndex){
        List<ExerciseBookCourseClassDto> result = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(list)){
            List<String> questionIds = list.stream().map(ExerciseQuetionDto::getQuestionId).collect(Collectors.toList());
            List<QuestionListKnowModel> questionKnows = kmtExerciseMapper.findQuestionKnowByQuestionIds(questionIds,null,0);
            List<TbLibManager> users = getDealUsers(list); //加工审核员用户信息
            for (ExerciseQuetionDto exerciseQuestionDto : list){
                this.setQuestionStatus(exerciseQuestionDto);
                exerciseQuestionDto.appendUserNameByIds(users);
                exerciseQuestionDto.setParentId(StringUtil.isBlank(exerciseQuestionDto.getParentId()) ?
                        exerciseQuestionDto.getQuestionId() : exerciseQuestionDto.getParentId());
                if (questionKnows != null){
                    this.setQuestionKnowledge(exerciseQuestionDto, questionKnows.stream().filter(map ->
                            exerciseQuestionDto.getQuestionId().equals(map.getQuestionId())).collect(Collectors.toList()));
                }
                if (hasBigQuestionIndex){
                    this.getBigQuestionClassList(result, exerciseQuestionDto).getList().add(exerciseQuestionDto);
                } else {
                    this.getBigQuestionClassListByParentId(result, exerciseQuestionDto).getList().add(exerciseQuestionDto);
                }
            }
            Collections.reverse(result);
        }
        return  result;
    }


    @Override
    public List<ExerciseBookCourseClassDto> getKmtQualityQuestionGroupList (List<ExerciseQuetionDto> list){
        List<ExerciseBookCourseClassDto> result = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(list)){
            List<String> questionIds = list.stream().map(ExerciseQuetionDto::getQuestionId).collect(Collectors.toList());
            List<QuestionListKnowModel> questionKnows = kmtExerciseMapper.findKmtQuestionKnowByQuestionIds(questionIds);
            result = getCommonQuestionGroupList(list, questionKnows, false);
        }
        return  result;
    }

    /**
     * 查询题目的加工审核员
     * @param list
     * @return
     */
    private List<TbLibManager> getDealUsers (List<ExerciseQuetionDto> list){
        List<String> userIds = this.findUserIdsInExerciseQuestionList(list);
        List<TbLibManager> users = null;
        if (CollectionUtils.isNotEmpty(userIds)){
            Integer[] types = {17, 18};
            users = libManagerMapper.findTbLibManagerByUserIdsAndType(userIds, Arrays.asList(types));
        }
        return users;
    }

    /**
     * 题目转成二级结构
     * @param list
     * @param questionKnows
     * @param hasBigQuestionIndex
     * @return
     */
    private List<ExerciseBookCourseClassDto> getCommonQuestionGroupList (List<ExerciseQuetionDto> list, List<QuestionListKnowModel> questionKnows, boolean hasBigQuestionIndex){
        List<ExerciseBookCourseClassDto> result = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(list)){
            List<TbLibManager> users = getDealUsers(list);
            for (ExerciseQuetionDto exerciseQuestionDto : list){
                this.setQuestionStatus(exerciseQuestionDto);
                exerciseQuestionDto.setParentId(StringUtil.isBlank(exerciseQuestionDto.getParentId()) ?
                        exerciseQuestionDto.getQuestionId() : exerciseQuestionDto.getParentId());
                exerciseQuestionDto.appendUserNameByIds(users);
                //如果用户记录没有，查询老的kmtSysUser表
                if (StringUtils.isBlank(exerciseQuestionDto.getDealUser()) && this.findUserIdsInExerciseQuestionList(list).size() > 0){
                    List<KmtSysUser> kmtSysUsers = kmtSysUserMapper.selectBatchIds(this.findUserIdsInExerciseQuestionList(list));
                    exerciseQuestionDto.appendKmtSysUserNameByIds(kmtSysUsers);
                }
                //有打回理由查询理由的最新记录
                if (StringUtils.isNotBlank(exerciseQuestionDto.getReason())){
                    KmtExerciseQuestionRollback rollback = kmtExerciseMapper.findLastExerciseRollbackByQuestionId(exerciseQuestionDto.getQuestionId());
                    if (rollback != null){
                        List<KmtSysUserAuthDto> authDtos = JSONArray.parseArray(PropertiesUtil.getValue("KMT_SYS_USER_AUTH"), KmtSysUserAuthDto.class);
                        String fromUserId=rollback.getUserId();
                        TbLibManager tbLibManager=kmtExerciseMapper.findByUserIdAndTypeIn(fromUserId);
                        if(null==tbLibManager){
                            //老用户
                            exerciseQuestionDto.setFromUser(this.getOldName(fromUserId,authDtos));
                        }else {
                            //新用户
                            exerciseQuestionDto.setFromUser(getAuthorityByType(rollback.getFromUserType()) + tbLibManager.getName());
                        }
                        if(StringUtils.isBlank(rollback.getToUserId())){
                            exerciseQuestionDto.setToUser("AI");
                        }else {
                            TbLibManager tbLibManager2=kmtExerciseMapper.findByUserIdAndTypeIn(rollback.getToUserId());
                            if(null==tbLibManager2){
                                //老
                                exerciseQuestionDto.setToUser(this.getOldName(rollback.getToUserId(),authDtos));
                            }else {
                                //新
//                                exerciseQuestionDto.setToUser(getAuthorityByType(rollback.getToUserType()) + tbLibManager.getName());
                                exerciseQuestionDto.setToUser(getAuthorityByType(rollback.getToUserType()) + tbLibManager2.getName());
                            }
                        }
                    }
                }

                //update_type=4表示是新录入的题， handle_user_id是录入员
                if ("4".equals(exerciseQuestionDto.getUpdateType())){
                    exerciseQuestionDto.setAddUser(exerciseQuestionDto.getDealUser());
                    exerciseQuestionDto.setDealUser("");
                }


                if (questionKnows != null){
                    this.setQuestionKnowledge(exerciseQuestionDto, questionKnows.stream().filter(map ->
                            exerciseQuestionDto.getQuestionId().equals(map.getQuestionId())).collect(Collectors.toList()));
                }
                if (hasBigQuestionIndex){
                    this.getBigQuestionClassList(result, exerciseQuestionDto).getList().add(exerciseQuestionDto);
                } else {
                    this.getBigQuestionClassListByParentId(result, exerciseQuestionDto).getList().add(exerciseQuestionDto);
                }
            }
            Collections.reverse(result);
        }
        return  result;
    }


    /**
     * 类型获取名称
     * @param type
     * @return
     */
    @Override
    public String getAuthorityByType (int type){
        String result = "管理员";
        if (type == 9){
            result = "";
        } else {
            switch (type){
                case 1: result = "题目加工人";break;
                case 2: result = "题目审核人";break;
                case 3: result = "解析加工人";break;
                case 4: result = "解析审核人";break;
                case 5: result = "题库迁移人";break;
                case 6: result = "题库审核人";break;
                default: result = "管理员"; break;
            }
        }
        return result;
    }


    private String getUser(ExerciseQuestionRollbacks exerciseQuestionRollbacks,List<KmtSysUserAuthDto> authDtos,int type){
        String user=null;
        String prefix = "AI";
        if(1==type){
            if (StringUtils.isNotBlank(exerciseQuestionRollbacks.getFromType())){
                for (KmtSysUserAuthDto authDto : authDtos){
                    if (exerciseQuestionRollbacks.getFromType().equals(authDto.getType())){
                        prefix = "(" + authDto.getName() + ")";
                    }
                }
            }
            user=prefix + StringUtil.defaultIfEmpty(exerciseQuestionRollbacks.getFromUser(), "");
        }else {
            prefix = "AI";
            if (StringUtils.isNotBlank(exerciseQuestionRollbacks.getToType())){
                for (KmtSysUserAuthDto authDto : authDtos){
                    if (exerciseQuestionRollbacks.getToType().equals(authDto.getType())){
                        prefix = "(" + authDto.getName() + ")";
                    }
                }
            }
            user=prefix + StringUtil.defaultIfEmpty(exerciseQuestionRollbacks.getToUser(), "");
        }
        return user;
    }
    /**
     * 获取大题对象，如果列表中没有就往列表中添加
     * @param result
     * @param exerciseQuestionDto
     * @return
     */
    private ExerciseBookCourseClassDto getBigQuestionClassList (List<ExerciseBookCourseClassDto> result, ExerciseQuetionDto exerciseQuestionDto){
        ExerciseBookCourseClassDto bookCourseClass = this.isBookCourseClassExist(result, exerciseQuestionDto.getCourseId(),
                exerciseQuestionDto.getBigQuestionIndex());
        if (bookCourseClass == null){
            bookCourseClass = new ExerciseBookCourseClassDto(exerciseQuestionDto.getCourseId(), exerciseQuestionDto.getBigQuestionIndex());
            //加入列表的第一个，方便搜索查询， 后续会反转排序
            result.add(0, bookCourseClass);
        }
        return bookCourseClass;
    }

    private ExerciseBookCourseClassDto getBigQuestionClassListByParentId (List<ExerciseBookCourseClassDto> result, ExerciseQuetionDto exerciseQuestionDto){
        ExerciseBookCourseClassDto bookCourseClass = this.isParentIdExist(result, exerciseQuestionDto.getParentId());
        if (bookCourseClass == null){
            bookCourseClass = new ExerciseBookCourseClassDto();
            bookCourseClass.setParentId(exerciseQuestionDto.getParentId());
            //加入列表的第一个，方便搜索查询， 后续会反转排序
            result.add(0, bookCourseClass);
        }
        return bookCourseClass;
    }

    /**
     * 加工的状态包含打回和初始加工，需要都设置未加工中
     * @param exerciseQuestionDto
     */
    private void setQuestionStatus (ExerciseQuetionDto exerciseQuestionDto){
        if (StringUtil.isNotBlank(exerciseQuestionDto.getDealUser())){
            if (exerciseQuestionDto.getQuestionStatus() < 2){
                exerciseQuestionDto.setQuestionStatus(1);
            }
        }
    }

    /**
     * 设置知识点列表
     * @param exerciseQuestionDto
     * @param questionKnowList
     */
    private void setQuestionKnowledge (ExerciseQuetionDto exerciseQuestionDto, List<QuestionListKnowModel> questionKnowList){
        List<KnowHowDetailsDTO> knowHowDTOList = new ArrayList<>();
        if  (CollectionUtils.isNotEmpty(questionKnowList)){
            questionKnowList.forEach(map -> {
                if (map.getHowDetailsId() == null) return;
                boolean exists = false;
                //创建考点对象
                TestPointDTO point = new TestPointDTO(map.getHowDetailsKdId(), map.getKdName());
                //判断已有数据结构中是否
                for (KnowHowDetailsDTO knowHowDetailsDTO : knowHowDTOList) {
                    if (knowHowDetailsDTO.getId().equals(map.getHowDetailsId())) {
                        exists = true;
                        if (StringUtils.isNotBlank(map.getHowDetailsKdId())) {
                            knowHowDetailsDTO.addPoint(point);
                        }
                        break;
                    }
                }
                if (!exists) {
                    KnowHowDetailsDTO knowHowDetailsDTO = new KnowHowDetailsDTO(map.getHowDetailsId(), map.getDetailName());
                    knowHowDetailsDTO.setGrade(map.getGrade());
                    knowHowDetailsDTO.setSemester(map.getSemester());
                    knowHowDetailsDTO.setVersionName(knowledgeVersion.getVersionNameBySubjectAndVersionNum(map.getCourseName(), map.getVersionType()));
                    knowHowDetailsDTO.setHowName(map.getHowName());
                    if (StringUtils.isNotBlank(map.getHowDetailsKdId())) {
                        knowHowDetailsDTO.addPoint(point);
                    }
                    knowHowDTOList.add(knowHowDetailsDTO);
                }


                exerciseQuestionDto.setGrade(map.getGrade());
                exerciseQuestionDto.setVersionType(map.getVersionType());
                exerciseQuestionDto.setSemester(map.getSemester());
                exerciseQuestionDto.setSubject(map.getCourseName());
                exerciseQuestionDto.setModuleType(map.getModuleType() == 1 ? 2 : 1);
            });
            exerciseQuestionDto.setKnowledges(knowHowDTOList);
        }
    }

    /**
     * 从题目列表中提取四个用户的id
     * @param list
     * @return
     */
    private List<String> findUserIdsInExerciseQuestionList (List<ExerciseQuetionDto> list){
        Set<String> result = new HashSet<>();
        if (list != null) {
            list.stream().forEach(item -> {
                if (StringUtils.isNotBlank(item.getHandleUserId())) {
                    result.add(item.getHandleUserId());
                }
                if (StringUtils.isNotBlank(item.getAuditingUserId())) {
                    result.add(item.getAuditingUserId());
                }
                if (StringUtils.isNotBlank(item.getAnalysisUserId())) {
                    result.add(item.getAnalysisUserId());
                }
                if (StringUtils.isNotBlank(item.getAnalysisAuditerId())) {
                    result.add(item.getAnalysisAuditerId());
                }
                if (StringUtils.isNotBlank(item.getFromUserId())) {
                    result.add(item.getFromUserId());
                }
                if (StringUtils.isNotBlank(item.getToUserId()) && !"1".equals(item.getToUserId())) {
                    result.add(item.getToUserId());
                }
            });
        }
        return new ArrayList<>(result);
    }

    /**
     * 把数据库实体题目列表封装成前端对应字段返回
     * @param list
     * @return
     */
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    public List<ExerciseBookCourseClassDto> changeEntityToQuestionList(List<KmtExerciseQuestion> list, KmtExerciseBookCourse bookCourse){
        List<ExerciseBookCourseClassDto> result = new ArrayList<>();
        if (list != null && list.size() > 0) {
            Set<String> dealUserIds = new HashSet<>();
            Set<String> auditingUserIds = new HashSet<>();
            Set<String> analysisHandlerIds = new HashSet<>();
            Set<String> analysisAuditerIds = new HashSet<>();
            for (KmtExerciseQuestion question : list){
                if (question.getHandleUserId() != null){
                    dealUserIds.add(question.getHandleUserId());
                }
                if (question.getAuditingUserId() != null){
                    auditingUserIds.add(question.getAuditingUserId());
                }
                if (question.getAnalysisUserId() != null){
                    analysisHandlerIds.add(question.getAnalysisUserId());
                }
                if (question.getAnalysisAuditerId() != null){
                    analysisAuditerIds.add(question.getAnalysisAuditerId());
                }
            }
            List<String> questionIds = list.stream().map(KmtExerciseQuestion::getId).collect(Collectors.toList());
            List<KmtSysUser> users = kmtSysUserMapper.selectBatchIds(new ArrayList(dealUserIds));
            List<KmtSysUser> auditingUsers = kmtSysUserMapper.selectBatchIds(new ArrayList(auditingUserIds));
            List<KmtSysUser> analysisHandlers = kmtSysUserMapper.selectBatchIds(new ArrayList(analysisHandlerIds));
            List<KmtSysUser> analysisAuditers = kmtSysUserMapper.selectBatchIds(new ArrayList(analysisAuditerIds));

            List<QuestionListKnowModel> questionKnows =  kmtExerciseMapper.findQuestionKnowByQuestionIds(questionIds,null,0);

            list.forEach(kmtExerciseQuestion -> {
                ExerciseBookCourseClassDto bookCourseClass = this.isBookCourseClassExist(result, kmtExerciseQuestion.getBookCourseId(),
                        kmtExerciseQuestion.getBigQuestionIndex());
                if (bookCourseClass == null){
                    bookCourseClass = new ExerciseBookCourseClassDto(kmtExerciseQuestion.getBookCourseId(), kmtExerciseQuestion.getBigQuestionIndex());
                    result.add(0, bookCourseClass);
                }

                ExerciseQuetionDto temp = new ExerciseQuetionDto();
                temp.setQuestionId(kmtExerciseQuestion.getId());
                temp.setBookName(bookCourse.getBookName());
                temp.setCourseName(kmtExerciseQuestion.getCourseName());
                temp.setSmallQuestionIndex(kmtExerciseQuestion.getSmallQuestionName());
                temp.setQuestion(kmtExerciseQuestion.getQuestion());
                temp.setAnswer(kmtExerciseQuestion.getOptionAnswer());
                temp.setAnswerJson(kmtExerciseQuestion.getAnswerJson());
                temp.setQuestionText(kmtExerciseQuestion.getQuestionText());
                temp.setCreateTime(DateUtil.format_yyyyMMddHHmmss(kmtExerciseQuestion.getCreateTime()));
                temp.setQuestionAnalysis(kmtExerciseQuestion.getAnalysis());
                temp.setAnalysisJson(kmtExerciseQuestion.getAnalysisJson());
                temp.setQuestionTypes(kmtExerciseQuestion.getQuestionType() == null ? 0 : kmtExerciseQuestion.getQuestionType());
                temp.setDifficultyLevels(kmtExerciseQuestion.getDifficultyLevel() == null ? 0 : kmtExerciseQuestion.getDifficultyLevel());
                temp.setRealExamTypes(kmtExerciseQuestion.getRealExamType());
                temp.setTrainingSkills(kmtExerciseQuestion.getTrainingSkill());
                temp.setQuestionStatus(kmtExerciseQuestion.getDealStatus());

                if (StringUtils.isNotBlank(kmtExerciseQuestion.getRemark())){
                    temp.setReason(kmtExerciseQuestion.getRemark());
                }

                //用户名称
                if (users != null) {
                    for (KmtSysUser u : users) {
                        if (u.getId().equals(kmtExerciseQuestion.getHandleUserId())){
                            temp.setDealUser(u.getName());
                            break;
                        }
                    }
                }
                //用户名称
                if (auditingUsers != null) {
                    for (KmtSysUser u : auditingUsers) {
                        if (u.getId().equals(kmtExerciseQuestion.getAuditingUserId())){
                            temp.setAuditingUser(u.getName());
                            break;
                        }
                    }
                }

                //用户名称
                if (analysisHandlers != null) {
                    for (KmtSysUser u : analysisHandlers) {
                        if (u.getId().equals(kmtExerciseQuestion.getAnalysisUserId())){
                            temp.setAnalysisDealUser(u.getName());
                            break;
                        }
                    }
                }

                //用户名称
                if (analysisAuditers != null) {
                    for (KmtSysUser u : analysisAuditers) {
                        if (u.getId().equals(kmtExerciseQuestion.getAnalysisAuditerId())){
                            temp.setAnalysisAuditingUser(u.getName());
                            break;
                        }
                    }
                }
                //知识点
                if (questionKnows != null) {
                    List<QuestionListKnowModel> questionKnowList = questionKnows.stream().filter(map ->
                            kmtExerciseQuestion.getId().equals(map.getQuestionId())).collect(Collectors.toList());
                    List<KnowHowDetailsDTO> knowHowDTOList = new ArrayList<>();
                    questionKnowList.forEach(map -> {
                        if (map.getHowDetailsId() == null) return;
                        boolean exists = false;
                        //创建考点对象
                        TestPointDTO point = new TestPointDTO(map.getHowDetailsKdId(), map.getKdName());
                        //判断已有数据结构中是否
                        for (KnowHowDetailsDTO knowHowDetailsDTO : knowHowDTOList){
                            if (knowHowDetailsDTO.getId().equals(map.getHowDetailsId())){
                                exists = true;
                                if (StringUtils.isNotBlank(map.getHowDetailsKdId())) {
                                    knowHowDetailsDTO.addPoint(point);
                                }
                                break;
                            }
                        }
                        if (!exists) {
                            KnowHowDetailsDTO knowHowDetailsDTO  = new KnowHowDetailsDTO(map.getHowDetailsId(), map.getDetailName());
                            if (StringUtils.isNotBlank(map.getHowDetailsKdId())) {
                                knowHowDetailsDTO.addPoint(point);
                            }
                            knowHowDTOList.add(knowHowDetailsDTO);
                        }
                    });
                    temp.setKnowledges(knowHowDTOList);
                }
                bookCourseClass.getList().add(temp);
            });
            Collections.reverse(result);
        }
        return result;
    }

    /**
     * 该课时的大题是否存在
     * @param list
     * @param courseId 课时id
     * @param className  大题名称
     * @return
     */
    private ExerciseBookCourseClassDto isBookCourseClassExist (List<ExerciseBookCourseClassDto> list, String courseId, String className){
        ExerciseBookCourseClassDto result = null;
        for (ExerciseBookCourseClassDto courseClass : list){
            if (courseClass.getCourseId().equals(courseId) && courseClass.getName().equals(className)){
                result = courseClass;
                break;
            }
        }
        return result;
    }

    /**
     * 大题是否存在
     * @param list
     * @param parentId 大题id
     * @return
     */
    private ExerciseBookCourseClassDto isParentIdExist (List<ExerciseBookCourseClassDto> list, String parentId){
        ExerciseBookCourseClassDto result = null;
        for (ExerciseBookCourseClassDto courseClass : list){
            if (courseClass.getParentId().equals(parentId)){
                result = courseClass;
                break;
            }
        }
        return result;
    }


    /**
     * 如果是练习册，获取的单元id为空，如果是测试卷，返回入参的单元id
     * @param bookId
     * @param unitId
     * @return
     */
    private String getAnswerListQueryUnitId(String bookId, String unitId){
        //调用es前的方法，2020年2月19日17:0:19
//    	int unitCount = kmtExerciseQuestionDao.countByBookIdAndPageIndex(bookId, 1);
//        // 一个页码出现在多个单元,说明是试卷，单元筛选生效
//        if (unitCount > 1){
//           return unitId;
//        }
//        return "";

        //-------------以下修改为调用es的方法，2020年2月19日17:0:19----------
        int unitCount = exerciseQuestionSearchService.countByBookIdAndPageIndex(bookId, 1);
        //一个页码出现在多个单元,说明是试卷，单元筛选生效
        if (unitCount > 1){
            return unitId;
        }
        return "";
    }


    @Override
    // TODO: 2020/11/18
//    @Cacheable(value = "key", keyGenerator = "keyGenerator_ExerciseQuestion" , cacheManager="cacheManager")
    public List<ExerciseQuestionDto> findQuestionByBookIdAndPage(String bookId, String unitId, String bookCourseId,
                                                                 List<Integer> pageIndex, int bookYear) {
        //调用es前的方法，2020年2月19日16:58:53
        //unitId = this.getAnswerListQueryUnitId(bookId, unitId);
        //return queryDao.findQuestionByBookIdAndPage(bookId, unitId, bookCourseId, pageIndex, bookYear);



        //-------------以下修改为调用es的方法，2020年2月19日17:0:19----------
        unitId = this.getAnswerListQueryUnitId(bookId, unitId);

        List<ExerciseQuestionDto> exerciseQuestionDtoList=new ArrayList<ExerciseQuestionDto>();
        ExerciseQuestionDto exerciseQuestionDto=null;
        List<Map> list=exerciseQuestionSearchService.findQuestionByBookIdAndPage(bookId, unitId, bookCourseId, pageIndex, bookYear);
        if(list!=null) {
            for (Map map : list) {
                exerciseQuestionDto=new ExerciseQuestionDto();
                exerciseQuestionDto.setQuestionId(map.getOrDefault("id","").toString());
                exerciseQuestionDto.setBigQuestionIndex(map.getOrDefault("bigQuestionIndex","").toString());
                exerciseQuestionDto.setSmallQuestionIndex(map.getOrDefault("smallQuestionName","").toString());
                exerciseQuestionDto.setOptionAnswer(map.getOrDefault("optionAnswer","").toString());
                exerciseQuestionDto.setAnswerJson(map.getOrDefault("answerJson","").toString());
                exerciseQuestionDto.setBookCourseId(map.getOrDefault("bookCourseId","").toString());
                exerciseQuestionDtoList.add(exerciseQuestionDto);
            }
        }
        return exerciseQuestionDtoList;
    }

    @Override
    // TODO: 2020/11/18
//    @Cacheable(value = "key", keyGenerator = "keyGenerator_ExerciseQuestion" , cacheManager="cacheManager")
    public List<ExerciseQuestionDto> findAnswerByExerciseBookCourseId(
            String exerciseBookCourseId, int bookYear) {
        //调用es前的方法，2020年2月19日16:46:55
        //return queryDao.findAnswerByExerciseBookCourseId(exerciseBookCourseId, bookYear) ;


        //-------------以下修改为调用es的方法，2020年2月19日17:0:19----------
        List<ExerciseQuestionDto> exerciseQuestionDtoList=new ArrayList<ExerciseQuestionDto>();
        ExerciseQuestionDto exerciseQuestionDto=null;
        List<Map> list=exerciseQuestionSearchService.findAnswerByExerciseBookCourseId(exerciseBookCourseId, bookYear) ;
        if(list!=null) {
            for (Map map : list) {
                exerciseQuestionDto=new ExerciseQuestionDto();
                exerciseQuestionDto.setQuestionId(map.getOrDefault("id","").toString());
                exerciseQuestionDto.setBigQuestionIndex(map.getOrDefault("bigQuestionIndex","").toString());
                exerciseQuestionDto.setSmallQuestionIndex(map.getOrDefault("smallQuestionName","").toString());
                exerciseQuestionDto.setOptionAnswer(map.getOrDefault("optionAnswer","").toString());
                exerciseQuestionDto.setAnswerJson(map.getOrDefault("answerJson","").toString());
                exerciseQuestionDto.setBookCourseId(map.getOrDefault("bookCourseId","").toString());
                exerciseQuestionDtoList.add(exerciseQuestionDto);
            }
        }
        return exerciseQuestionDtoList;
    }


    @Override
    public List<ExerciseQuestionRollbacks> findQuestionRollbackRecord (String questionId){
        List<KmtExerciseQuestionRollback> list = kmtExerciseMapper.findKmtExerciseQuestionRollbackListByQuestionId(questionId);
        List<ExerciseQuestionRollbacks> list1=new ArrayList<>();
        List<KmtSysUserAuthDto> authDtos = JSONArray.parseArray(PropertiesUtil.getValue("KMT_SYS_USER_AUTH"), KmtSysUserAuthDto.class);
        list.stream().forEach(a->{
            ExerciseQuestionRollbacks exerciseQuestionRollbacks=new ExerciseQuestionRollbacks();
            String fromUser="";
            String toUser="";
            String fromUserId=a.getUserId();
            TbLibManager tbLibManager=kmtExerciseMapper.findByUserIdAndTypeIn(fromUserId);
            if(null==tbLibManager){
                //来向老用户
                fromUser=this.getOldName(fromUserId,authDtos);
            }else {
                //来向新用户
                fromUser=this.getNewName(tbLibManager,1,a);
            }
            if(null==a.getToUserId()){
                //去向为ai
                toUser="AI";
            }else {
                TbLibManager tbLibManager1=kmtExerciseMapper.findByUserIdAndTypeIn(a.getToUserId());
                if(null==tbLibManager1){
                    //去向老用户
                    toUser=this.getOldName(a.getToUserId(),authDtos);
                }else {
                    //去向新用户
                    toUser=this.getNewName(tbLibManager1,2,a);
                }
            }
            exerciseQuestionRollbacks.setFromUser(fromUser);
            exerciseQuestionRollbacks.setToUser(toUser);
            exerciseQuestionRollbacks.setCreateTime(a.getCreateTime());
            exerciseQuestionRollbacks.setReason(a.getReason());
            exerciseQuestionRollbacks.setId(a.getId());
            list1.add(exerciseQuestionRollbacks);
        });
        Collections.reverse(list1);
        return list1;
    }
    //查询老用户
    private  String getOldName(String userId, List<KmtSysUserAuthDto> authDtos){
        String name="AI";
        //查询老用户
        KmtSysUser user = kmtSysUserMapper.selectById(userId);
        //老用户里能查到的情况
        if(null!=user){
            if(StringUtils.isNotBlank(user.getType())){
                for(KmtSysUserAuthDto authDto : authDtos){
                    if (user.getType().equals(authDto.getType())){
                        name = "(" + authDto.getName() + ")"+StringUtil.defaultIfEmpty(user.getName(), "");
                    }
                }
            }

        }


        return name;
    }

    //查询新用户
    private String getNewName(TbLibManager tbLibManager,int type,KmtExerciseQuestionRollback kmt){
        String name="";
        String prefex="";
        if(1==type){
            prefex=this.getName(kmt.getFromUserType());
        }else {
            prefex=this.getName(kmt.getToUserType());
        }
        name=prefex+StringUtil.defaultIfEmpty(tbLibManager.getName(), "");
        return name;

    }

    private String getName(int type){
        Map<Integer,Object> hashMap=new HashMap<>();
        hashMap.put(0,"管理员");
        hashMap.put(1,"练习册加工员");
        hashMap.put(2,"练习册审核员");
        hashMap.put(3,"练习册解析员");
        hashMap.put(4,"练习册解审员");
        hashMap.put(5,"题库迁移员");
        hashMap.put(6,"题库审核员");
        hashMap.put(7,"题库录入员");
        hashMap.put(9,"AI");
        String typeName=hashMap.get(type).toString();
        return typeName;
    }

    @Override
    public List<QuestionAttributeFilterDto> findQuestionAttributeFilter() {
        String key = "exercise_questions_attribute_filter";
        String resultMapStr = redisUtil.get(key);
        List<QuestionAttributeFilterDto> result = new ArrayList<>();
        if (StringUtil.isNotBlank(resultMapStr)){
            return JSONObject.parseArray(resultMapStr, QuestionAttributeFilterDto.class);
        }
        QuestionAttributeFilterDto temp = null;
        for (String subject : CourseConstant.MAIN_SUBJECT_LIST){
            temp = new QuestionAttributeFilterDto();
            temp.setSubject(subject);
            temp.setQuestionTypes(this.getQuestionAttributeBySubject(subject, JSONArray.parseArray(PropertiesUtil.getValue("QUESTION_TYPES"), QuestionSubjectAttribute.class)));
            temp.setTrainingSkills(this.getQuestionAttributeBySubject(subject, JSONArray.parseArray(PropertiesUtil.getValue("TRAINING_SKILL"), QuestionSubjectAttribute.class)));
            temp.setDifficultyLevels(JSONArray.parseArray(PropertiesUtil.getValue("DIFFICULTY_LEVELS"), KnowledgeBaseDto.class));
            temp.setRealExamTypes(JSONArray.parseArray(PropertiesUtil.getValue("REAL_EXAM_TYPE"), KnowledgeBaseDto.class));
            result.add(temp);
        }
        redisUtil.setex(key, JSON.toJSONString(result), 24 * 3600);
        return result;
    }

    /**
     * 查询科目的难题属性筛选项
     * @param subject
     * @param list
     * @return
     */
    private List<KnowledgeBaseDto> getQuestionAttributeBySubject (String subject, List<QuestionSubjectAttribute> list){
        List<KnowledgeBaseDto> result = null;
        for (QuestionSubjectAttribute attribute : list){
            if (subject.equals(attribute.getSubject())){
                result = attribute.getList();
            }
        }
        return result;
    }

    @Override
    public List<ExerciseQuestionDto> findQuestionByQuestionIds(List<String> questionIds) {
        //调用es前的方法，2020年2月19日16:46:55
        //return queryDao.findQuestionByQuestionIds(questionIds);


        //-------------以下修改为调用es的方法，2020年2月19日17:0:19----------
        List<ExerciseQuestionDto> exerciseQuestionDtoList=new ArrayList<ExerciseQuestionDto>();
        ExerciseQuestionDto exerciseQuestionDto=null;
        List<Map> list=exerciseQuestionSearchService.findQuestionByQuestionIds(questionIds);
        if(list!=null) {
            for (Map map : list) {
                exerciseQuestionDto=new ExerciseQuestionDto();
                exerciseQuestionDto.setQuestionId(map.getOrDefault("id","").toString());
                exerciseQuestionDto.setBigQuestionIndex(map.getOrDefault("bigQuestionIndex","").toString());
                exerciseQuestionDto.setSmallQuestionIndex(map.getOrDefault("smallQuestionName","").toString());
                exerciseQuestionDto.setOptionAnswer(map.getOrDefault("optionAnswer","").toString());
                exerciseQuestionDto.setAnswerJson(map.getOrDefault("answerJson","").toString());
                exerciseQuestionDto.setBookCourseId(map.getOrDefault("bookCourseId","").toString());
                exerciseQuestionDtoList.add(exerciseQuestionDto);
            }
        }
        return exerciseQuestionDtoList;
    }


    @Override
    public boolean saveToEs(KmtExerciseQuestion kmtExerciseQuestion) {
        try {
            exerciseQuestionSearchService.save(JSON.parseObject(JSON.toJSONString(kmtExerciseQuestion), new TypeReference<Map<String, Object>>() {}));
            logger.info("保存kmt_exercise_questions到ES：->已完成。");
            return true;
        } catch (Exception e) {
            redisUtil.lpush(Constant.ES_SAVE_ERROR_QUESTIONS, kmtExerciseQuestion.getId());
            logger.error("保存kmt_exercise_questions到ES异常", e);
        }
        return false;
    }

    @Override
    public void bulkKmtExerciseQuestions(List<KmtExerciseQuestion> kmtExerciseQuestions) {
        if (CollectionUtils.isNotEmpty(kmtExerciseQuestions)){
            String[] questionIds = new String[kmtExerciseQuestions.size()];
            try {
                KmtExerciseQuestion exerciseQuestion;
                ElasticEntity elasticEntity;
                List<ElasticEntity> list = new LinkedList<>();
                for (int i = 0; i < kmtExerciseQuestions.size(); i++){
                    exerciseQuestion = kmtExerciseQuestions.get(i);
                    questionIds[i] = exerciseQuestion.getId();
                    elasticEntity = new ElasticEntity();
                    elasticEntity.setId(exerciseQuestion.getId());
                    elasticEntity.setData(JSON.parseObject(JSON.toJSONString(exerciseQuestion), new TypeReference<Map<String, Object>>() {}));
                    list.add(elasticEntity);
                }
                exerciseQuestionSearchService.insertBatch("kmt_exercise_questions", list);
                logger.info("保存kmt_exercise_questions到ES：->已完成。");
            } catch (Exception e) {
                redisUtil.lpush(Constant.ES_SAVE_ERROR_QUESTIONS, questionIds);
                logger.error("保存kmt_exercise_questions到ES异常", e);
            }
        }
    }

    @Override
    public void bulkBookExerciseQuestions(List<String> kmtExerciseBookIds) {
        if (CollectionUtils.isNotEmpty(kmtExerciseBookIds)){
            List<KmtExerciseQuestion> questionList;
            KmtExerciseBook exerciseBook = null;
            for (String exerciseBookId : kmtExerciseBookIds){
                exerciseBook = kmtExerciseBookMapper.selectById(exerciseBookId);
                questionList = kmtExerciseMapper.findExerciseQuestionByExerciseBookId(exerciseBookId);
                kmtExerciseService.appendQuestionDataFromJSONFile(questionList);
                if (CollectionUtils.isNotEmpty(questionList)){
                    for (KmtExerciseQuestion kmtExerciseQuestion : questionList){
                        kmtExerciseQuestion.setHandleUserId(exerciseBook.getHandleUserId());
                        kmtExerciseQuestion.setAuditingUserId(exerciseBook.getAuditingUserId());
                        kmtExerciseQuestion.setAnalysisUserId(exerciseBook.getAnalysisUserId());
                        kmtExerciseQuestion.setAnalysisAuditerId(exerciseBook.getAnalysisAuditerId());
                    }
                    this.bulkKmtExerciseQuestions(questionList);
                }
            }
        }
    }

    @Override
    public void updateEsQuestionHandler(List<String> kmtExerciseBookIds, String teacherId, int type) {
        if (CollectionUtils.isNotEmpty(kmtExerciseBookIds)){
            QueryBuilder queryBuilder = QueryBuilders.termsQuery("exerciseBookId.keyword", kmtExerciseBookIds);
            Script script = new Script(ScriptType.INLINE, "painless",
                    this.getUpdateDSL(teacherId, type),
                    Collections.emptyMap());
            exerciseQuestionSearchService.updteByQuery("kmt_exercise_questions", queryBuilder, script);
        }
    }

    @Override
    public void updateEsQuestionHandlers(List<String> kmtExerciseBookIds, List<ExerciseTeacherClassRequest> teachers) {
        if (CollectionUtils.isNotEmpty(kmtExerciseBookIds) && CollectionUtils.isNotEmpty(teachers)){
            QueryBuilder queryBuilder = QueryBuilders.termsQuery("exerciseBookId.keyword", kmtExerciseBookIds);
            StringBuilder sb = new StringBuilder();
            for (ExerciseTeacherClassRequest request : teachers){
                sb.append(this.getUpdateDSL(request.getTeacherId(), request.getType()));
            }
            Script script = new Script(ScriptType.INLINE, "painless",
                    sb.toString(),
                    Collections.emptyMap());
            exerciseQuestionSearchService.updteByQuery("kmt_exercise_questions", queryBuilder, script);
        }
    }

    /**
     * es更新语句
     * @param teacherId
     * @param type
     * @return
     */
    private String getUpdateDSL (String teacherId, int type){
        StringBuilder sb = new StringBuilder();
        switch (type){
            case 1: sb.append("ctx._source['handleUserId']='").append(teacherId).append("';");break;
            case 2: sb.append("ctx._source['auditingUserId']='").append(teacherId).append("';");break;
            case 3: sb.append("ctx._source['analysisUserId']='").append(teacherId).append("';");break;
            case 4: sb.append("ctx._source['analysisAuditerId']='").append(teacherId).append("';");break;
            case 8: sb.append("ctx._source['answerCorrecterId']='").append(teacherId).append("';"); break;
            default: break;
        }
        return sb.toString();
    }


    @Override
    public void save2EsImmediate(KmtExerciseQuestion kmtExerciseQuestion) {
        try {
            exerciseQuestionSearchService.saveImmediate(JSON.parseObject(JSON.toJSONString(kmtExerciseQuestion), new TypeReference<Map<String, Object>>() {}));
        } catch (Exception e) {
            redisUtil.lpush(Constant.ES_SAVE_ERROR_QUESTIONS, kmtExerciseQuestion.getId());
            logger.error("保存kmt_exercise_questions到ES异常", e);
        }
    }


    @Override
    public void save2EsBatch(List<KmtExerciseQuestion> kmtExerciseQuestionList) {
        if (CollectionUtils.isEmpty(kmtExerciseQuestionList)) {
            return;
        }

        int remainTimes = 5;
        boolean updateResult;

        List<ElasticEntity> elasticEntityList = new ArrayList<>(kmtExerciseQuestionList.size());
        ElasticEntity elasticEntity;
        List<String> kmtExerciseQuestionIdList = new ArrayList<>(kmtExerciseQuestionList.size());
        for (KmtExerciseQuestion kmtExerciseQuestion : kmtExerciseQuestionList) {
            if (StringUtils.isBlank(kmtExerciseQuestion.getId())) {
                continue;
            }
            elasticEntity = new ElasticEntity();
            elasticEntity.setId(kmtExerciseQuestion.getId());
            elasticEntity.setData(JSON.parseObject(JSON.toJSONString(kmtExerciseQuestion), new TypeReference<Map<String, Object>>() {}));
            elasticEntityList.add(elasticEntity);

            kmtExerciseQuestionIdList.add(kmtExerciseQuestion.getId());
        }

        // 更新kmt_exercise_questions索引
        do {
            updateResult = exerciseQuestionSearchService.insertOrUpdateBatch(elasticEntityList);

            remainTimes --;
        } while (remainTimes > 0 && !updateResult);
        if (!updateResult) {
            logger.error("更新练习册题目数据到es失败，共尝试5次");
        }

        // 更新kmt_exercise_question_know索引
        remainTimes = 5;
        List<Map<String, Object>> kmtExerciseQuestionKnowData2EsList = kmtExerciseMapper.findData2EsByQuestionList(kmtExerciseQuestionIdList);
        if (CollectionUtils.isEmpty(kmtExerciseQuestionKnowData2EsList)) {
            return;
        }

        // 获取题目扩展信息
        kmtExerciseService.appendQuestionMapFromJSONFile(kmtExerciseQuestionKnowData2EsList);

        do {
            updateResult = baseSearchService.updateOrInsertBatch("kmt_exercise_question_know",
                    kmtExerciseQuestionKnowData2EsList);
            remainTimes --;
        } while (remainTimes > 0 && !updateResult);
        if (!updateResult) {
            logger.error("更新练习册题目知识点关联数据到es失败，共尝试5次");
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    public KmtExerciseQuestion getEsById(String id) {
        List<KmtExerciseQuestion> list=exerciseQuestionSearchService.getById(id,KmtExerciseQuestion.class);
        if (list == null || list.size()==0) {
            return null;
        }
        try {
            return list.get(0);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public void updateAnswerCorrecterIdByExerciseBookIds(String teacherId, List<String> exerciseBookIds){
        //kmtExerciseQuestionDao.updateAnswerCorrecterIdByExerciseBookIds(teacherId, exerciseBookIds);
        kmtExerciseMapper.updateAnswerCorrecterIdByExerciseBookIds(teacherId, exerciseBookIds);
//        kmtExerciseQuestionMapper.updateAnswerCorrecterIdByExerciseBookIds(teacherId, exerciseBookIds);
    }
}
