package com.aijia.kmt.service.impl;

import com.aijia.core.utils.StringUtil;
import com.aijia.kmt.common.constant.KnowledgeVersion;
import com.aijia.kmt.dto.KnowHowDTO;
import com.aijia.kmt.dto.KnowHowDetailsDTO;
import com.aijia.kmt.dto.TestPointDTO;
import com.aijia.kmt.dto.knowledge.KmtKnowledgeModel;
import com.aijia.kmt.dto.knowledge.KnowledgeGradeDto;
import com.aijia.kmt.enums.GradeEnum;
import com.aijia.kmt.enums.SubjectEnum;
import com.aijia.kmt.mapper.KmtKnowledgeMapper;
import com.aijia.kmt.service.KmtKnowledgeService;
import com.aijia.kmt.utils.RedisUtil;
import com.aijia.kmt.vo.KnowledgeRequest;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class KmtKnowledgeServiceImpl implements KmtKnowledgeService {
    @Autowired
    private KmtKnowledgeMapper kmtKnowledgeMapper;
    @Autowired
    private KnowledgeVersion knowledgeVersion;
    @Autowired
    private RedisUtil redisUtil;

    @Override
    public Map<String, Object> findKnowledgeFilter() {
        String key = "exercise:knowledge:filter";
        String resultMapStr = redisUtil.get(key);
        Map<String, Object> resultMap = null;
        if (StringUtil.isNotBlank(resultMapStr)){
            return JSONObject.parseObject(resultMapStr, Map.class);
        }

        resultMap = new HashMap<>(16);
        List<KnowledgeGradeDto> data = new ArrayList<>();
        data.add(new KnowledgeGradeDto("一年级"));
        data.add(new KnowledgeGradeDto("二年级"));
        data.add(new KnowledgeGradeDto("三年级"));
        data.add(new KnowledgeGradeDto("四年级"));
        data.add(new KnowledgeGradeDto("五年级"));
        data.add(new KnowledgeGradeDto("六年级"));
        resultMap.put("list", data);

        List<Map<String, String>> yearList = new LinkedList<>();
        String[] arrays = {"2016-2017", "2017-2018", "2018-2019", "2019-2020", "2020-2021"};
        for (String item : arrays){
            Map<String, String> yearItem = new HashMap<>();
            yearItem.put("yearName", item);
            yearList.add(yearItem);
        }
        resultMap.put("yearList", yearList);

        String progressListString="[{\"progress\":\"已录入\"},{\"progress\":\"已购买\"},{\"progress\":\"已对接\"},{\"progress\":\"已接收\"},{\"progress\":\"已上后台\"},{\"progress\":\"已发布\"}]";
        JSONArray progressList = JSONObject.parseArray(progressListString);
        resultMap.put("progressList", progressList);
        String semesterListString="[{\"name\":\"上册\",\"type\":1},{\"name\":\"下册\",\"type\":2}]";
        JSONArray semesterList = JSONObject.parseArray(semesterListString);
        resultMap.put("semesterList", semesterList);
        String bookTypeListString="[{\"name\":\"整本\",\"type\":0},{\"name\":\"上半本\",\"type\":1},{\"name\":\"下半本\",\"type\":2},{\"name\":\"不同\",\"type\":3}]";
        JSONArray bookTypeList = JSONObject.parseArray(bookTypeListString);
        resultMap.put("bookTypeList", bookTypeList);
        String bookYearListString="[{\"bookYear\":2019},{\"bookYear\":2020},{\"bookYear\":2021}]";
        JSONArray bookYearList = JSONObject.parseArray(bookYearListString);
        resultMap.put("bookYearList", bookYearList);
        redisUtil.setex(key, JSON.toJSONString(resultMap), 24 * 3600);
        return resultMap;
    }

    @Override
    public List<KnowHowDTO> findSortKnowledgeByCondition(KnowledgeRequest request) {
        StringBuilder sb = new StringBuilder();
        sb.append("knowledge:").append(GradeEnum.getIndexByGrade(request.getGrade())).append(":")
                .append(SubjectEnum.getEnglishByChinese(request.getSubject())).append(":")
                .append(request.getVersionType()).append(":")
                .append(request.getSemester()).append(":")
                .append(request.getModuleType()).append(":")
                .append(request.getSearchKey());
        String key = sb.toString();
        String resultMapStr = redisUtil.get(key);
        List<KnowHowDTO> knowHowDTOList = new ArrayList<>();
        if (StringUtil.isNotBlank(resultMapStr)){
            return JSONObject.parseArray(resultMapStr, KnowHowDTO.class);
        }

        Integer moduleType = null;
        if (request.getModuleType() == 1){
            moduleType = 0;
        } else if (request.getModuleType() == 2){
            moduleType = 1;
        }
        List<KmtKnowledgeModel> mapList = kmtKnowledgeMapper.findKmtKnowledgeByGradeCourseNameTypeSemester(request.getGrade(),
                request.getSubject(), request.getVersionType(), request.getSemester(), moduleType, request.getSearchKey());
        if (CollectionUtils.isNotEmpty(mapList)){
            Map<String, List<KmtKnowledgeModel>> knwHowMap = mapList.stream().collect(Collectors.groupingBy(temp ->
                    String.valueOf(temp.getKnowHowId()), LinkedHashMap::new, Collectors.toList()));
            knwHowMap.forEach((knowHowId, value) -> {
                KnowHowDTO knowHowDTO = new KnowHowDTO();
                knowHowDTO.setId(knowHowId);
                knowHowDTO.setName(String.valueOf(value.get(0).getKnowHowName()));
                LinkedHashMap<String, List<KmtKnowledgeModel>> knowHowDetailsMap = value.stream().collect(
                        Collectors.groupingBy(temp -> String.valueOf(temp.getKnowHowDetailsId()), LinkedHashMap::new,
                                Collectors.toList()));
                List<KnowHowDetailsDTO> knowHowDetailsDTOList = new ArrayList<>(value.size());
                knowHowDetailsMap.forEach((knowHowDetailId, value2) -> {
                    if (knowHowDetailId != null) {
                        KnowHowDetailsDTO knowHowDetailsDTO = new KnowHowDetailsDTO();
                        knowHowDetailsDTO.setId(knowHowDetailId);
                        knowHowDetailsDTO.setName(String.valueOf(value2.get(0).getKnowHowDetailsName()));
                        knowHowDetailsDTO.setGrade(request.getGrade());
                        knowHowDetailsDTO.setSemester(request.getSemester());
                        knowHowDetailsDTO.setVersionName(knowledgeVersion.getVersionNameBySubjectAndVersionNum(request.getSubject(), request.getVersionType()));

                        List<TestPointDTO> testPointDTOList = new ArrayList<>();
                        value2.forEach(temp4kd -> {
                            String howDetailsKdId = temp4kd.getHowDetailsKdId();
                            if (howDetailsKdId != null) {
                                TestPointDTO testPointDTO = new TestPointDTO();
                                testPointDTO.setId(String.valueOf(howDetailsKdId));
                                testPointDTO.setName(String.valueOf(temp4kd.getKnowHowDetailsKdName()));
                                testPointDTOList.add(testPointDTO);
                            }
                        });
                        knowHowDetailsDTO.setPoints(testPointDTOList);
                        knowHowDetailsDTOList.add(knowHowDetailsDTO);
                    }
                });

                knowHowDTO.setDetails(knowHowDetailsDTOList);
                knowHowDTOList.add(knowHowDTO);
            });
            redisUtil.setex(key, JSON.toJSONString(knowHowDTOList), 24 * 3600);
        }
        return knowHowDTOList;
    }


    @Override
    public List<KnowHowDetailsDTO> findDetailsListWithKd (List<String> detailIds){
        List<KmtKnowledgeModel> mapList = kmtKnowledgeMapper.findDetailsListWithKd(detailIds);
        List<KnowHowDetailsDTO> knowHowDetailsDTOList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(mapList)){
            Map<String, List<KmtKnowledgeModel>> listMap = mapList.stream().collect(Collectors.groupingBy(temp ->
                    String.valueOf(temp.getKnowHowDetailsId()), LinkedHashMap::new, Collectors.toList()));
            listMap.forEach((key, value) -> {
                KnowHowDetailsDTO knowHowDetailsDTO = new KnowHowDetailsDTO();
                knowHowDetailsDTO.setId(key);
                knowHowDetailsDTO.setName(String.valueOf(value.get(0).getKnowHowDetailsName()));
                knowHowDetailsDTO.setGrade(value.get(0).getGrade());
                knowHowDetailsDTO.setSemester(value.get(0).getSemester());
                knowHowDetailsDTO.setVersionName(knowledgeVersion.getVersionNameBySubjectAndVersionNum(value.get(0).getCourseName(), value.get(0).getType()));

                List<TestPointDTO> testPointDTOList = new ArrayList<>(value.size());
                value.forEach(map -> {
                    if (StringUtil.isNotBlank((String)map.getHowDetailsKdId())){
                        TestPointDTO testPointDTO = new TestPointDTO();
                        testPointDTO.setId(String.valueOf(map.getHowDetailsKdId()));
                        testPointDTO.setName(String.valueOf(map.getKnowHowDetailsKdName()));
                        testPointDTOList.add(testPointDTO);
                    }
                });

                knowHowDetailsDTO.setPoints(testPointDTOList);
                knowHowDetailsDTOList.add(knowHowDetailsDTO);
            });
        }
        return knowHowDetailsDTOList;
    }

}
