package com.aijia.kmt.task;

import com.aijia.kmt.common.taskthread.AijiaThreadPool;
import com.aijia.kmt.service.KmtExerciseSysService;
import com.aijia.kmt.service.TokenManager;
import com.aijia.kmt.utils.SystemResourceLocator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.TimeUnit;

/**
 * 异步刷新练习册分配导出数据
 *
 * @author yrf
 * @date 2020/6/2 14:42
 */
public class SyncRefreshExerciseBookDistributionCountTask implements Callable<Integer> {

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    private List<String> selectedExerciseBookIdList;
    private int triggerType;

    public SyncRefreshExerciseBookDistributionCountTask(List<String> selectedExerciseBookIdList, int triggerType) {
        super();
        this.selectedExerciseBookIdList = selectedExerciseBookIdList;
        this.triggerType = triggerType;
    }

    @Override
    public Integer call() {
        StringBuilder sb = new StringBuilder();
        sb.append("线程池中正在执行任务的线程数量=").append(AijiaThreadPool.getInstance().getActiveCount())
                .append(";创建过的最大线程数量=").append(AijiaThreadPool.getInstance().getLargestPoolSize())
                .append(";线程池当前的线程数量=").append(AijiaThreadPool.getInstance().getPoolSize());
        logger.info(sb.toString());
        TokenManager tokenManager = null;
        String threadName = "";
        try {
            // 延迟三秒，防止数据的修改还未生效
            TimeUnit.SECONDS.sleep(3);

//            logger.info("异步刷新练习册分配导出数据开始，selectedExerciseBookIdList={}", selectedExerciseBookIdList);

            int addLockTimeOut = 0;
            KmtExerciseSysService kmtExerciseSysService = SystemResourceLocator.getBean(KmtExerciseSysService.class);
            tokenManager = SystemResourceLocator.getBean(TokenManager.class);
            boolean success;
            threadName = Thread.currentThread().getName();
            int count = 0;
            while (addLockTimeOut < 60000) {
                success = tokenManager.addLock("exercise_book_distribution_count_lock", threadName, 60);
                if (success) {
//                    logger.info("异步刷新练习册分配导出数据加锁成功");
                    count = kmtExerciseSysService.refreshExerciseBookDistributionCount(selectedExerciseBookIdList, triggerType);
                    break;
                }

                TimeUnit.MILLISECONDS.sleep(200);
                addLockTimeOut += 200;
            }

//            logger.info("异步刷新练习册分配导出数据完成，更新{}条数据", count);
        } catch (Exception e) {
            logger.error("selectedExerciseBookIdList={}", selectedExerciseBookIdList);
            logger.error("异步刷新练习册分配导出数据异常", e);
        } finally {
            if (tokenManager != null) {
                tokenManager.unLock("exercise_book_distribution_count_lock", threadName);
            }
        }
        return 1;
    }

}
