package com.aijia.kmt.task;

import com.aijia.core.utils.StringUtil;
import com.aijia.kmt.common.taskthread.AijiaThreadPool;
import com.aijia.kmt.es.entiy.ElasticEntity;
import com.aijia.kmt.es.service.ExerciseQuestionSearchService;
import com.aijia.kmt.mapper.KmtExerciseMapper;
import com.aijia.kmt.po.KmtExerciseQuestion;
import com.aijia.kmt.utils.JSONFileUtil;
import com.aijia.kmt.utils.PropertiesUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.stream.Collectors;


public class SyncSaveExerciseBookCourseTask implements Callable<Integer> {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    private String bookCourseId = null;

    private KmtExerciseMapper kmtExerciseMapper;

    private ExerciseQuestionSearchService exerciseQuestionSearchService;

    public SyncSaveExerciseBookCourseTask(KmtExerciseMapper kmtExerciseMapper, ExerciseQuestionSearchService exerciseQuestionSearchService,  String bookCourseId) {
        this.bookCourseId = bookCourseId;
        this.kmtExerciseMapper = kmtExerciseMapper;
        this.exerciseQuestionSearchService = exerciseQuestionSearchService;
    }

    @Override
    public Integer call() {
        try {
            StringBuilder sb = new StringBuilder();
            sb.append("线程池中正在执行任务的线程数量=").append(AijiaThreadPool.getInstance().getActiveCount())
                    .append(";创建过的最大线程数量=").append(AijiaThreadPool.getInstance().getLargestPoolSize())
                    .append(";线程池当前的线程数量=").append(AijiaThreadPool.getInstance().getPoolSize());
            logger.info(sb.toString());
            logger.info("同步课时开始");
            List<KmtExerciseQuestion> kmtExerciseQuestions = kmtExerciseMapper.findByBookCourseId(this.bookCourseId);
            this.appendQuestionDataFromJSONFile(kmtExerciseQuestions);
            for (KmtExerciseQuestion kmtExerciseQuestion : kmtExerciseQuestions){
                exerciseQuestionSearchService.save(JSON.parseObject(JSON.toJSONString(kmtExerciseQuestion), new TypeReference<Map<String, Object>>() {}));
            }
            List<String> questionIds = kmtExerciseQuestions.stream().map(KmtExerciseQuestion::getId).collect(Collectors.toList());
            List<Map<String, Object>> knowEsData = kmtExerciseMapper.findData2EsByQuestionList(questionIds);
            if (CollectionUtils.isNotEmpty(knowEsData)) {
                this.appendQuestionMapFromJSONFile(knowEsData);

                List<ElasticEntity> elasticEntityList = new ArrayList<>(knowEsData.size());
                ElasticEntity elasticEntity;
                String id = null;
                for (Map<String, Object> kes : knowEsData) {
                    id = kes.getOrDefault("id", "").toString();
                    if (StringUtils.isBlank(id)){
                        continue;
                    }
                    elasticEntity = new ElasticEntity();
                    elasticEntity.setId(id);
                    elasticEntity.setData(JSON.parseObject(JSON.toJSONString(kes), new TypeReference<Map<String, Object>>() {}));
                    elasticEntityList.add(elasticEntity);
                }
                exerciseQuestionSearchService.insertOrUpdateBatch("kmt_exercise_question_know", elasticEntityList);
            }
            logger.info("同步课时结束");
        } catch (Exception e){
            logger.error("异步保存es异常", e);
        }
        return  1;
    }

    private void appendQuestionDataFromJSONFile(List<KmtExerciseQuestion> kmtExerciseQuestions) {
        for (KmtExerciseQuestion kmtExerciseQuestion : kmtExerciseQuestions){
            //读取大字段文件
            if (StringUtil.isNotBlank(kmtExerciseQuestion.getQuestionPath())){
                String content = JSONFileUtil.readJSONFile(PropertiesUtil.getValue("EXERCISE_JSON_FILE_PATH") + kmtExerciseQuestion.getQuestionPath());
                if (StringUtils.isNotBlank(content)){
                    JSONObject jsonObject = JSONObject.parseObject(content);
                    kmtExerciseQuestion.setQuestion(jsonObject.getString("question"));
                    kmtExerciseQuestion.setQuestionText(jsonObject.getString("question_text"));
                    kmtExerciseQuestion.setQuestionJson(jsonObject.getString("question_json"));
                    kmtExerciseQuestion.setOptionAnswer(jsonObject.getString("option_answer"));
                    kmtExerciseQuestion.setOptionAnswerText(jsonObject.getString("option_answer_text"));
                    kmtExerciseQuestion.setAnswerJson(jsonObject.getString("answer_json"));
                    kmtExerciseQuestion.setAnalysis(jsonObject.getString("analysis"));
                    kmtExerciseQuestion.setAnalysisJson(jsonObject.getString("analysis_json"));
                }
            }
        }
    }
    private void appendQuestionMapFromJSONFile(List<Map<String, Object>> questions) {
        for (Map<String, Object> map : questions){
            //读取大字段文件
            if (StringUtil.isNotBlank(map.get("question_path") + "")){
                String content = JSONFileUtil.readJSONFile(PropertiesUtil.getValue("EXERCISE_JSON_FILE_PATH") + map.get("question_path"));
                if (StringUtils.isNotBlank(content)){
                    JSONObject jsonObject = JSONObject.parseObject(content);
                    map.put("question",jsonObject.getString("question"));
                    map.put("question_text",jsonObject.getString("question_text"));
                    map.put("question_json",jsonObject.getString("question_json"));
                    map.put("option_answer",jsonObject.getString("option_answer"));
                    map.put("option_answer_text",jsonObject.getString("option_answer_text"));
                    map.put("answer_json",jsonObject.getString("answer_json"));
                    map.put("analysis",jsonObject.getString("analysis"));
                    map.put("analysis_json",jsonObject.getString("analysis_json"));
                }
            }
        }
    }
}
