package com.aijia.kmt.task;

import com.aijia.kmt.common.taskthread.AijiaThreadPool;
import com.aijia.kmt.service.KmtExerciseQuestionService;
import com.aijia.kmt.vo.ExerciseTeacherClassRequest;
import com.alibaba.fastjson.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.concurrent.Callable;

/**
 * 更新练习册题目校对老师数据
 *
 * @author Mount Liu
 * @date 2020/5/12
 */
public class UpdateExerciseQuestionThreadTask implements Callable<Integer> {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    /**
     * 练习册id
     */
    private List<String> exerciseBookIds;

    private KmtExerciseQuestionService kmtExerciseQuestionService;

    /**
     * 分配老师列表, 更新老师类型，1题目加工，2题目审核，3解析加工，4， 解析审核, 8校对
     */
    private List<ExerciseTeacherClassRequest> teachers;

    public UpdateExerciseQuestionThreadTask(List<ExerciseTeacherClassRequest> teachers, List<String> exerciseBookIds, KmtExerciseQuestionService kmtExerciseQuestionService) {
        super();
        this.exerciseBookIds = exerciseBookIds;
        this.kmtExerciseQuestionService = kmtExerciseQuestionService;
        this.teachers = teachers;
    }

    @Override
    public Integer call() {
        StringBuilder sb = new StringBuilder();
        sb.append("线程池中正在执行任务的线程数量=").append(AijiaThreadPool.getInstance().getActiveCount())
                .append(";创建过的最大线程数量=").append(AijiaThreadPool.getInstance().getLargestPoolSize())
                .append(";线程池当前的线程数量=").append(AijiaThreadPool.getInstance().getPoolSize());
        logger.info(sb.toString());
        logger.info("开始分配练习册题目给老师teacherId=[{}], exerciseBookIds=[{}]", JSONObject.toJSONString(teachers), exerciseBookIds);
        try {
//            switch (type){
//                case 1: kmtExerciseQuestionService.updateHandleUserIdByExerciseBookIds(teacherId, exerciseBookIds);break;
//                case 2: kmtExerciseQuestionService.updateAuditingUserIdByExerciseBookIds(teacherId, exerciseBookIds);break;
//                case 3: kmtExerciseQuestionService.updateAnalysisUserIdByExerciseBookIds(teacherId, exerciseBookIds);break;
//                case 4: kmtExerciseQuestionService.updateAnalysisAuditerIdByExerciseBookIds(teacherId, exerciseBookIds);break;
//                case 8: kmtExerciseQuestionService.updateAnswerCorrecterIdByExerciseBookIds(teacherId, exerciseBookIds); break;
//                default: break;
//            }
            kmtExerciseQuestionService.updateEsQuestionHandlers(exerciseBookIds, teachers);

        } catch (Exception e) {
            logger.error("teacherId=[{}], exerciseBookIds=[{}]", JSONObject.toJSONString(teachers), exerciseBookIds);
            logger.error("分配练习册题目给老师异常", e);
        }

        logger.info("[{}]结束分配练习册题目给老师(id=[{}])", Thread.currentThread().getName(), JSONObject.toJSONString(teachers));

        boolean dealTeacherUpdate = false;
        for (ExerciseTeacherClassRequest request : teachers){
            if (request.getType() >= 1 && request.getType() <= 4){
                dealTeacherUpdate = true;
                break;
            }
        }
        //只要有加工审核老师的分配， 则更新练习册数据
        if (dealTeacherUpdate){
            AijiaThreadPool.getInstance().submit(new SyncRefreshExerciseBookStatusTask(exerciseBookIds, 1));
        }

        return 1;
    }
}
