package com.aijia.kmt.utils;

import com.aijia.kmt.dto.exercise.ExerciseQuestionAnswerStyle;
import com.aijia.kmt.dto.exercise.ExerciseQuestionAnswerValue;
import com.aijia.utils.HttpUtils;
import com.aijia.utils.StringUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.http.HttpResponse;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * @author yrf
 * @date 2020/6/30 20:18
 */
public class ExerciseUtil {

    private static final Logger logger = LoggerFactory.getLogger(ExerciseUtil.class);

    private ExerciseUtil() {
    }

    /**
     * 获取图片宽高
     * @param imageUrl
     * @return
     */
    public static Map<String, Integer> getImageWidthAndHeight (String imageUrl){
        Map<String, Integer> result = new HashMap<>();
        try {
//            String prefix = PropertiesUtil.getValue("OSS_INTERNAL_PATH");
            String prefix = "https://img-sharingschool-new.oss-cn-shenzhen-internal.aliyuncs.com";
            HttpResponse response = null;
            try{
                 response = HttpUtils.doGet(prefix, imageUrl ,
                        "Get",
                        Collections.emptyMap(),
                        Collections.singletonMap("x-oss-process", "image/info")
                );
            }catch (Exception e){
                prefix = "https://img-sharingschool-new.oss-cn-shenzhen-internal.aliyuncs.com";
                response = HttpUtils.doGet(prefix, imageUrl ,
                        "Get",
                        Collections.emptyMap(),
                        Collections.singletonMap("x-oss-process", "image/info")
                );
            }

            if(response.getStatusLine().getStatusCode() != 200){
                return null;
            }
            String imageInfo = EntityUtils.toString(response.getEntity());
            if("Error".equals(imageInfo)){
                return null;
            }
            if (StringUtil.isNotBlank(imageInfo)){
                JSONObject jsonObject = JSON.parseObject(imageInfo);
                result.put("height", jsonObject.getJSONObject("ImageHeight").getInteger("value"));
                result.put("width", jsonObject.getJSONObject("ImageWidth").getInteger("value"));
                return result;
            }
            return null;
        } catch (Exception e) {
            logger.error("异常", e);
        }
        return null;
    }


    public static ExerciseQuestionAnswerValue getExerciseQuestionAnswerValue(String imageUrl) {
        try {
            String prefix = PropertiesUtil.getValue("OSS_INTERNAL_PATH");
//            String prefix = "https://xt-sharingschool.oss-cn-shenzhen.aliyuncs.com";
            HttpResponse response = HttpUtils.doGet(prefix, imageUrl ,
                    "Get",
                    Collections.emptyMap(),
                    Collections.singletonMap("x-oss-process", "image/info")
            );
            if(response.getStatusLine().getStatusCode() != 200){
                return null;
            }
            String imageInfo = EntityUtils.toString(response.getEntity());
            if("Error".equals(imageInfo)){
                return null;
            }
            if (StringUtil.isNotBlank(imageInfo)){
                JSONObject jsonObject = JSON.parseObject(imageInfo);
                ExerciseQuestionAnswerValue questionJson = new ExerciseQuestionAnswerValue();
                questionJson.setType(2);
                questionJson.setValue(PropertiesUtil.getValue("STATIC_FILE_URL") + imageUrl);
                ExerciseQuestionAnswerStyle questionStyle  = new ExerciseQuestionAnswerStyle();
                questionStyle.setHeight(jsonObject.getJSONObject("ImageHeight").getInteger("value"));
                questionStyle.setWidth(jsonObject.getJSONObject("ImageWidth").getInteger("value"));
                questionJson.setStyle(questionStyle);

                return questionJson;
            }
            return null;
        } catch (Exception e) {
            logger.error("异常", e);
        }

        return null;
    }

}
