package com.aijia.kmt.utils;

import net.sourceforge.pinyin4j.PinyinHelper;
import net.sourceforge.pinyin4j.format.HanyuPinyinCaseType;
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat;
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType;
import net.sourceforge.pinyin4j.format.exception.BadHanyuPinyinOutputFormatCombination;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

public class SortByChineseUtil {
    private static Logger logger = LoggerFactory.getLogger(SortByChineseUtil.class);

    public static List sort(List list) {

        return sortByChinese(list);
    }

    public static Map sortByKey(Map map) {

        ArrayList list = new ArrayList(map.keySet());
        List<String> sortList = sortByChinese(list);
        LinkedHashMap result = new LinkedHashMap();
        for (String str : sortList) {
            result.put(str, map.get(str));
        }

        return result;
    }


    public static Map sortByValue(Map map) {

        List<String> list = new ArrayList<String>(map.values());
        List<String> sortList = sortByChinese(list);
        LinkedHashMap result = new LinkedHashMap();
        for (String str : sortList) {
            result.put(getKey(map, str), str);
        }
        return result;
    }

    private static String getKey(Map map, String value) {
        for (Object key : map.keySet()) {
            if (map.get(key).equals(value)) {
                return key.toString();
            }
        }
        return null;
    }

    /**
     * 看是否是汉字，是汉字进行匹配
     * 看是否是英文，是英文直接转成小写加进去。
     * 其他则直接加
     * @param list
     * @return
     */

    public static List sortByChinese(List<String> list) {
        TreeMap map = new TreeMap();
        for (String str : list) {
            map.put(ToPinyin(str), str);
        }
        List<String> result = new ArrayList<String>(map.values());
        return result;
    }

    /**
     * 汉字转为拼音
     * @param chinese
     * @return
     */
    public static String ToPinyin(String chinese){
        StringBuilder pinyingStr = new StringBuilder();
        char[] newChar = chinese.toCharArray();
        HanyuPinyinOutputFormat defaultFormat = new HanyuPinyinOutputFormat();
        defaultFormat.setCaseType(HanyuPinyinCaseType.LOWERCASE);
        defaultFormat.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
        for (int i = 0; i < newChar.length; i++) {
            try {
                if (Character.toString(newChar[i]).matches("[\\u4E00-\\u9FA5]+")){
                    pinyingStr.append(PinyinHelper.toHanyuPinyinStringArray(newChar[i], defaultFormat)[0]);
                } else {
                    pinyingStr.append(newChar[i]);
                }
            } catch (BadHanyuPinyinOutputFormatCombination e) {
                logger.error("异常", e);
            }
        }
        return pinyingStr.toString().toLowerCase();
    }

    /**
     *  解决key一样会覆盖的问题
     */
    private static String setMapKey(TreeMap map,String key){
        if(map.get(key) == null){
            return key;
        }else{
            return  SortByChineseUtil.setMapKey(map, key + key.substring(0, 1));
        }

    }

    public static void main(String[] args) {
        String  str1 = SortByChineseUtil.ToPinyin("深圳市小学英语\"第1课堂\"(2021)");
        System.out.println("str=" + str1);
    }
}
