package com.aijia.questioninfo.service.impl;

import com.aijia.core.utils.StringUtil;
import com.aijia.kmt.mapper.KmtExerciseQuestionMapper;
import com.aijia.kmt.po.KmtExerciseQuestion;
import com.aijia.kmt.utils.JSONFileUtil;
import com.aijia.kmt.utils.PropertiesUtil;
import com.aijia.questioninfo.bean.QuestionInfo;
import com.aijia.questioninfo.dto.QuestionDto;
import com.aijia.questioninfo.dto.QuestionLineDto;
import com.aijia.questioninfo.dto.QuestionSpaceDto;
import com.aijia.questioninfo.mapper.KmtExerciseQuestionInfoMapper;
import com.aijia.questioninfo.service.KmtExerciseQuestionInfoService;
import com.aijia.train.bean.TrTrainUserExercise;
import com.aijia.train.mapper.TrainUserExerciseMapper;
import com.aijia.utils.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.collections4.MapUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Map;

@Service
public class KmtExerciseQuestionInfoServiceImpl implements KmtExerciseQuestionInfoService {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private KmtExerciseQuestionInfoMapper kmtExerciseQuestionInfoMapper;
    @Autowired
    private KmtExerciseQuestionMapper kmtExerciseQuestionMapper;

    @Autowired
    private TrainUserExerciseMapper userExerciseMapper;

    @Override
    public QuestionDto getQuestionInfoByQuestionId(String questionId) {
        QuestionDto result = kmtExerciseQuestionInfoMapper.findQuestionByQuestionId(questionId);
        if (result != null){
            String jsonStr = JSONFileUtil.readJSONFile(PropertiesUtil.getValue("EXERCISE_JSON_FILE_PATH") + result.getQuestionPath());
            if (StringUtil.isNotBlank(jsonStr)){
                JSONObject jsonObject = JSONObject.parseObject(jsonStr);
                result.setAnswerJson(JSON.parseArray(jsonObject.getString("answer_json"), QuestionLineDto.class));
                result.setAnalysisJson(JSON.parseArray(jsonObject.getString("analysis_json"), QuestionLineDto.class));
                result.setQuestionJson(JSON.parseArray(jsonObject.getString("question_json"), QuestionSpaceDto.class));
            }
        }
        return result;
    }

    @Override
    public QuestionInfo getQuestionDetailByQuestionId(String questionId) {
        QuestionDto result = kmtExerciseQuestionInfoMapper.findQuestionByQuestionId(questionId);
        QuestionInfo questionInfo = null;
        if (result != null){
            questionInfo = new QuestionInfo();
            questionInfo.setId(questionId);

            String jsonStr = JSONFileUtil.readJSONFile(PropertiesUtil.getValue("EXERCISE_JSON_FILE_PATH") + result.getQuestionPath());
            if (StringUtil.isNotBlank(jsonStr)){
                JSONObject jsonObject = JSONObject.parseObject(jsonStr);
                questionInfo.setQuestion(jsonObject.getString("question"));
                questionInfo.setQuestionText(jsonObject.getString("question_text"));
                questionInfo.setQuestionJson(jsonObject.getString("question_json"));
                questionInfo.setAnswer(jsonObject.getString("option_answer"));
                questionInfo.setAnswerText(jsonObject.getString("option_answer_text"));
                questionInfo.setAnswerJson(jsonObject.getString("answer_json"));
                questionInfo.setAnalysis(jsonObject.getString("analysis"));
                questionInfo.setAnalysisJson(jsonObject.getString("analysis_json"));
            }
        }
        return questionInfo;
    }

    @Override
    public QuestionDto getQuestionInfoFromDbByQuestionId(String questionId) {
        QuestionInfo questionInfo = kmtExerciseQuestionInfoMapper.selectById(questionId);
        QuestionDto result = new QuestionDto();
        if (questionInfo != null){
            result.setQuestionId(questionInfo.getId());
            result.setQuestionJson(JSON.parseArray(questionInfo.getQuestionJson(), QuestionSpaceDto.class));
            result.setAnswerJson(JSON.parseArray(questionInfo.getAnswerJson(), QuestionLineDto.class));
            result.setAnalysisJson(JSON.parseArray(questionInfo.getAnalysisJson(), QuestionLineDto.class));
        }
        return result;
    }

    @Override
    public void updateQuestionInfoByQuestionId(String questionId, Map<String, Object> data) {
        TrTrainUserExercise questionInfo = userExerciseMapper.selectOne(
                new QueryWrapper<TrTrainUserExercise>().eq("question_id",questionId)
                .eq("user_id", MapUtils.getString(data,"userId"))
        );
        if (questionInfo != null){
            questionInfo.setUserId(MapUtils.getString(data,"userId"));
            questionInfo.setExerciseId(MapUtils.getString(data,"bookId"));
            questionInfo.setQuestion((String)data.get("question"));
            questionInfo.setQuestionText((String)data.get("question_text"));
            questionInfo.setQuestionJson((String)data.get("question_json"));
            questionInfo.setAnswer((String)data.get("option_answer"));
            questionInfo.setAnswerJson((String)data.get("answer_json"));
            questionInfo.setAnalysis((String)data.get("analysis"));
            questionInfo.setAnalysisJson((String)data.get("analysis_json"));
            questionInfo.setUpdateTime(DateUtil.format_yyyyMMddHHmmss(new Date()));
            questionInfo.setStatus(1);
            userExerciseMapper.updateById(questionInfo);
        } else {
            questionInfo = new TrTrainUserExercise();
            KmtExerciseQuestion kmtExerciseQuestion = kmtExerciseQuestionMapper.selectById(questionId);
            questionInfo.setUserId(MapUtils.getString(data,"userId"));
            questionInfo.setExerciseId(MapUtils.getString(data,"bookId"));
            questionInfo.setQuestionId(questionId);
            questionInfo.setQuestionOrg(kmtExerciseQuestion.getImage());
            questionInfo.setQuestion((String)data.get("question"));
            questionInfo.setQuestionText((String)data.get("question_text"));
            questionInfo.setQuestionJson((String)data.get("question_json"));
            questionInfo.setAnswer((String)data.get("option_answer"));
            questionInfo.setAnswerJson((String)data.get("answer_json"));
            questionInfo.setAnalysis((String)data.get("analysis"));
            questionInfo.setAnalysisJson((String)data.get("analysis_json"));
            questionInfo.setCreateTime(DateUtil.format_yyyyMMddHHmmss(new Date()));
            questionInfo.setStatus(1);
            userExerciseMapper.insert(questionInfo);
        }
    }

    @Override
    public void updateQuestionAndJsonByQuestionId(String questionId, Map<String, Object> data) {
        TrTrainUserExercise questionInfo = userExerciseMapper.selectOne(
                new QueryWrapper<TrTrainUserExercise>().eq("question_id",questionId)
                        .eq("user_id",MapUtils.getString(data,"userId")));
        if (questionInfo != null){
            questionInfo.setQuestion((String) data.get("question"));
            questionInfo.setQuestionJson(JSON.toJSONString(data.get("question_json")));
            questionInfo.setUpdateTime(DateUtil.format_yyyyMMddHHmmss(new Date()));
            questionInfo.setQuestionPath(data.get("questionPath").toString());
            userExerciseMapper.updateById(questionInfo);
        } else {
            questionInfo = new TrTrainUserExercise();
            questionInfo.setQuestion((String) data.get("question"));
            questionInfo.setQuestionJson(JSON.toJSONString(data.get("question_json")));
            questionInfo.setUpdateTime(DateUtil.format_yyyyMMddHHmmss(new Date()));
            questionInfo.setCreateTime(DateUtil.format_yyyyMMddHHmmss(new Date()));
            questionInfo.setQuestionPath(data.get("questionPath").toString());
            questionInfo.setUserId(MapUtils.getString(data,"userId"));
            questionInfo.setExerciseId(MapUtils.getString(data,"bookId"));
            questionInfo.setQuestionId(questionId);
            questionInfo.setStatus(2);
            userExerciseMapper.insert(questionInfo);
            logger.warn("题目({})不存在{}", questionId, JSON.toJSONString(data));
        }
    }

}
