package com.aijia.train.service.impl;

import com.aijia.core.web.ResponseEntity;
import com.aijia.kmt.service.TokenManager;
import com.aijia.kmt.utils.AliyunSmsUtil;
import com.aijia.kmt.utils.SystemResourceLocator;
import com.aijia.train.bean.TrTrainFiles;
import com.aijia.train.bean.TrTrainPartUser;
import com.aijia.train.bean.TrTrainQuestionCheck;
import com.aijia.train.bean.TrTrainUserInfo;
import com.aijia.train.mapper.*;
import com.aijia.train.service.ITrainService;
import com.aijia.train.vo.FileVO;
import com.aliyuncs.dysmsapi.model.v20170525.SendSmsResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author sh
 * @className TrainServiceImpl
 * @description serviceImpl
 * @date 2021/7/26
 */
@Service
public class TrainServiceImpl implements ITrainService {

    @Autowired
    private TrainUserServiceMapper userServiceMapper;

    @Autowired
    private TokenManager tokenManager;

    @Autowired
    private TrainAllMapper allMapper;

    @Autowired
    private TrainFileServiceMapper fileServiceMapper;

    @Autowired
    private TrainUserInfoServiceMapper userInfoServiceMapper;

    @Autowired
    private TrainUserExerciseMapper userExerciseMapper;

    @Autowired
    private TrainCheckQuestion checkQuestionMapper;

    @Override
    public TrTrainPartUser getTrainUserByMobile(String loginName, int type,String password) {
        TrTrainPartUser user = userServiceMapper.getTrainUserByMobile(loginName);
        if(Objects.isNull(user) && type==0){
            user = new TrTrainPartUser();
            user.setMobile(loginName);
            user.setStatus(1);
            user.setCreateTime(new Date());
            user.setRoleType(0);
            user.setManagerType(type);
            user.setPassword(password);
            userServiceMapper.insert(user);
        }else{
            if(user.getStatus() == 2){
                user = null;
            }
        }
        return user;
    }

    @Override
    public ResponseEntity sendSmsV2(String phoneNumber) {
        ResponseEntity responseEntity = new ResponseEntity();
        try {
            //获取6位随机验证码
            String code = AliyunSmsUtil.getRandNum();
            //所有验证码使用通用验证码：验证码${code}，若非本人操作，请勿泄露。
            SendSmsResponse sm = AliyunSmsUtil.sendSms(phoneNumber, code,"SMS_175541171");
            if("ok".equals(sm.getCode().toLowerCase())){
                //发送成功后 60秒内不重新发送
                tokenManager.setString("trainRepeatCode_".concat(phoneNumber), code, 60);
                //五分钟后失效
                tokenManager.setString("trainCacheCode_".concat(phoneNumber), code, 300);
                responseEntity.success("验证码发送成功");
            }else{
                responseEntity.failure("验证码发送失败，".concat(sm.getMessage()));
            }
        }catch (Exception e){
            responseEntity.failure("验证码发送失败，请稍后重新获取");
        }

        return responseEntity;
    }

    @Override
    public ResponseEntity confirmSms(String phoneNumber, String code) {
        ResponseEntity responseEntity = new ResponseEntity();
        //判断是否是测试环境，如果是测试环境则跳过验证码校验
        String sysEnv = (String) SystemResourceLocator.getValue("SYS_ENV");
        String correctCode;
        if("2".equals(sysEnv)) {
            correctCode = code;
        }else {
            correctCode = tokenManager.getString("trainCacheCode_".concat(phoneNumber));
        }
        if (StringUtils.isBlank(correctCode)) {
            responseEntity.failure("该验证码已经失效，请重新获取");
        } else {
            if (correctCode.equals(code)) {
                responseEntity.success("校验成功");
                //移除缓存
                tokenManager.delString("trainCacheCode_".concat(phoneNumber));
            } else {
                responseEntity.failure("验证码不正确，请重新输入");
            }
        }
        return responseEntity;
    }

    @Override
    public TrTrainPartUser getUserById(String userId) {
        return userServiceMapper.selectById(userId);
    }

    @Override
    public List<FileVO> findTrainFileList(String userId,int roleType) {
        return allMapper.findTrainFileList(userId,roleType);
    }

    @Override
    public ResponseEntity unlockNextFile(String id, Integer roleType, String subject,String userId) {
        ResponseEntity responseEntity = new ResponseEntity();
        TrTrainFiles files = fileServiceMapper.selectById(id);
        //查看本文件是否解锁
        TrTrainUserInfo info = userInfoServiceMapper.selectOne(
                new QueryWrapper<TrTrainUserInfo>().eq("user_id",userId)
                        .eq("file_id",id).eq("status",1)
        );
        if(Objects.isNull(info)){
            return  responseEntity.failure("不能跳过顺序解锁学习进度！");
        }
        //查找下一个文件
        TrTrainFiles nextFiles = fileServiceMapper.findNextFile(files.getFileIndex()+1,roleType,subject);
        if(Objects.isNull(nextFiles)){
            //是最后一个文件，进入到考核阶段。改变用户状态
            TrTrainPartUser user = userServiceMapper.selectById(userId);
            user.setStudyStatus(3);
            userServiceMapper.updateById(user);
        }
        if(Objects.nonNull(nextFiles)){
            saveUserInfoData(nextFiles.getId(),roleType,subject,userId);
        }
        return  responseEntity.success("解锁成功");

    }

    @Override
    public void startStudy(Integer roleType, String subject, String userId) {
        TrTrainFiles firstFiles = fileServiceMapper.findNextFile(1,roleType,subject);
        if(Objects.nonNull(firstFiles)) {
            saveUserInfoData(firstFiles.getId(), roleType, subject, userId);
            TrTrainPartUser user = userServiceMapper.selectById(userId);
            user.setStudyStatus(1);
            userServiceMapper.updateById(user);
        }
    }

    @Override
    public List<Map<String, Object>> partTeacherList(Integer roleType, String subject, Integer type) {
        return allMapper.partTeacherList(roleType,subject,type);
    }

    @Override
    public void studyAgain(String id) {
        /**
         * 更新状态为学习中,学习次数+1
         */
        TrTrainPartUser user = userServiceMapper.selectById(id);
        user.setStudyStatus(1);
        user.setStudyTimes(1);
        userServiceMapper.updateById(user);
        //重新学习，更新学习记录为无效
        userInfoServiceMapper.updateByUserId(id);

        //插入重新学习记录
        this.startStudy(user.getRoleType(),user.getSubject(),user.getId());
    }

    @Override
    public void teacherAudit(Integer type,String userId) {
        TrTrainPartUser user = userServiceMapper.selectById(userId);
        if(type == 1){
            //审核通过
            user.setStudyStatus(5);
            userServiceMapper.updateById(user);
        }else{
            userInfoServiceMapper.updateByUserId(userId);
            userExerciseMapper.updateByUserId(userId);
            //知识点删除
            userExerciseMapper.deleteKnowData(userId);
            //审核不通过
            if(user.getCheckTimes() == 1){
                //两次不通过
                user.setCheckTimes(2);
                user.setStatus(2);
                user.setStudyStatus(6);
            }else{
                //重新学习
                user.setStudyStatus(1);
                user.setCheckTimes(1);
            }
            userServiceMapper.updateById(user);
            if(user.getCheckTimes() == 1){
                this.startStudy(user.getRoleType(),user.getSubject(),user.getId());
            }
            //退出
            String token = tokenManager.getToken(userId,"train");
            tokenManager.removeToken(token,"train");
        }
    }

    @Override
    public ResponseEntity teacherInfoEdit(String userId, String name, Integer roleType, String subject) {
        ResponseEntity responseEntity = new ResponseEntity();
        TrTrainPartUser user = userServiceMapper.selectById(userId);
        if(user.getStudyStatus()>3){
            //考核中
            return  responseEntity.failure("当前处于考核阶段资料料不不可更更改");
        }

        //老用户判断是否有修改资料
        if(!subject.equals(user.getSubject()) || !roleType.equals(user.getRoleType())){
            //清空学习状态，重新学习
            user.setStudyStatus(1);
            user.setSubject(subject);
            user.setRoleType(roleType);
            user.setName(name);
            userServiceMapper.updateById(user);
            //重新学习，更新学习记录为无效
            userInfoServiceMapper.updateByUserId(userId);

            //插入重新学习记录
            this.startStudy(user.getRoleType(),user.getSubject(),user.getId());
        }
        //新用户完善资料
        user.setSubject(subject);
        user.setRoleType(roleType);
        user.setName(name);
        userServiceMapper.updateById(user);
        return responseEntity.success("完善信息成功！");
    }

    @Override
    public ResponseEntity saveCheckQuestion(List<String> questionIds, TrTrainPartUser user) {
        ResponseEntity responseEntity = new ResponseEntity();
        //先失效再保存
        String subject = user.getSubject();
        int roleType = user.getRoleType();
        checkQuestionMapper.updateCheckQuestion(subject,roleType);
        questionIds.forEach(s->{
            TrTrainQuestionCheck check = new TrTrainQuestionCheck();
            check.setQuestionId(s);
            check.setCreateTime(new Date());
            check.setRoleType(roleType);
            check.setSubject(subject);
            check.setStatus(1);
            checkQuestionMapper.insert(check);
        });
        return responseEntity.success("保存成功！");
    }

    private void saveUserInfoData(String id,int roleType,String subject,String userId){
        TrTrainUserInfo info = userInfoServiceMapper.selectOne(
                new QueryWrapper<TrTrainUserInfo>().eq("user_id",userId)
                .eq("file_id",id).eq("status",1)
        );
        if(Objects.isNull(info)){
            info = new TrTrainUserInfo();
            info.setFileId(id);
            info.setCreateTime(new Date());
            info.setRoleType(roleType);
            info.setSubject(subject);
            info.setUserId(userId);
            info.setStatus(1);
            userInfoServiceMapper.insert(info);
        }

    }
}
