package com.aijia.train.task;

import com.aijia.train.bean.TrTrainFiles;
import com.aijia.train.bean.TrTrainPartUser;
import com.aijia.train.bean.TrTrainUserInfo;
import com.aijia.train.mapper.TrainFileServiceMapper;
import com.aijia.train.mapper.TrainUserExerciseMapper;
import com.aijia.train.mapper.TrainUserInfoServiceMapper;
import com.aijia.train.mapper.TrainUserServiceMapper;
import com.aijia.train.service.ITrainService;
import com.aijia.utils.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.collections4.CollectionUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author sh
 * @className ChangeUserStatTask
 * @description 定时器
 * @date 2021/8/17
 */
@Component
public class ChangeUserStatTask {

    @Autowired
    private TrainUserServiceMapper userServiceMapper;
    @Autowired
    private TrainFileServiceMapper fileServiceMapper;

    @Autowired
    private TrainUserInfoServiceMapper userInfoServiceMapper;

    @Autowired
    private TrainUserExerciseMapper userExerciseMapper;

    /**
     * // 每分钟执行一次
     * @throws Exception
     */
    @Scheduled(cron = "0 0/3 * * * ?")
    public void work() throws Exception {
        List<TrTrainPartUser> list = userServiceMapper.selectList(new QueryWrapper<TrTrainPartUser>().eq("status",1));
        list.forEach(u->{
            if(u.getStudyStatus() == 1){
                TrTrainFiles nextFiles = fileServiceMapper.findNextFile(1,u.getRoleType(),u.getSubject());
                if(Objects.nonNull(nextFiles)){
                    TrTrainUserInfo info = userInfoServiceMapper.selectOne(
                            new QueryWrapper<TrTrainUserInfo>().eq("user_id",u.getId())
                                    .eq("file_id",nextFiles.getId()).eq("status",1));
                    if(Objects.nonNull(info)){
                        //相差秒数
                        long time = (System.currentTimeMillis() - info.getCreateTime().getTime()) / 1000L;
                        //相差分钟数
                        long mi = time/60L;
                        long hi = mi/60L;
                        if(hi>=48){
                            u.setStudyStatus(2);
                            if(u.getStudyTimes()==1){
                                u.setStudyTimes(2);
                                u.setStatus(2);
                                userInfoServiceMapper.updateByUserId(u.getId());
                            }else{
                                u.setStudyTimes(1);
                            }
                            userServiceMapper.updateById(u);
                        }
                    }
//                    //相差小时 测试环境注释这个
//                    long hi = mi/60L;
//                    if(hi>48){
//                        u.setStudyStatus(2);
//                        userServiceMapper.updateById(u);
//                    }
                }
            }else if(u.getStudyStatus() == 3 && u.getCheckTimes()<2){
                //看接入考核状态是否超过24小时
                List<TrTrainUserInfo> infoList = userInfoServiceMapper.selectList(
                        new QueryWrapper<TrTrainUserInfo>().eq("user_id",u.getId()).orderByDesc("create_time"));
                if(CollectionUtils.isNotEmpty(infoList)){
                    TrTrainUserInfo info = infoList.get(0);
                    //相差秒数
                    long time = (System.currentTimeMillis() - info.getCreateTime().getTime()) / (1000L);
                    //相差分钟数
                    long mi = time/60L;
//                if(mi>5){
//                    u.setStudyStatus(2);
//                    userServiceMapper.updateById(u);
//                }
//                    //相差小时 测试环境注释这个
                    long hi = mi/60L;
                    if(hi>=48){
                        userInfoServiceMapper.updateByUserId(u.getId());
                        userExerciseMapper.updateByUserId(u.getId());
                        //审核不通过
                        if(u.getCheckTimes() == 1){
                            //两次不通过
                            u.setCheckTimes(2);
                            u.setStatus(2);
                            u.setStudyStatus(6);
                        }else{
                            //重新学习
                            u.setStudyStatus(1);
                            u.setCheckTimes(1);
                        }
                        userServiceMapper.updateById(u);
                        if(u.getCheckTimes() == 1){
                            TrTrainFiles firstFiles = fileServiceMapper.findNextFile(1,u.getRoleType(),u.getSubject());
                            if(Objects.nonNull(firstFiles)) {
                                TrTrainUserInfo info2 = userInfoServiceMapper.selectOne(
                                        new QueryWrapper<TrTrainUserInfo>().eq("user_id",u.getId())
                                                .eq("file_id",firstFiles.getId()).eq("status",1)
                                );
                                if(Objects.isNull(info2)){
                                    info = new TrTrainUserInfo();
                                    info.setFileId(firstFiles.getId());
                                    info.setCreateTime(new Date());
                                    info.setRoleType(u.getRoleType());
                                    info.setSubject(u.getSubject());
                                    info.setUserId(u.getId());
                                    info.setStatus(1);
                                    userInfoServiceMapper.insert(info);
                                }
                            }
                        }
                    }
                }

            }

        });
    }

//    @Scheduled(fixedRate = 5000)//每5秒执行一次
//    public void play() throws Exception {
//        System.out.println("执行Quartz定时器任务："+new Date());
//    }
}
